@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -9,10 +9,16 @@
 #pragma once
 
 #include <algorithm>
+#include <chrono>
 #include <climits>
+#include <cmath>
+#include <folly/Conv.h>
+#include <folly/futures/Future.h>
+#include <folly/io/IOBuf.h>
 #include <functional>
-#include <math.h>
 #include <mutex>
+#include <proxygen/lib/http/HTTPException.h>
+#include <proxygen/lib/http/HTTPMessage.h>
 #include <random>
 #include <vector>
 
@@ -25,33 +31,46 @@
 #include <folly/executors/GlobalExecutor.h>
 #include <folly/io/async/AsyncTimeout.h>
 #include <folly/io/async/EventBaseManager.h>
-#include <proxygen/httpserver/samples/hq/HQParams.h>
-#include <proxygen/httpserver/samples/hq/PartiallyReliableCurlClient.h>
+#include <proxygen/httpserver/samples/hq/HQServer.h>
 #include <proxygen/lib/http/session/HTTPTransaction.h>
 
-namespace quic { namespace samples {
+namespace quic::samples {
+
+/**
+ * The Dispatcher object is responsible for spawning
+ * new request handlers, based on the path.
+ */
+struct HandlerParams {
+  std::string protocol;
+  uint16_t port;
+  std::string httpVersion;
+
+  HandlerParams(std::string pro, uint16_t po, std::string h)
+      : protocol(std::move(pro)), port(po), httpVersion(std::move(h)) {
+  }
+};
+
+class Dispatcher {
+ public:
+  explicit Dispatcher(HandlerParams params) : params_(std::move(params)) {
+  }
+
+  proxygen::HTTPTransactionHandler* getRequestHandler(
+      proxygen::HTTPMessage* /* msg */);
+
+  HandlerParams params_;
+};
 
 using random_bytes_engine =
     std::independent_bits_engine<std::default_random_engine,
                                  CHAR_BIT,
                                  unsigned char>;
 
-constexpr folly::StringPiece kPartiallyReliableScriptHeader{"x-pr-script"};
-constexpr folly::StringPiece kPartiallyReliableChunkDelayMsHeader{
-    "x-pr-chunk-delay-ms"};
-constexpr folly::StringPiece kPartiallyReliableChunkSizeHeader{
-    "x-pr-chunk-size"};
-constexpr folly::StringPiece kPartiallyReliableChunkDelayCapMsHeader{
-    "x-pr-chunk-delay-cap-ms"};
-
-const uint64_t kDefaultPartiallyReliableChunkSize = 16;
-const uint64_t kDefaultPartiallyReliableChunkDelayMs = 0;
-
 class BaseSampleHandler : public proxygen::HTTPTransactionHandler {
  public:
   BaseSampleHandler() = delete;
 
-  explicit BaseSampleHandler(const HQParams& params) : params_(params) {
+  explicit BaseSampleHandler(const HandlerParams& params) : params_(params) {
   }
 
   void setTransaction(proxygen::HTTPTransaction* txn) noexcept override {
@@ -87,8 +106,7 @@ class BaseSampleHandler : public proxygen::HTTPTransactionHandler {
     }
     msg.getHeaders().add(
         proxygen::HTTP_HEADER_ALT_SVC,
-        folly::format("{}=\":{}\"; ma=3600", params_.protocol, params_.port)
-            .str());
+        fmt::format("{}=\":{}\"; ma=3600", params_.protocol, params_.port));
   }
 
   // clang-format off
@@ -129,18 +147,129 @@ class BaseSampleHandler : public proxygen::HTTPTransactionHandler {
     return footer;
   }
 
+  static uint32_t getQueryParamAsNumber(
+      std::unique_ptr<proxygen::HTTPMessage>& msg,
+      const std::string& name,
+      uint32_t defValue) noexcept {
+    return folly::tryTo<uint32_t>(msg->getQueryParam(name)).value_or(defValue);
+  }
+
  protected:
-  const std::string& getHttpVersion() const {
-    return params_.httpVersion.canonical;
+  [[nodiscard]] const std::string& getHttpVersion() const {
+    return params_.httpVersion;
   }
 
   proxygen::HTTPTransaction* txn_{nullptr};
-  const HQParams& params_;
+  const HandlerParams& params_;
+};
+
+/*
+** A handler which returns chunked responses spread over time
+** Generally used to simulat live video downloads where every frame
+** is delivered at 1/30 s (or similar) cadance.
+** Query parameters used:
+**  - keyFrame - size in bytes of the first chunk in the response
+**  - frame - size in bytes of all other chunks
+**  - segment - total time of the response in milliseconds.
+*/
+class ChunkedHandler
+    : public BaseSampleHandler
+    , folly::DelayedDestruction {
+ public:
+  explicit ChunkedHandler(const HandlerParams& params, folly::EventBase* evb)
+      : BaseSampleHandler(params), evb_(evb) {
+  }
+
+  ChunkedHandler() = delete;
+
+  void onHeadersComplete(
+      std::unique_ptr<proxygen::HTTPMessage> msg) noexcept override {
+    VLOG(10) << "ChunkedHandler::onHeadersComplete";
+    proxygen::HTTPMessage resp;
+    resp.setStatusCode(200);
+    resp.setStatusMessage("Ok");
+    resp.setIsChunked(true);
+    maybeAddAltSvcHeader(resp);
+    txn_->sendHeaders(resp);
+    firstFrameSize_ =
+        std::min(getQueryParamAsNumber(msg, "keyFrame", 5000), kMaxFrameSize);
+    otherFrameSize_ =
+        std::min(getQueryParamAsNumber(msg, "frame", 500), kMaxFrameSize);
+    auto segment = std::min(getQueryParamAsNumber(msg, "segment", 2000),
+                            kMaxSegmentLength);
+    totalChunkCount_ = segment / frameDelay_.count();
+  }
+  void onBody(std::unique_ptr<folly::IOBuf> chain) noexcept override {
+  }
+
+  void onEOM() noexcept override {
+    VLOG(10) << "ChunkedHandler::onEOM";
+    sleepFutureCallback();
+  }
+
+  void onError(const proxygen::HTTPException& /*error*/) noexcept override {
+    txn_->sendAbort();
+    failed_ = true;
+  }
+
+ private:
+  ~ChunkedHandler() override = default;
+
+  std::unique_ptr<folly::IOBuf> genRandBytes(uint32_t len) {
+    auto buffer = folly::IOBuf::create(len);
+    buffer->append(len);
+    std::generate(
+        buffer->writableData(), buffer->writableData() + len, std::ref(rbe_));
+    return buffer;
+  }
+
+  void sendChunkRandomData(uint32_t chunkSize) {
+    auto data = genRandBytes(chunkSize);
+    txn_->sendChunkHeader(chunkSize);
+    if (!failed_) {
+      txn_->sendBody(std::move(data));
+    }
+    if (!failed_) {
+      txn_->sendChunkTerminator();
+    }
+  }
+
+  void sleepFutureCallback() {
+    DestructorGuard destructorGuard(this);
+    uint32_t chunkSize = chunk_ == 0 ? firstFrameSize_ : otherFrameSize_;
+    if (failed_) {
+      return;
+    }
+
+    chunk_++;
+    if (chunk_ > totalChunkCount_) {
+      txn_->sendEOM();
+      return;
+    }
+    sendChunkRandomData(chunkSize);
+    sleepFuture_ =
+        folly::futures::sleep(frameDelay_).via(evb_).then([this](auto&&) {
+          sleepFutureCallback();
+        });
+  }
+
+  const uint32_t kMaxFrameSize{1000000};
+  const uint32_t kMaxSegmentLength{60000};
+  uint32_t firstFrameSize_{5000};
+  uint32_t otherFrameSize_{500};
+  std::chrono::milliseconds frameDelay_{33};
+  uint32_t totalChunkCount_{60};
+  uint32_t chunk_{0};
+  folly::EventBase* evb_;
+  folly::SemiFuture<folly::Unit> sleepFuture_;
+  bool failed_{false};
+  random_bytes_engine rbe_;
 };
 
 class EchoHandler : public BaseSampleHandler {
  public:
-  explicit EchoHandler(const HQParams& params) : BaseSampleHandler(params) {
+  explicit EchoHandler(const HandlerParams& params)
+      : BaseSampleHandler(params) {
   }
 
   EchoHandler() = delete;
@@ -219,318 +348,9 @@ class TransportCallbackBase
   }
 };
 
-/**
- * Streams ASCII cat in partially realible mode.
- */
-class PrCatHandler
-    : public EchoHandler
-    , public TransportCallbackBase
-    , public folly::AsyncTimeout {
- public:
-  explicit PrCatHandler(const HQParams& params)
-      : EchoHandler(params),
-        chunkSize_(params.prChunkSize),
-        chunkDelayMs_(params.prChunkDelayMs) {
-  }
-
-  PrCatHandler() = delete;
-
-  void setPrParams(folly::Optional<uint64_t> prChunkSize,
-                   folly::Optional<uint64_t> prChunkDelayMs) {
-    chunkSize_ = prChunkSize;
-    chunkDelayMs_ = prChunkDelayMs;
-  }
-
-  // TransportCallbackBase
-  void lastEgressHeaderByteAcked() noexcept override {
-    attachEventBase(folly::EventBaseManager::get()->getEventBase());
-    timeoutExpired();
-  }
-
-  void onEOM() noexcept override {
-  }
-
-  void onHeadersComplete(
-      std::unique_ptr<proxygen::HTTPMessage> /* msg */) noexcept override {
-    VLOG(10) << "PrCatHandler::onHeadersComplete";
-    proxygen::HTTPMessage resp;
-    VLOG(10) << "Setting http-version to " << getHttpVersion();
-    resp.setVersionString(getHttpVersion());
-    resp.setStatusCode(200);
-    resp.setStatusMessage("Ok");
-    resp.getHeaders().add("pr-chunk-size", folly::to<std::string>(*chunkSize_));
-    resp.setWantsKeepalive(true);
-    resp.setPartiallyReliable();
-    txn_->setTransportCallback(this);
-    txn_->sendHeaders(resp);
-  }
-
-  void onBodyPeek(uint64_t offset,
-                  const folly::IOBuf& chain) noexcept override {
-    LOG(INFO) << __func__ << ": got " << chain.computeChainDataLength()
-              << " bytes at offset " << offset;
-  }
-
-  void onBodySkipped(uint64_t /* newOffset */) noexcept override {
-    LOG(FATAL) << __func__ << ": wrong side to receive this callback ";
-  }
-
-  void onBodyRejected(uint64_t offset) noexcept override {
-    cancelTimeout();
-    LOG(INFO) << __func__ << ": data for chunk " << curPartNum_
-              << " (offset = " << offset << ") cancelled ( was scheduled with "
-              << delayMs_ << "ms delay)";
-
-    auto chunk = dataSndr_.generateChunk();
-    CHECK(chunk);
-    bool eom = !dataSndr_.hasMoreData();
-
-    if (eom) {
-      LOG(INFO) << __func__ << ":    sending EOM after last chunk "
-                << curPartNum_;
-      txn_->sendEOM();
-    } else {
-      curPartNum_ = offset / *chunkSize_;
-      delayMs_ = folly::Random::rand64(*chunkDelayMs_);
-      scheduleTimeout(delayMs_);
-    }
-  }
-
-  void timeoutExpired() noexcept override {
-    LOG(INFO) << ": sending body part " << curPartNum_ << " with delay "
-              << delayMs_ << " ms";
-    auto chunk = dataSndr_.generateChunk();
-    CHECK(chunk);
-    bool eom = !dataSndr_.hasMoreData();
-
-    txn_->sendBody(std::move(*chunk));
-    if (eom) {
-      LOG(INFO) << __func__ << ":    sending EOM with chunk " << curPartNum_;
-      txn_->sendEOM();
-    } else {
-      delayMs_ = folly::Random::rand64(*chunkDelayMs_);
-      scheduleTimeout(delayMs_);
-    }
-
-    curPartNum_++;
-  }
-
- private:
-  uint64_t delayMs_{0};
-  uint64_t curPartNum_{0};
-  folly::Optional<uint64_t> chunkSize_{kDefaultPartiallyReliableChunkSize};
-  folly::Optional<uint64_t> chunkDelayMs_{
-      kDefaultPartiallyReliableChunkDelayMs};
-  PartiallyReliableSender dataSndr_{kDefaultPartiallyReliableChunkSize};
-};
-
-/**
- * Sends body/skip sequence according to the script received in client headers.
- */
-class PrSkipHandler
-    : public EchoHandler
-    , public folly::AsyncTimeout {
- public:
-  explicit PrSkipHandler(const HQParams& params) : EchoHandler(params) {
-  }
-
-  PrSkipHandler() = delete;
-
-  void onEOM() noexcept override {
-  }
-
-  void onHeadersComplete(
-      std::unique_ptr<proxygen::HTTPMessage> msg) noexcept override {
-
-    msg->getHeaders().forEach(
-        [&](const std::string& header, const std::string& val) {
-          if (header == kPartiallyReliableScriptHeader) {
-            script = val;
-          } else if (header == kPartiallyReliableChunkDelayMsHeader) {
-            auto res = folly::tryTo<uint64_t>(val);
-            if (res.hasError()) {
-              LOG(ERROR) << __func__ << ": failed to convert " << header << " '"
-                         << val << "' to uint64_t";
-              txn_->sendAbort();
-            }
-            chunkDelayMs_ = *res;
-          }
-        });
-
-    proxygen::HTTPMessage resp;
-    resp.setVersionString(getHttpVersion());
-    resp.setStatusCode(200);
-    resp.setStatusMessage("Ok");
-    resp.setWantsKeepalive(true);
-    resp.setPartiallyReliable();
-    txn_->sendHeaders(resp);
-
-    if (script.length() == 0) {
-      txn_->sendEOM();
-    }
-  }
-
-  void onUnframedBodyStarted(uint64_t /* offset */) noexcept override {
-  }
-
-  void onBody(std::unique_ptr<folly::IOBuf> chain) noexcept override {
-    chunkSize_ = chain->computeChainDataLength();
-    if (chunkSize_ == 0) {
-      txn_->sendEOM();
-      return;
-    }
-
-    chunkData_ = std::move(chain);
-
-    attachEventBase(folly::EventBaseManager::get()->getEventBase());
-    timeoutExpired();
-  }
-
-  void timeoutExpired() noexcept override {
-    bool eom = (curPartNum_ == script.length() - 1);
-
-    if (script[curPartNum_] == 'b') {
-      LOG(INFO) << ": sending body part " << curPartNum_ << " with delay "
-                << delayMs_ << " ms";
-      txn_->sendBody(chunkData_->clone());
-    } else {
-      uint64_t nextOffset =
-          chunkData_->computeChainDataLength() * (curPartNum_ + 2);
-      auto res = txn_->skipBodyTo(nextOffset);
-      if (res.hasError()) {
-        LOG(ERROR) << __func__ << ": failed to skip body to offset "
-                   << nextOffset << ": " << getErrorCodeString(res.error());
-        txn_->sendAbort();
-        return;
-      } else {
-        LOG(INFO) << ": skipping body part " << curPartNum_ << " with delay "
-                  << delayMs_ << " ms";
-      }
-    }
-
-    if (eom) {
-      LOG(INFO) << __func__ << ":    sending EOM with chunk " << curPartNum_;
-      txn_->sendEOM();
-    } else {
-      delayMs_ = folly::Random::rand64(chunkDelayMs_);
-      scheduleTimeout(delayMs_);
-    }
-
-    curPartNum_++;
-  }
-
- private:
-  uint64_t delayMs_{0};
-  uint64_t curPartNum_{0};
-
-  std::unique_ptr<folly::IOBuf> chunkData_{nullptr};
-  std::string script;
-  uint64_t chunkSize_{0};
-  uint64_t chunkDelayMs_{0};
-};
-
-/**
- * Sends a body chunk, then waits for a reject from the client.
- * Once reject arrives, sends another body chunk.
- * Number of pieces/skips to send determined by the script received in client
- * headers.
- */
-class PrRejectHandler
-    : public EchoHandler
-    , public folly::AsyncTimeout {
- public:
-  explicit PrRejectHandler(const HQParams& params) : EchoHandler(params) {
-  }
-
-  PrRejectHandler() = delete;
-
-  void onEOM() noexcept override {
-  }
-
-  void onHeadersComplete(
-      std::unique_ptr<proxygen::HTTPMessage> msg) noexcept override {
-
-    msg->getHeaders().forEach(
-        [&](const std::string& header, const std::string& val) {
-          if (header == kPartiallyReliableScriptHeader) {
-            script = val;
-          } else if (header == kPartiallyReliableChunkSizeHeader) {
-            auto res = folly::tryTo<uint64_t>(val);
-            if (res.hasError()) {
-              LOG(ERROR) << __func__ << ": failed to convert " << header << " '"
-                         << val << "' to uint64_t";
-              txn_->sendAbort();
-              return;
-            }
-            chunkSize_ = *res;
-          } else if (header == kPartiallyReliableChunkDelayCapMsHeader) {
-            auto res = folly::tryTo<uint64_t>(val);
-            if (res.hasError()) {
-              LOG(ERROR) << __func__ << ": failed to convert " << header << " '"
-                         << val << "' to uint64_t";
-              txn_->sendAbort();
-              return;
-            }
-            clientDelayCapMs_ = *res;
-          }
-        });
-
-    proxygen::HTTPMessage resp;
-    resp.setVersionString(getHttpVersion());
-    resp.setStatusCode(200);
-    resp.setStatusMessage("Ok");
-    resp.setWantsKeepalive(true);
-    resp.setPartiallyReliable();
-    txn_->sendHeaders(resp);
-
-    if (chunkSize_ == 0 || script.length() == 0) {
-      txn_->sendEOM();
-      return;
-    }
-
-    attachEventBase(folly::EventBaseManager::get()->getEventBase());
-    sendScriptedBody();
-  }
-
-  void sendScriptedBody() {
-    if (script.length() == 0) {
-      txn_->sendEOM();
-      return;
-    }
-
-    auto curStep = script[0];
-    script.erase(0, 1);
-
-    if (curStep == 'b') {
-      txn_->sendBody(folly::IOBuf::copyBuffer(std::string(chunkSize_, 'b')));
-      if (script.length() == 0) {
-        txn_->sendEOM();
-      } else {
-        scheduleTimeout(clientDelayCapMs_ / 3);
-      }
-    } else {
-      scheduleTimeout(clientDelayCapMs_ * 2);
-    }
-  }
-
-  void onBodyRejected(uint64_t /* offset */) noexcept override {
-    cancelTimeout();
-    sendScriptedBody();
-  }
-
-  void timeoutExpired() noexcept override {
-    sendScriptedBody();
-  }
-
- private:
-  std::string script;
-  uint64_t chunkSize_{0};
-  uint64_t clientDelayCapMs_{0};
-};
-
 class ContinueHandler : public EchoHandler {
  public:
-  explicit ContinueHandler(const HQParams& params) : EchoHandler(params) {
+  explicit ContinueHandler(const HandlerParams& params) : EchoHandler(params) {
   }
 
   ContinueHandler() = delete;
@@ -554,7 +374,7 @@ class ContinueHandler : public EchoHandler {
 
 class RandBytesGenHandler : public BaseSampleHandler {
  public:
-  explicit RandBytesGenHandler(const HQParams& params)
+  explicit RandBytesGenHandler(const HandlerParams& params)
       : BaseSampleHandler(params) {
   }
 
@@ -685,7 +505,8 @@ class RandBytesGenHandler : public BaseSampleHandler {
 
 class DummyHandler : public BaseSampleHandler {
  public:
-  explicit DummyHandler(const HQParams& params) : BaseSampleHandler(params) {
+  explicit DummyHandler(const HandlerParams& params)
+      : BaseSampleHandler(params) {
   }
 
   DummyHandler() = delete;
@@ -728,9 +549,59 @@ class DummyHandler : public BaseSampleHandler {
                              "response with random bytes");
 };
 
+class DelayHandler
+    : public BaseSampleHandler
+    , private folly::AsyncTimeout {
+ public:
+  explicit DelayHandler(const HandlerParams& params, folly::EventBase* evb)
+      : BaseSampleHandler(params), AsyncTimeout(evb) {
+  }
+
+  DelayHandler() = delete;
+
+  void onHeadersComplete(
+      std::unique_ptr<proxygen::HTTPMessage> msg) noexcept override {
+    VLOG(10) << "DelayHandler::onHeadersComplete";
+    proxygen::HTTPMessage resp;
+    VLOG(10) << "Setting http-version to " << getHttpVersion();
+    resp.setVersionString(getHttpVersion());
+    resp.setStatusCode(200);
+    resp.setStatusMessage("Ok");
+    resp.setWantsKeepalive(true);
+    maybeAddAltSvcHeader(resp);
+    VLOG(10) << "DelayHandler::onHeadersComplete calling sendHeaders";
+    txn_->sendHeaders(resp);
+
+    auto duration = getQueryParamAsNumber(msg, "duration", 0);
+    responseBody_ = fmt::format(
+        "Response Body for: {} {}", msg->getMethodString(), msg->getURL());
+    scheduleTimeout(std::chrono::milliseconds(duration));
+  }
+
+  void onBody(std::unique_ptr<folly::IOBuf> /*chain*/) noexcept override {
+    VLOG(10) << "DelayHandler::onBody";
+  }
+
+  void onEOM() noexcept override {
+    VLOG(10) << "DelayHandler::onEOM";
+  }
+
+  void onError(const proxygen::HTTPException& /*error*/) noexcept override {
+    cancelTimeout();
+  }
+
+ private:
+  void timeoutExpired() noexcept override {
+    txn_->sendBody(folly::IOBuf::copyBuffer(responseBody_));
+    txn_->sendEOM();
+  }
+
+  std::string responseBody_;
+};
+
 class HealthCheckHandler : public BaseSampleHandler {
  public:
-  HealthCheckHandler(bool healthy, const HQParams& params)
+  HealthCheckHandler(bool healthy, const HandlerParams& params)
       : BaseSampleHandler(params), healthy_(healthy) {
   }
 
@@ -775,7 +646,7 @@ class HealthCheckHandler : public BaseSampleHandler {
 
 class WaitReleaseHandler : public BaseSampleHandler {
  public:
-  WaitReleaseHandler(folly::EventBase* evb, const HQParams& params)
+  WaitReleaseHandler(folly::EventBase* evb, const HandlerParams& params)
       : BaseSampleHandler(params), evb_(evb) {
   }
 
@@ -867,7 +738,7 @@ class ServerPushHandler : public BaseSampleHandler {
   };
 
  public:
-  explicit ServerPushHandler(const HQParams& params)
+  explicit ServerPushHandler(const HandlerParams& params)
       : BaseSampleHandler(params) {
   }
 
@@ -902,8 +773,8 @@ class ServerPushHandler : public BaseSampleHandler {
 
 class StaticFileHandler : public BaseSampleHandler {
  public:
-  explicit StaticFileHandler(const HQParams& params)
-      : BaseSampleHandler(params), staticRoot_(params.staticRoot) {
+  StaticFileHandler(const HandlerParams& params, std::string staticRoot)
+      : BaseSampleHandler(params), staticRoot_(std::move(staticRoot)) {
   }
 
   void onHeadersComplete(
@@ -936,7 +807,7 @@ class StaticFileHandler : public BaseSampleHandler {
     maybeAddAltSvcHeader(resp);
     txn_->sendHeaders(resp);
     // use a CPU executor since read(2) of a file can block
-    folly::getCPUExecutor()->add(
+    folly::getUnsafeMutableGlobalCPUExecutor()->add(
         std::bind(&StaticFileHandler::readFile,
                   this,
                   folly::EventBaseManager::get()->getEventBase()));
@@ -961,7 +832,7 @@ class StaticFileHandler : public BaseSampleHandler {
   void onEgressResumed() noexcept override {
     VLOG(10) << "StaticFileHandler::onEgressResumed";
     paused_ = false;
-    folly::getCPUExecutor()->add(
+    folly::getUnsafeMutableGlobalCPUExecutor()->add(
         std::bind(&StaticFileHandler::readFile,
                   this,
                   folly::EventBaseManager::get()->getEventBase()));
@@ -1011,7 +882,7 @@ class StaticFileHandler : public BaseSampleHandler {
 
   std::unique_ptr<folly::File> file_;
   std::atomic<bool> paused_{false};
-  const std::string staticRoot_;
+  std::string staticRoot_;
 };
 
-}} // namespace quic::samples
+} // namespace quic::samples
