@@ -1,26 +1,33 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
  * LICENSE file in the root directory of this source tree.
  */
 
-#include <proxygen/httpserver/HTTPServer.h>
+#include <memory>
+
 #include <boost/thread.hpp>
 #include <folly/FileUtil.h>
+#include <folly/executors/IOThreadPoolExecutor.h>
 #include <folly/experimental/TestUtil.h>
 #include <folly/io/async/AsyncSSLSocket.h>
 #include <folly/io/async/AsyncServerSocket.h>
 #include <folly/io/async/EventBaseManager.h>
+#include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 #include <folly/ssl/OpenSSLCertUtils.h>
 #include <folly/ssl/OpenSSLPtrTypes.h>
 #include <proxygen/httpclient/samples/curl/CurlClient.h>
+#include <proxygen/httpserver/HTTPServer.h>
 #include <proxygen/httpserver/ResponseBuilder.h>
 #include <proxygen/httpserver/ScopedHTTPServer.h>
 #include <proxygen/lib/http/HTTPConnector.h>
 #include <proxygen/lib/utils/TestUtils.h>
+#include <wangle/acceptor/Acceptor.h>
+#include <wangle/acceptor/ServerSocketConfig.h>
+#include <wangle/client/ssl/SSLSessionCallbacks.h>
 
 using namespace folly;
 using namespace folly::ssl;
@@ -100,7 +107,9 @@ class WaitableServerThread {
     t_.join();
   }
 
-  bool start() {
+  bool start(
+      std::shared_ptr<wangle::AcceptorFactory> acceptorFactory = nullptr,
+      std::shared_ptr<folly::IOThreadPoolExecutor> ioExecutor = nullptr) {
     bool throws = false;
     t_ = std::thread([&]() {
       server_->start([&]() { barrier_.wait(); },
@@ -108,7 +117,9 @@ class WaitableServerThread {
                        throws = true;
                        server_ = nullptr;
                        barrier_.wait();
-                     });
+                     },
+                     acceptorFactory,
+                     ioExecutor);
       baton_.wait();
     });
     barrier_.wait();
@@ -168,6 +179,76 @@ TEST(HttpServerStartStop, TestRepeatStopCalls) {
   st->exitThread();
 }
 
+TEST(HttpServerStartStop, TestUseExistingIoExecutor) {
+  HTTPServerOptions options;
+  auto server = std::make_unique<HTTPServer>(std::move(options));
+  auto ioExecutor = std::make_shared<folly::IOThreadPoolExecutor>(1);
+  auto st = std::make_unique<WaitableServerThread>(server.get());
+  EXPECT_TRUE(
+      st->start(/*acceptorFactory=*/nullptr, /*ioExecutor=*/ioExecutor));
+
+  server->stop();
+  // Let the WaitableServerThread exit
+  st->exitThread();
+}
+
+class MockRequestHandlerFactory : public RequestHandlerFactory {
+ public:
+  MOCK_METHOD((void), onServerStart, (folly::EventBase*), (noexcept));
+  MOCK_METHOD((void), onServerStop, (), (noexcept));
+  MOCK_METHOD((RequestHandler*),
+              onRequest,
+              (RequestHandler*, HTTPMessage*),
+              (noexcept));
+};
+
+TEST(HttpServerStartStop, TestZeroThreadsMeansNumCPUs) {
+  // Create a handler factory whose callback will
+  // be called for each worker thread.
+  std::unique_ptr<MockRequestHandlerFactory> handlerFactory =
+      std::make_unique<MockRequestHandlerFactory>();
+  MockRequestHandlerFactory* rawHandlerFactory = handlerFactory.get();
+
+  HTTPServerOptions options;
+  options.threads = 0;
+  options.handlerFactories.push_back(std::move(handlerFactory));
+
+  // threads = 0 should start num of CPUs threads
+  // each calling the handlerFactory onServerStart()
+  EXPECT_CALL(*rawHandlerFactory, onServerStart(testing::_))
+      .Times(std::thread::hardware_concurrency());
+
+  auto server = std::make_unique<HTTPServer>(std::move(options));
+  auto st = std::make_unique<WaitableServerThread>(server.get());
+  EXPECT_TRUE(st->start());
+
+  server->stop();
+  // Let the WaitableServerThread exit
+  st->exitThread();
+}
+
+class AcceptorFactoryForTest : public wangle::AcceptorFactory {
+ public:
+  std::shared_ptr<wangle::Acceptor> newAcceptor(
+      folly::EventBase* /*eventBase*/) override {
+    wangle::ServerSocketConfig config;
+    auto acc = std::make_shared<wangle::Acceptor>(config);
+    return acc;
+  }
+};
+
+TEST(HttpServerStartStop, TestUseExistingAcceptorFactory) {
+  HTTPServerOptions options;
+  auto server = std::make_unique<HTTPServer>(std::move(options));
+  auto acceptorFactory = std::make_shared<AcceptorFactoryForTest>();
+  auto st = std::make_unique<WaitableServerThread>(server.get());
+  EXPECT_TRUE(st->start(acceptorFactory));
+
+  server->stop();
+  // Let the WaitableServerThread exit
+  st->exitThread();
+}
+
 // Make an SSL connection to the server
 class Cb : public folly::AsyncSocket::ConnectCallback {
  public:
@@ -175,14 +256,10 @@ class Cb : public folly::AsyncSocket::ConnectCallback {
   }
   void connectSuccess() noexcept override {
     success = true;
-    reusedSession = sock_->getSSLSessionReused();
-    session = sock_->getSSLSession();
-    if (sock_->getPeerCertificate()) {
+    if (auto cert = sock_->getPeerCertificate()) {
       // keeps this alive until Cb is destroyed, even if sock is closed
-      auto cert = sock_->getPeerCertificate();
-      peerCert_ = cert->getX509();
+      peerCert_ = folly::OpenSSLTransportCertificate::tryExtractX509(cert);
     }
-    sock_->close();
   }
 
   void connectErr(const folly::AsyncSocketException&) noexcept override {
@@ -197,19 +274,57 @@ class Cb : public folly::AsyncSocket::ConnectCallback {
   }
 
   bool success{false};
-  bool reusedSession{false};
-  std::shared_ptr<folly::ssl::SSLSession> session;
   folly::AsyncSSLSocket* sock_{nullptr};
   folly::ssl::X509UniquePtr peerCert_{nullptr};
 };
 
-TEST(SSL, SSLTest) {
-  HTTPServer::IPConfig cfg{folly::SocketAddress("127.0.0.1", 0),
-                           HTTPServer::Protocol::HTTP};
+wangle::SSLContextConfig getSslContextConfig(bool useMultiCA) {
   wangle::SSLContextConfig sslCfg;
   sslCfg.isDefault = true;
   sslCfg.setCertificate(
       kTestDir + "certs/test_cert1.pem", kTestDir + "certs/test_key1.pem", "");
+  if (useMultiCA) {
+    sslCfg.clientCAFiles =
+        std::vector<std::string>{kTestDir + "/certs/ca_cert.pem",
+                                 kTestDir + "/certs/client_ca_cert.pem"};
+  } else {
+    sslCfg.clientCAFile = kTestDir + "/certs/ca_cert.pem";
+  }
+  sslCfg.clientVerification =
+      folly::SSLContext::VerifyClientCertificate::IF_PRESENTED;
+  return sslCfg;
+}
+
+TEST(SSL, SSLTest) {
+  HTTPServer::IPConfig cfg{folly::SocketAddress("127.0.0.1", 0),
+                           HTTPServer::Protocol::HTTP};
+  wangle::SSLContextConfig sslCfg = getSslContextConfig(false);
+  cfg.sslConfigs.push_back(sslCfg);
+
+  HTTPServerOptions options;
+  options.threads = 4;
+
+  auto server = std::make_unique<HTTPServer>(std::move(options));
+
+  std::vector<HTTPServer::IPConfig> ips{cfg};
+  server->bind(ips);
+
+  ServerThread st(server.get());
+  EXPECT_TRUE(st.start());
+
+  folly::EventBase evb;
+  auto ctx = std::make_shared<SSLContext>();
+  folly::AsyncSSLSocket::UniquePtr sock(new folly::AsyncSSLSocket(ctx, &evb));
+  Cb cb(sock.get());
+  sock->connect(&cb, server->addresses().front().address, 1000);
+  evb.loop();
+  EXPECT_TRUE(cb.success);
+}
+
+TEST(SSL, SSLTestWithMultiCAs) {
+  HTTPServer::IPConfig cfg{folly::SocketAddress("127.0.0.1", 0),
+                           HTTPServer::Protocol::HTTP};
+  wangle::SSLContextConfig sslCfg = getSslContextConfig(true);
   cfg.sslConfigs.push_back(sslCfg);
 
   HTTPServerOptions options;
@@ -269,7 +384,7 @@ class TestHandlerFactory : public RequestHandlerFactory {
       auto& transport = txn->getTransport();
       if (auto cert =
               transport.getUnderlyingTransport()->getPeerCertificate()) {
-        auto x509 = cert->getX509();
+        auto x509 = folly::OpenSSLTransportCertificate::tryExtractX509(cert);
         if (x509) {
           certHeader = OpenSSLCertUtils::getCommonName(*x509).value_or("");
         }
@@ -309,6 +424,9 @@ setupServer(bool allowInsecureConnectionsOnSecureServer = false,
   sslCfg.isDefault = true;
   sslCfg.setCertificate(
       kTestDir + "certs/test_cert1.pem", kTestDir + "certs/test_key1.pem", "");
+  sslCfg.clientCAFile = kTestDir + "/certs/ca_cert.pem";
+  sslCfg.clientVerification =
+      folly::SSLContext::VerifyClientCertificate::IF_PRESENTED;
   cfg.sslConfigs.push_back(sslCfg);
   cfg.allowInsecureConnectionsOnSecureServer =
       allowInsecureConnectionsOnSecureServer;
@@ -381,71 +499,201 @@ TEST(SSL, DisallowInsecureOnSecureServer) {
   EXPECT_EQ(nullptr, response);
 }
 
-TEST(SSL, TestResumptionWithTickets) {
+constexpr size_t kBufferSize = 4096;
+const std::string kSessionCacheContext{"some random string"};
+
+// A dummy callback to handle session ticket reads. With TLS 1.3, which the
+// resumption tests are configured to use, the session ticket(s) arrives after
+// the handshake is complete. We can use instances of this class to read from
+// the socket, while the corresponding EventBase is looping, and eventually
+// receive the parsed session.
+struct SSLSessionReadCallback
+    : public wangle::SSLSessionCallbacks
+    , public folly::AsyncTransport::ReadCallback {
+  SSLSessionReadCallback() {
+    readBuffer_ = folly::IOBuf::create(kBufferSize);
+  }
+  void setSSLSession(
+      const std::string& /* ignore */,
+      folly::ssl::SSLSessionUniquePtr sessionPtr) noexcept override {
+    session = std::move(sessionPtr);
+    // without a read callback, the event base will stop looping
+    if (socket) {
+      socket->setReadCB(nullptr);
+    }
+  }
+  folly::ssl::SSLSessionUniquePtr getSSLSession(
+      const std::string&) const noexcept override {
+    return nullptr;
+  }
+  bool removeSSLSession(const std::string&) noexcept override {
+    return true;
+  }
+  void readDataAvailable(size_t) noexcept override {
+  }
+  void getReadBuffer(void** buf, size_t* lenReturn) override {
+    *buf = readBuffer_->writableData();
+    *lenReturn = kBufferSize;
+  }
+  void readEOF() noexcept override {
+  }
+  void readErr(const folly::AsyncSocketException&) noexcept override {
+  }
+  folly::AsyncSocket* socket{nullptr};
+  folly::ssl::SSLSessionUniquePtr session;
+
+ private:
+  std::unique_ptr<folly::IOBuf> readBuffer_;
+};
+
+TEST(SSL, TestResumptionWithTicketsTLS12) {
   std::unique_ptr<HTTPServer> server;
+
   std::unique_ptr<ServerThread> st;
   wangle::TLSTicketKeySeeds seeds;
-  seeds.currentSeeds.push_back(hexlify("hello"));
+  seeds.currentSeeds.push_back(hexlify(std::string(32, 'a')));
   std::tie(server, st) = setupServer(false, seeds);
 
   folly::EventBase evb;
-  auto ctx = std::make_shared<SSLContext>();
+  auto ctx =
+      std::make_shared<SSLContext>(folly::SSLContext::SSLVersion::TLSv1_2);
+  // explicitly disable TLS 1.3
+  ctx->disableTLS13();
+  ctx->setSessionCacheContext(kSessionCacheContext);
+  SSLSessionReadCallback sessionCb;
+  wangle::SSLSessionCallbacks::attachCallbacksToContext(ctx.get(), &sessionCb);
   folly::AsyncSSLSocket::UniquePtr sock(new folly::AsyncSSLSocket(ctx, &evb));
+  sock->setSessionKey(kSessionCacheContext);
   Cb cb(sock.get());
   sock->connect(&cb, server->addresses().front().address, 1000);
+  // In TLS 1.2, the session arrives during the handshake/connect. We don't need
+  // to register a read callback. Once the connect completes, there won't be any
+  // events to process and the loop will exit.
   evb.loop();
   ASSERT_TRUE(cb.success);
-  ASSERT_NE(nullptr, cb.session.get());
-  ASSERT_FALSE(cb.reusedSession);
+  ASSERT_NE(sessionCb.session.get(), nullptr);
+  ASSERT_FALSE(sock->getSSLSessionReused());
 
   folly::AsyncSSLSocket::UniquePtr sock2(new folly::AsyncSSLSocket(ctx, &evb));
-  sock2->setSSLSession(cb.session);
+  auto sslSession = std::move(sessionCb.session);
+  ASSERT_NE(sslSession.get(), nullptr);
+  sock2->setRawSSLSession(std::move(sslSession));
   Cb cb2(sock2.get());
   sock2->connect(&cb2, server->addresses().front().address, 1000);
   evb.loop();
   ASSERT_TRUE(cb2.success);
-  ASSERT_NE(nullptr, cb2.session.get());
-  ASSERT_TRUE(cb2.reusedSession);
+  ASSERT_TRUE(sock2->getSSLSessionReused());
+}
+
+TEST(SSL, TestResumptionWithTickets) {
+  std::unique_ptr<HTTPServer> server;
+
+  std::unique_ptr<ServerThread> st;
+  wangle::TLSTicketKeySeeds seeds;
+  seeds.currentSeeds.push_back(hexlify(std::string(32, 'a')));
+  std::tie(server, st) = setupServer(false, seeds);
+
+  folly::EventBase evb;
+  auto ctx = std::make_shared<SSLContext>();
+  ctx->enableTLS13();
+  ctx->setSessionCacheContext(kSessionCacheContext);
+  SSLSessionReadCallback sessionCb;
+  wangle::SSLSessionCallbacks::attachCallbacksToContext(ctx.get(), &sessionCb);
+  folly::AsyncSSLSocket::UniquePtr sock(new folly::AsyncSSLSocket(ctx, &evb));
+  sock->setSessionKey(kSessionCacheContext);
+  Cb cb(sock.get());
+  sock->connect(&cb, server->addresses().front().address, 1000);
+  // For TLS 1.3, we need the read callback so that we can read the
+  // NewSessionTicket message
+  sock->setReadCB(&sessionCb);
+  // attach the socket so session callback can unregister read callback
+  sessionCb.socket = sock.get();
+  evb.loop();
+  ASSERT_TRUE(cb.success);
+  ASSERT_NE(sessionCb.session.get(), nullptr);
+  ASSERT_FALSE(sock->getSSLSessionReused());
+
+  folly::AsyncSSLSocket::UniquePtr sock2(new folly::AsyncSSLSocket(ctx, &evb));
+  auto sslSession = std::move(sessionCb.session);
+  ASSERT_NE(sslSession.get(), nullptr);
+  sock2->setRawSSLSession(std::move(sslSession));
+  Cb cb2(sock2.get());
+  sock2->connect(&cb2, server->addresses().front().address, 1000);
+  evb.loop();
+  ASSERT_TRUE(cb2.success);
+  ASSERT_TRUE(sock2->getSSLSessionReused());
 }
 
 TEST(SSL, TestResumptionAfterUpdateFails) {
   std::unique_ptr<HTTPServer> server;
   std::unique_ptr<ServerThread> st;
   wangle::TLSTicketKeySeeds seeds;
-  seeds.currentSeeds.push_back(hexlify("hello"));
+  seeds.currentSeeds.push_back(hexlify(std::string(32, 'a')));
   std::tie(server, st) = setupServer(false, seeds);
 
   folly::EventBase evb;
   auto ctx = std::make_shared<SSLContext>();
-  folly::AsyncSSLSocket::UniquePtr sock(new folly::AsyncSSLSocket(ctx, &evb));
-  Cb cb(sock.get());
-  sock->connect(&cb, server->addresses().front().address, 1000);
-  evb.loop();
-  ASSERT_TRUE(cb.success);
-  ASSERT_NE(nullptr, cb.session.get());
-  ASSERT_FALSE(cb.reusedSession);
+  ctx->enableTLS13();
+  ctx->setAdvertisedNextProtocols({"http/1.1"});
+  ctx->setSessionCacheContext(kSessionCacheContext);
+  SSLSessionReadCallback sessionCb;
+  wangle::SSLSessionCallbacks::attachCallbacksToContext(ctx.get(), &sessionCb);
+  {
+    folly::AsyncSSLSocket::UniquePtr sock(new folly::AsyncSSLSocket(ctx, &evb));
+    sock->setSessionKey(kSessionCacheContext);
+    Cb cb(sock.get());
+    sock->connect(&cb, server->addresses().front().address, 1000);
+    // For TLS 1.3, we need the read callback so that we can read the
+    // NewSessionTicket message
+    sock->setReadCB(&sessionCb);
+    // attach the socket so session callback can unregister read callback
+    sessionCb.socket = sock.get();
+    evb.loop();
+    ASSERT_TRUE(cb.success);
+    ASSERT_NE(sessionCb.session.get(), nullptr);
+    ASSERT_FALSE(sock->getSSLSessionReused());
+  }
 
+  // change ticket secrets
   wangle::TLSTicketKeySeeds newSeeds;
-  newSeeds.currentSeeds.push_back(hexlify("goodbyte"));
+  newSeeds.currentSeeds.push_back(hexlify(std::string(32, 'b')));
   server->updateTicketSeeds(newSeeds);
 
-  folly::AsyncSSLSocket::UniquePtr sock2(new folly::AsyncSSLSocket(ctx, &evb));
-  sock2->setSSLSession(cb.session);
-  Cb cb2(sock2.get());
-  sock2->connect(&cb2, server->addresses().front().address, 1000);
-  evb.loop();
-  ASSERT_TRUE(cb2.success);
-  ASSERT_NE(nullptr, cb2.session.get());
-  ASSERT_FALSE(cb2.reusedSession);
+  {
+    // We can't resume with the ticket received during the first connection
+    // because the server has since changed its ticket secrets. We expect this
+    // resumption to fail. We also expect to receive a new session ticket that
+    // we can use to resume a follow up connection.
+    folly::AsyncSSLSocket::UniquePtr sock2(
+        new folly::AsyncSSLSocket(ctx, &evb));
+    sock2->setRawSSLSession(std::move(sessionCb.session));
+    sock2->setSessionKey(kSessionCacheContext);
+    Cb cb2(sock2.get());
+    sock2->connect(&cb2, server->addresses().front().address, 1000);
+    sock2->setReadCB(&sessionCb);
+    // attach the socket so session callback can unregister read callback
+    sessionCb.socket = sock2.get();
+    evb.loop();
+    ASSERT_TRUE(cb2.success);
+    ASSERT_NE(sessionCb.session.get(), nullptr);
+    ASSERT_FALSE(sock2->getSSLSessionReused());
+  }
 
-  folly::AsyncSSLSocket::UniquePtr sock3(new folly::AsyncSSLSocket(ctx, &evb));
-  sock3->setSSLSession(cb2.session);
-  Cb cb3(sock3.get());
-  sock3->connect(&cb3, server->addresses().front().address, 1000);
-  evb.loop();
-  ASSERT_TRUE(cb3.success);
-  ASSERT_NE(nullptr, cb3.session.get());
-  ASSERT_TRUE(cb3.reusedSession);
+  {
+    // 3rd request expected to succeed resumption (against 2nd's ticket)
+    folly::AsyncSSLSocket::UniquePtr sock3(
+        new folly::AsyncSSLSocket(ctx, &evb));
+    sock3->setRawSSLSession(std::move(sessionCb.session));
+    sock3->setSessionKey(kSessionCacheContext);
+    Cb cb3(sock3.get());
+    sock3->connect(&cb3, server->addresses().front().address, 1000);
+    sock3->setReadCB(&sessionCb);
+    // attach the socket so session callback can unregister read callback
+    sessionCb.socket = sock3.get();
+    evb.loop();
+    ASSERT_TRUE(cb3.success);
+    ASSERT_TRUE(sock3->getSSLSessionReused());
+  }
 }
 
 TEST(SSL, TestUpdateTLSCredentials) {
@@ -478,6 +726,9 @@ TEST(SSL, TestUpdateTLSCredentials) {
   sslCfg.isDefault = true;
   copyCreds(kTestDir + "certs/test_cert1.pem",
             kTestDir + "certs/test_key1.pem");
+  sslCfg.clientCAFile = kTestDir + "/certs/ca_cert.pem";
+  sslCfg.clientVerification =
+      folly::SSLContext::VerifyClientCertificate::IF_PRESENTED;
   sslCfg.setCertificate(credFile.path().string(), credFile.path().string(), "");
   cfg.sslConfigs.push_back(sslCfg);
 
@@ -622,8 +873,6 @@ TEST(UseExistingSocket, TestWithMultipleSocketFds) {
     // This is fine because we are trying to bind to multiple ports
   }
 
-  HTTPServer::IPConfig cfg{folly::SocketAddress("127.0.0.1", 0),
-                           HTTPServer::Protocol::HTTP};
   HTTPServerOptions options;
   options.handlerFactories =
       RequestHandlerChain().addThen<TestHandlerFactory>().build();
@@ -638,8 +887,9 @@ TEST(UseExistingSocket, TestWithMultipleSocketFds) {
 
   auto server = std::make_unique<HTTPServer>(std::move(options));
   auto st = std::make_unique<ServerThread>(server.get());
-  std::vector<HTTPServer::IPConfig> ips{cfg};
-  server->bind(ips);
+  server->bind(
+      {{folly::SocketAddress("127.0.0.1", 0), HTTPServer::Protocol::HTTP},
+       {folly::SocketAddress("127.0.0.1", 0), HTTPServer::Protocol::HTTP2}});
 
   EXPECT_TRUE(st->start());
 
@@ -746,6 +996,9 @@ TEST_F(ScopedServerTest, StartSSLWithInsecure) {
   sslCfg.isDefault = true;
   sslCfg.setCertificate(
       kTestDir + "certs/test_cert1.pem", kTestDir + "certs/test_key1.pem", "");
+  sslCfg.clientCAFile = kTestDir + "/certs/ca_cert.pem";
+  sslCfg.clientVerification =
+      folly::SSLContext::VerifyClientCertificate::IF_PRESENTED;
   cfg_.sslConfigs.push_back(sslCfg);
   cfg_.allowInsecureConnectionsOnSecureServer = true;
   auto server = createScopedServer();
@@ -779,7 +1032,7 @@ class ConnectionFilterTest : public ScopedServerTest {
           if (!cert) {
             throw std::runtime_error("Client cert is missing");
           }
-          auto x509 = cert->getX509();
+          auto x509 = folly::OpenSSLTransportCertificate::tryExtractX509(cert);
           if (!x509 || OpenSSLCertUtils::getCommonName(*x509).value_or("") !=
                            "testuser1") {
             throw std::runtime_error("Client cert is invalid.");
@@ -796,7 +1049,8 @@ TEST_F(ConnectionFilterTest, Test) {
       kTestDir + "certs/test_cert1.pem", kTestDir + "certs/test_key1.pem", "");
   sslCfg.clientCAFile = kTestDir + "certs/client_ca_cert.pem";
   // Permissive client auth.
-  sslCfg.clientVerification = folly::SSLContext::SSLVerifyPeerEnum::VERIFY;
+  sslCfg.clientVerification =
+      folly::SSLContext::VerifyClientCertificate::IF_PRESENTED;
   cfg_.sslConfigs.push_back(sslCfg);
 
   auto server = createScopedServer();
