@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -31,7 +31,13 @@ std::locale defaultLocale;
 
 namespace proxygen {
 
-const int8_t HTTPMessage::kMaxPriority = 7;
+std::string httpPriorityToString(uint8_t urgency, bool incremental) {
+  return folly::to<std::string>(
+      "u=",
+      std::min(static_cast<uint8_t>(proxygen::kMaxPriority), urgency),
+      incremental ? ",i" : "");
+}
+
 std::mutex HTTPMessage::mutexDump_;
 
 const pair<uint8_t, uint8_t> HTTPMessage::kHTTPVersion09(0, 9);
@@ -68,16 +74,13 @@ HTTPMessage::HTTPMessage()
       sslCipher_(nullptr),
       protoStr_(nullptr),
       pri_(kDefaultHttpPriorityUrgency),
-      incremental_(kDefaultHttpPriorityIncremental),
       version_(1, 0),
       parsedCookies_(false),
       parsedQueryParams_(false),
       chunked_(false),
       upgraded_(false),
       wantsKeepalive_(true),
-      trailersAllowed_(false),
-      secure_(false),
-      partiallyReliable_(false) {
+      trailersAllowed_(false) {
 }
 
 HTTPMessage::~HTTPMessage() {
@@ -100,7 +103,6 @@ HTTPMessage::HTTPMessage(const HTTPMessage& message)
       sslCipher_(message.sslCipher_),
       protoStr_(message.protoStr_),
       pri_(message.pri_),
-      incremental_(message.incremental_),
       h2Pri_(message.h2Pri_),
       version_(message.version_),
       parsedCookies_(message.parsedCookies_),
@@ -109,8 +111,7 @@ HTTPMessage::HTTPMessage(const HTTPMessage& message)
       upgraded_(message.upgraded_),
       wantsKeepalive_(message.wantsKeepalive_),
       trailersAllowed_(message.trailersAllowed_),
-      secure_(message.secure_),
-      partiallyReliable_(message.partiallyReliable_) {
+      scheme_(message.scheme_) {
   if (isRequest()) {
     setURL(request().url_);
   }
@@ -142,7 +143,6 @@ HTTPMessage::HTTPMessage(HTTPMessage&& message) noexcept
       sslCipher_(message.sslCipher_),
       protoStr_(message.protoStr_),
       pri_(message.pri_),
-      incremental_(message.incremental_),
       h2Pri_(message.h2Pri_),
       version_(message.version_),
       parsedCookies_(message.parsedCookies_),
@@ -151,8 +151,7 @@ HTTPMessage::HTTPMessage(HTTPMessage&& message) noexcept
       upgraded_(message.upgraded_),
       wantsKeepalive_(message.wantsKeepalive_),
       trailersAllowed_(message.trailersAllowed_),
-      secure_(message.secure_),
-      partiallyReliable_(message.partiallyReliable_) {
+      scheme_(message.scheme_) {
   if (isRequest()) {
     setURL(request().url_);
   }
@@ -187,7 +186,6 @@ HTTPMessage& HTTPMessage::operator=(const HTTPMessage& message) {
   sslCipher_ = message.sslCipher_;
   protoStr_ = message.protoStr_;
   pri_ = message.pri_;
-  incremental_ = message.incremental_;
   h2Pri_ = message.h2Pri_;
   parsedCookies_ = message.parsedCookies_;
   parsedQueryParams_ = message.parsedQueryParams_;
@@ -195,7 +193,7 @@ HTTPMessage& HTTPMessage::operator=(const HTTPMessage& message) {
   upgraded_ = message.upgraded_;
   wantsKeepalive_ = message.wantsKeepalive_;
   trailersAllowed_ = message.trailersAllowed_;
-  secure_ = message.secure_;
+  scheme_ = message.scheme_;
   upgradeWebsocket_ = message.upgradeWebsocket_;
 
   if (message.trailers_) {
@@ -230,7 +228,6 @@ HTTPMessage& HTTPMessage::operator=(HTTPMessage&& message) {
   sslCipher_ = message.sslCipher_;
   protoStr_ = message.protoStr_;
   pri_ = message.pri_;
-  incremental_ = message.incremental_;
   h2Pri_ = message.h2Pri_;
   parsedCookies_ = message.parsedCookies_;
   parsedQueryParams_ = message.parsedQueryParams_;
@@ -238,7 +235,7 @@ HTTPMessage& HTTPMessage::operator=(HTTPMessage&& message) {
   upgraded_ = message.upgraded_;
   wantsKeepalive_ = message.wantsKeepalive_;
   trailersAllowed_ = message.trailersAllowed_;
-  secure_ = message.secure_;
+  scheme_ = message.scheme_;
   upgradeWebsocket_ = message.upgradeWebsocket_;
   trailers_ = std::move(message.trailers_);
   return *this;
@@ -541,25 +538,29 @@ const std::map<std::string, std::string>& HTTPMessage::getQueryParams() const {
   return queryParams_;
 }
 
-bool HTTPMessage::setQueryString(const std::string& query) {
-  return setQueryStringImpl(query, true);
+bool HTTPMessage::setQueryString(const std::string& query, bool strict) {
+  return setQueryStringImpl(query, true, strict);
 }
 
-bool HTTPMessage::setQueryStringImpl(const std::string& query, bool unparse) {
-  ParseURL u(request().url_);
+bool HTTPMessage::setQueryStringImpl(const std::string& query,
+                                     bool unparse,
+                                     bool strict) {
+  // No need to strictly verify the URL when reparsing it
+  auto u = ParseURL::parseURL(request().url_, /*strict=*/false);
 
-  if (u.valid()) {
+  if (u) {
     // Recreate the URL by just changing the query string
-    setURLImpl(createUrl(u.scheme(),
-                         u.authority(),
-                         u.path(),
-                         query, // new query string
-                         u.fragment()),
-               unparse);
-    return true;
+    auto res = setURLImpl(createUrl(u->scheme(),
+                                    u->authority(),
+                                    u->path(),
+                                    query, // new query string
+                                    u->fragment()),
+                          unparse,
+                          strict);
+    return !strict || res.valid();
   }
 
-  VLOG(4) << "Error parsing URL during setQueryString: " << request().url_;
+  DVLOG(4) << "Error parsing URL during setQueryString: " << request().url_;
   return false;
 }
 
@@ -575,11 +576,12 @@ bool HTTPMessage::removeQueryParam(const std::string& name) {
   }
 
   auto query = createQueryString(queryParams_, request().query_.size());
-  return setQueryStringImpl(query, false);
+  return setQueryStringImpl(query, false, /*strict=*/false);
 }
 
 bool HTTPMessage::setQueryParam(const std::string& name,
-                                const std::string& value) {
+                                const std::string& value,
+                                bool strict) {
   // Parse the query parameters if we haven't done so yet
   if (!parsedQueryParams_) {
     parseQueryParams();
@@ -587,7 +589,7 @@ bool HTTPMessage::setQueryParam(const std::string& name,
 
   queryParams_[name] = value;
   auto query = createQueryString(queryParams_, request().query_.size());
-  return setQueryStringImpl(query, false);
+  return setQueryStringImpl(query, false, strict);
 }
 
 std::string HTTPMessage::createQueryString(
@@ -719,14 +721,12 @@ std::ostream& operator<<(std::ostream& os, const HTTPMessage& msg) {
 }
 
 void HTTPMessage::dumpMessage(int vlogLevel) const {
-  VLOG(vlogLevel) << *this;
+  DVLOG(vlogLevel) << *this;
 }
 
 void HTTPMessage::describe(std::ostream& os) const {
   os << ", chunked: " << chunked_ << ", upgraded: " << upgraded_
-     << ", secure: " << secure_
-     << ", partially reliable: " << partiallyReliable_
-     << ", Fields for message:" << std::endl;
+     << ", scheme: " << getScheme() << ", Fields for message:" << std::endl;
 
   // Common fields to both requests and responses.
   std::vector<std::pair<const char*, folly::StringPiece>> fields{{
@@ -939,6 +939,16 @@ const char* HTTPMessage::getDefaultReason(uint16_t status) {
       return "Expectation Failed";
     case 418:
       return "I'm a teapot";
+    case 426:
+      return "Upgrade Required";
+    case 428:
+      return "Precondition Required";
+    case 429:
+      return "Too Many Requests";
+    case 431:
+      return "Request Header Fields Too Large";
+    case 451:
+      return "Unavailable For Legal Reasons";
     case 500:
       return "Internal Server Error";
     case 501:
@@ -958,15 +968,18 @@ const char* HTTPMessage::getDefaultReason(uint16_t status) {
   return "-";
 }
 
-ParseURL HTTPMessage::setURLImplInternal(bool unparse) {
+ParseURL HTTPMessage::setURLImplInternal(bool unparse, bool strict) {
   auto& req = request();
-  ParseURL u(req.url_);
+  auto u = ParseURL::parseURLMaybeInvalid(req.url_, strict);
   if (u.valid()) {
-    VLOG(9) << "set path: " << u.path() << " query:" << u.query();
+    DVLOG(9) << "set path: " << u.path() << " query:" << u.query();
     req.path_ = u.path();
     req.query_ = u.query();
+    if (req.path_.empty()) {
+      req.path_.reset("/", 1);
+    }
   } else {
-    VLOG(4) << "Error in parsing URL: " << req.url_;
+    DVLOG(4) << "Error in parsing URL: " << req.url_;
     req.path_.clear();
     req.query_.clear();
   }
@@ -978,4 +991,15 @@ ParseURL HTTPMessage::setURLImplInternal(bool unparse) {
   return u;
 }
 
+void HTTPMessage::setHTTPPriority(uint8_t urgency, bool incremental) {
+  headers_.set(HTTP_HEADER_PRIORITY,
+               httpPriorityToString(urgency, incremental));
+}
+
+void HTTPMessage::setHTTPPriority(HTTPPriority httpPriority) {
+  headers_.set(
+      HTTP_HEADER_PRIORITY,
+      httpPriorityToString(httpPriority.urgency, httpPriority.incremental));
+}
+
 } // namespace proxygen
