@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -20,6 +20,7 @@
 #include <proxygen/lib/http/HTTPHeaderSize.h>
 #include <proxygen/lib/http/HTTPHeaders.h>
 #include <proxygen/lib/http/HTTPMethod.h>
+#include <proxygen/lib/http/HeaderConstants.h>
 #include <proxygen/lib/utils/ParseURL.h>
 #include <proxygen/lib/utils/Time.h>
 #include <string>
@@ -31,7 +32,34 @@ namespace proxygen {
 // opposed to False. This is because our transport layer has been behaving
 // like that before the HTTP priority support is introduced.
 constexpr uint8_t kDefaultHttpPriorityUrgency = 3;
+// We default incremental to True, different from the draft
 constexpr bool kDefaultHttpPriorityIncremental = true;
+constexpr int8_t kMaxPriority = 7;
+
+struct HTTPPriority {
+  uint8_t urgency : 3;
+  bool incremental : 1;
+
+  HTTPPriority()
+      : urgency(kDefaultHttpPriorityUrgency),
+        incremental(kDefaultHttpPriorityIncremental) {
+  }
+
+  HTTPPriority(uint8_t urgencyIn, bool incrementalIn)
+      : urgency(std::min(urgencyIn, static_cast<uint8_t>(kMaxPriority))),
+        incremental(incrementalIn) {
+  }
+
+  virtual ~HTTPPriority() = default;
+};
+
+// Convert Priority to a string representation in the form of "u=urgency[,i]"
+std::string httpPriorityToString(uint8_t urgency, bool incremental);
+
+class HTTPMessage;
+
+folly::Optional<HTTPPriority> httpPriorityFromHTTPMessage(
+    const HTTPMessage& message);
 
 /**
  * An HTTP request or response minus the body.
@@ -50,6 +78,12 @@ class HTTPMessage {
     EGRESS,
   };
 
+  enum class Scheme {
+    HTTP,
+    HTTPS,
+    MASQUE,
+  };
+
   HTTPMessage();
   ~HTTPMessage();
   HTTPMessage(HTTPMessage&& message) noexcept;
@@ -220,14 +254,14 @@ class HTTPMessage {
    * valid, this is a full URL, not just a path.
    */
   template <typename T> // T = string
-  ParseURL setURL(T&& url) {
-    return setURLImpl(std::forward<T>(url), true);
+  ParseURL setURL(T&& url, bool strict = false) {
+    return setURLImpl(std::forward<T>(url), true, strict);
   }
 
   // The template function above doesn't work with char*,
   // so explicitly convert to a string first.
-  void setURL(const char* url) {
-    setURL(std::string(url));
+  ParseURL setURL(const char* url, bool strict = false) {
+    return setURL(std::string(url), strict);
   }
   const std::string& getURL() const {
     return request().url_;
@@ -375,6 +409,13 @@ class HTTPMessage {
    */
   bool isHTTP1_1() const;
 
+  /**
+   * Returns true if these are final headers.
+   */
+  bool isFinal() const {
+    return (isRequest() || !is1xxResponse() || getStatusCode() == 101);
+  }
+
   /**
    * Returns true if this is a 1xx response.
    */
@@ -547,7 +588,7 @@ class HTTPMessage {
    *
    * Returns true if the query string was changed successfully.
    */
-  bool setQueryString(const std::string& query);
+  bool setQueryString(const std::string& query, bool strict = false);
 
   /**
    * Remove the query parameter with the specified name.
@@ -561,7 +602,9 @@ class HTTPMessage {
    *
    * Returns true if the query parameter was successfully set.
    */
-  bool setQueryParam(const std::string& name, const std::string& value);
+  bool setQueryParam(const std::string& name,
+                     const std::string& value,
+                     bool strict = false);
 
   /**
    * Get the cookie with the specified name.
@@ -606,28 +649,59 @@ class HTTPMessage {
   }
 
   void setSecure(bool secure) {
-    secure_ = secure;
+    if (secure && scheme_ != Scheme::MASQUE) {
+      scheme_ = Scheme::HTTPS;
+    } else if (!secure) {
+      scheme_ = Scheme::HTTP;
+    }
   }
+
   bool isSecure() const {
-    return secure_;
+    return (scheme_ == Scheme::HTTPS || scheme_ == Scheme::MASQUE);
   }
+
+  void setMasque() {
+    scheme_ = Scheme::MASQUE;
+  }
+
+  bool isMasque() const {
+    return scheme_ == Scheme::MASQUE;
+  }
+
+  const std::string& getScheme() const {
+    switch (scheme_) {
+      case HTTPMessage::Scheme::HTTP:
+        return headers::kHttp;
+      case HTTPMessage::Scheme::HTTPS:
+        return headers::kHttps;
+      case HTTPMessage::Scheme::MASQUE:
+        return headers::kMasque;
+    }
+    return headers::kHttp;
+  }
+
   int getSecureVersion() const {
     return sslVersion_;
   }
+
   const char* getSecureCipher() const {
     return sslCipher_;
   }
+
   void setSecureInfo(int ver, const char* cipher) {
     // cipher is a static const char* provided and managed by openssl lib
     sslVersion_ = ver;
     sslCipher_ = cipher;
   }
+
   void setAdvancedProtocolString(const std::string& protocol) {
     protoStr_ = &protocol;
   }
+
   bool isAdvancedProto() const {
     return protoStr_ != nullptr;
   }
+
   const std::string* getAdvancedProtocolString() const {
     return protoStr_;
   }
@@ -654,8 +728,6 @@ class HTTPMessage {
    * value for this SPDY version (i.e. 3 for SPDY/2 or 7 for SPDY/3),
    * -2 the second largest (i.e. 2 for SPDY/2 or 6 for SPDY/3).
    */
-  const static int8_t kMaxPriority;
-
   static uint8_t normalizePriority(int8_t pri) {
     if (pri > kMaxPriority || pri < -kMaxPriority) {
       // outside [-7, 7] => highest priority
@@ -674,20 +746,30 @@ class HTTPMessage {
     return pri_;
   }
 
-  void setIncremental(bool val) {
-    incremental_ = val;
-  }
-  bool getIncremental() const noexcept {
-    return incremental_;
+  /**
+   * Set a Priority header on the HTTPMessage by urgency and incremental.
+   */
+  void setHTTPPriority(uint8_t urgency, bool incremental);
+
+  folly::Optional<HTTPPriority> getHTTPPriority() const noexcept {
+    return httpPriorityFromHTTPMessage(*this);
   }
 
-  using HTTPPriority = std::tuple<uint64_t, bool, uint8_t>;
+  /**
+   * Set a Priority header on the HTTPMessage by httpPriority.
+   *
+   * There is no getter of HTTPPriority in a HTTPMessage.
+   * Use httpPriorityFromHTTPMessage for that.
+   */
+  void setHTTPPriority(HTTPPriority httpPriority);
 
-  folly::Optional<HTTPPriority> getHTTP2Priority() const {
+  using HTTP2Priority = std::tuple<uint64_t, bool, uint8_t>;
+
+  folly::Optional<HTTP2Priority> getHTTP2Priority() const {
     return h2Pri_;
   }
 
-  void setHTTP2Priority(HTTPPriority h2Pri) {
+  void setHTTP2Priority(HTTP2Priority h2Pri) {
     h2Pri_ = h2Pri;
   }
 
@@ -802,22 +884,6 @@ class HTTPMessage {
   static std::string createQueryString(
       const std::map<std::string, std::string>& params, uint32_t maxSize);
 
-  /**
-   * Marks the message partially reliable.
-   * Partial reliability flag allows HTTP transaction to execute partially
-   * reliable API methods such as skipBodyTo() and rejectBodyTo().
-   *
-   * Message, once marked partially reliable, cannot go back to full
-   * reliability, hence only one set method below.
-   */
-  void setPartiallyReliable() {
-    partiallyReliable_ = true;
-  }
-
-  bool isPartiallyReliable() const {
-    return partiallyReliable_;
-  }
-
  protected:
   // Message start time, in msec since the epoch.
   TimePoint startTime_;
@@ -826,17 +892,19 @@ class HTTPMessage {
   void parseCookies() const;
 
   template <typename T> // T = string
-  ParseURL setURLImpl(T&& url, bool unparse) {
-    VLOG(9) << "setURL: " << url;
+  ParseURL setURLImpl(T&& url, bool unparse, bool strict) {
+    DVLOG(9) << "setURL: " << std::forward<T>(url);
 
     // Set the URL, path, and query string parameters
     request().url_ = std::forward<T>(url);
-    return setURLImplInternal(unparse);
+    return setURLImplInternal(unparse, strict);
   }
 
-  ParseURL setURLImplInternal(bool unparse);
+  ParseURL setURLImplInternal(bool unparse, bool strict);
 
-  bool setQueryStringImpl(const std::string& queryString, bool unparse);
+  bool setQueryStringImpl(const std::string& queryString,
+                          bool unparse,
+                          bool strict);
   void parseQueryParams() const;
   void unparseQueryParams();
 
@@ -997,6 +1065,8 @@ class HTTPMessage {
     if (fields_.which_ == MessageType::NONE) {
       fields_.which_ = MessageType::REQUEST;
       new (&fields_.data_.request) Request();
+    } else if (fields_.which_ == MessageType::RESPONSE) {
+      throw std::runtime_error("Invoked Request API on HTTP Response");
     }
 
     return fields_.data_.request;
@@ -1014,6 +1084,8 @@ class HTTPMessage {
     if (fields_.which_ == MessageType::NONE) {
       fields_.which_ = MessageType::RESPONSE;
       new (&fields_.data_.response) Response();
+    } else if (fields_.which_ == MessageType::REQUEST) {
+      throw std::runtime_error("Invoked Response API on HTTP Request");
     }
 
     return fields_.data_.response;
@@ -1045,8 +1117,7 @@ class HTTPMessage {
   const std::string* protoStr_;
   std::unique_ptr<std::string> upgradeProtocol_;
   uint8_t pri_;
-  bool incremental_ : 1;
-  folly::Optional<HTTPPriority> h2Pri_;
+  folly::Optional<HTTP2Priority> h2Pri_;
 
   std::pair<uint8_t, uint8_t> version_;
   mutable bool parsedCookies_ : 1;
@@ -1056,13 +1127,7 @@ class HTTPMessage {
   bool wantsKeepalive_ : 1;
   bool trailersAllowed_ : 1;
 
-  // Whether the message is received in HTTPS.
-  bool secure_ : 1;
-
-  // Whether the message is partially reliable.
-  // This is used by transaction/codec to allow/disallow certain operations on
-  // transaction.
-  bool partiallyReliable_ : 1;
+  Scheme scheme_{Scheme::HTTP};
 
   // used by atomicDumpMessage
   static std::mutex mutexDump_;
@@ -1085,5 +1150,4 @@ std::string stripCntrlChars(const Str& str) {
   }
   return res;
 }
-
 } // namespace proxygen
