@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -37,11 +37,6 @@ enum class ErrorCode : uint8_t {
   ENHANCE_YOUR_CALM = 11,
   INADEQUATE_SECURITY = 12,
   HTTP_1_1_REQUIRED = 13,
-  // This code is *NOT* to be used outside of SPDYCodec. Delete this
-  // when we deprecate SPDY.
-  _SPDY_INVALID_STREAM = 100,
-  // Partially reliable bad offset code.
-  _HTTP3_PR_INVALID_OFFSET = 200,
 };
 
 extern const uint8_t kMaxErrorCode;
