@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -10,11 +10,12 @@
 
 #include <folly/Memory.h>
 #include <folly/Random.h>
+#include <folly/base64.h>
 #include <folly/ssl/OpenSSLHash.h>
 #include <proxygen/lib/http/HTTPHeaderSize.h>
 #include <proxygen/lib/http/RFC2616.h>
 #include <proxygen/lib/http/codec/CodecProtocol.h>
-#include <proxygen/lib/utils/Base64.h>
+#include <proxygen/lib/http/codec/CodecUtil.h>
 
 using folly::IOBuf;
 using folly::IOBufQueue;
@@ -76,7 +77,9 @@ void appendString(IOBufQueue& queue, size_t& len, StringPiece str) {
 
 namespace proxygen {
 
-HTTP1xCodec::HTTP1xCodec(TransportDirection direction, bool force1_1)
+HTTP1xCodec::HTTP1xCodec(TransportDirection direction,
+                         bool force1_1,
+                         bool strictValidation)
     : callback_(nullptr),
       ingressTxnID_(0),
       egressTxnID_(0),
@@ -85,6 +88,7 @@ HTTP1xCodec::HTTP1xCodec(TransportDirection direction, bool force1_1)
       transportDirection_(direction),
       keepaliveRequested_(KeepaliveRequested::UNSET),
       force1_1_(force1_1),
+      strictValidation_(strictValidation),
       parserActive_(false),
       pendingEOF_(false),
       parserPaused_(false),
@@ -170,6 +174,31 @@ const http_parser_settings* HTTP1xCodec::getParserSettings() {
 }
 
 size_t HTTP1xCodec::onIngress(const IOBuf& buf) {
+  folly::io::Cursor cursor(&buf);
+
+  size_t totalBytesParsed = 0;
+  while (!cursor.isAtEnd()) {
+    folly::IOBuf currentReadBuf;
+    auto bufSize = cursor.peekBytes().size();
+    cursor.cloneAtMost(currentReadBuf, bufSize);
+    size_t bytesParsed = onIngressImpl(currentReadBuf);
+    if (bytesParsed == 0) {
+      // If the codec didn't make any progress with current input, we
+      // wait for more data until this is called again
+      break;
+    }
+    totalBytesParsed += bytesParsed;
+    if (bufSize > bytesParsed) {
+      cursor.retreat(bufSize - bytesParsed);
+    }
+    if (isParserPaused()) {
+      break;
+    }
+  }
+  return totalBytesParsed;
+}
+
+size_t HTTP1xCodec::onIngressImpl(const IOBuf& buf) {
   if (parserError_) {
     return 0;
   } else if (ingressUpgradeComplete_) {
@@ -190,10 +219,14 @@ size_t HTTP1xCodec::onIngress(const IOBuf& buf) {
       onHeadersComplete(0);
       parserActive_ = false;
       ingressUpgradeComplete_ = true;
-      return onIngress(buf);
+      return onIngressImpl(buf);
     }
-    size_t bytesParsed = http_parser_execute(
-        &parser_, getParserSettings(), (const char*)buf.data(), buf.length());
+    size_t bytesParsed = http_parser_execute_options(
+        &parser_,
+        getParserSettings(),
+        strictValidation_ ? F_HTTP_PARSER_OPTIONS_URL_STRICT : 0,
+        (const char*)buf.data(),
+        buf.length());
     // in case we parsed a section of the headers but we're not done parsing
     // the headers we need to keep accounting of it for total header size
     if (!headersComplete_) {
@@ -272,7 +305,10 @@ void HTTP1xCodec::onParserError(const char* what) {
   } else if (parser_errno == HPE_HEADER_OVERFLOW ||
              parser_errno == HPE_INVALID_CONSTANT ||
              (parser_errno >= HPE_INVALID_VERSION &&
-              parser_errno <= HPE_HUGE_CONTENT_LENGTH)) {
+              parser_errno <= HPE_HUGE_CONTENT_LENGTH) ||
+             parser_errno == HPE_CB_header_field ||
+             parser_errno == HPE_CB_header_value ||
+             parser_errno == HPE_CB_headers_complete) {
     error.setProxygenError(kErrorParseHeader);
   } else if (parser_errno == HPE_INVALID_CHUNK_SIZE ||
              parser_errno == HPE_HUGE_CHUNK_SIZE) {
@@ -307,7 +343,7 @@ constexpr folly::StringPiece kWSMagicString =
 std::string HTTP1xCodec::generateWebsocketKey() const {
   std::array<unsigned char, 16> arr;
   folly::Random::secureRandom(arr.data(), arr.size());
-  return Base64::encode(folly::ByteRange(arr.data(), arr.size()));
+  return folly::base64Encode(std::string_view((char*)arr.data(), arr.size()));
 }
 
 std::string HTTP1xCodec::generateWebsocketAccept(const std::string& key) const {
@@ -318,7 +354,8 @@ std::string HTTP1xCodec::generateWebsocketAccept(const std::string& key) const {
   std::array<unsigned char, 20> arr;
   folly::MutableByteRange accept(arr.data(), arr.size());
   digest.hash_final(accept);
-  return Base64::encode(accept);
+  return folly::base64Encode(
+      std::string_view((char*)accept.data(), accept.size()));
 }
 
 void HTTP1xCodec::serializeWebsocketHeader(IOBufQueue& writeBuf,
@@ -347,11 +384,13 @@ void HTTP1xCodec::serializeWebsocketHeader(IOBufQueue& writeBuf,
   }
 }
 
-void HTTP1xCodec::generateHeader(IOBufQueue& writeBuf,
-                                 StreamID txn,
-                                 const HTTPMessage& msg,
-                                 bool eom,
-                                 HTTPHeaderSize* size) {
+void HTTP1xCodec::generateHeader(
+    IOBufQueue& writeBuf,
+    StreamID txn,
+    const HTTPMessage& msg,
+    bool eom,
+    HTTPHeaderSize* size,
+    const folly::Optional<HTTPHeaders>& extraHeaders) {
   if (keepalive_ && disableKeepalivePending_) {
     keepalive_ = false;
   }
@@ -487,7 +526,7 @@ void HTTP1xCodec::generateHeader(IOBufQueue& writeBuf,
     parser_.http_minor = 9;
     return;
   }
-  const string* deferredContentLength = nullptr;
+  folly::StringPiece deferredContentLength;
   bool hasTransferEncodingChunked = false;
   bool hasDateHeader = false;
   bool hasUpgradeHeader = false;
@@ -495,77 +534,85 @@ void HTTP1xCodec::generateHeader(IOBufQueue& writeBuf,
   size_t lastConnectionToken = 0;
   bool egressWebsocketUpgrade = msg.isEgressWebsocketUpgrade();
   bool hasUpgradeTokeninConnection = false;
-  msg.getHeaders().forEachWithCode(
-      [&](HTTPHeaderCode code, const string& header, const string& value) {
-        if (code == HTTP_HEADER_CONTENT_LENGTH) {
-          // Write the Content-Length last (t1071703)
-          deferredContentLength = &value;
-          return; // continue
-        } else if (code == HTTP_HEADER_CONNECTION &&
-                   (!is1xxResponse_ || egressWebsocketUpgrade)) {
-          static const string kClose = "close";
-          static const string kKeepAlive = "keep-alive";
-          folly::split(',', value, connectionTokens);
-          for (auto curConnectionToken = lastConnectionToken;
-               curConnectionToken < connectionTokens.size();
-               curConnectionToken++) {
-            auto token = trimWhitespace(connectionTokens[curConnectionToken]);
-            if (caseInsensitiveEqual(token, "upgrade")) {
-              hasUpgradeTokeninConnection = true;
-            }
-            if (caseInsensitiveEqual(token, kClose)) {
-              keepalive_ = false;
-            } else if (!caseInsensitiveEqual(token, kKeepAlive)) {
-              connectionTokens[lastConnectionToken++] = token;
-            } // else eat the keep-alive token
-          }
-          connectionTokens.resize(lastConnectionToken);
-          // We'll generate a new Connection header based on the keepalive_
-          // state
-          return;
-        } else if (code == HTTP_HEADER_UPGRADE && txn == 1) {
-          hasUpgradeHeader = true;
-          if (upstream) {
-            // save in case we get a 101 Switching Protocols
-            upgradeHeader_ = value;
-          }
-        } else if (!hasTransferEncodingChunked &&
-                   code == HTTP_HEADER_TRANSFER_ENCODING) {
-          if (!caseInsensitiveEqual(value, kChunked)) {
-            return;
-          }
-          hasTransferEncodingChunked = true;
-          if (!mayChunkEgress_) {
-            return;
-          }
-        } else if (!hasDateHeader && code == HTTP_HEADER_DATE) {
-          hasDateHeader = true;
-        } else if (egressWebsocketUpgrade &&
-                   code == HTTP_HEADER_SEC_WEBSOCKET_KEY) {
-          // will generate our own key per hop, not client's.
-          return;
-        } else if (egressWebsocketUpgrade &&
-                   code == HTTP_HEADER_SEC_WEBSOCKET_ACCEPT) {
-          // will generate our own accept per hop, not client's.
-          return;
+  auto headerEncoder = [&](HTTPHeaderCode code,
+                           folly::StringPiece header,
+                           folly::StringPiece value) {
+    if (code == HTTP_HEADER_CONTENT_LENGTH) {
+      // Write the Content-Length last (t1071703)
+      deferredContentLength = value;
+      return; // continue
+    } else if (code == HTTP_HEADER_CONNECTION &&
+               (!is1xxResponse_ || egressWebsocketUpgrade)) {
+      static const string kClose = "close";
+      static const string kKeepAlive = "keep-alive";
+      folly::split(',', value, connectionTokens);
+      for (auto curConnectionToken = lastConnectionToken;
+           curConnectionToken < connectionTokens.size();
+           curConnectionToken++) {
+        auto token = trimWhitespace(connectionTokens[curConnectionToken]);
+        if (caseInsensitiveEqual(token, "upgrade")) {
+          hasUpgradeTokeninConnection = true;
         }
-        size_t lineLen =
-            header.length() + value.length() + 4; // 4 for ": " + CRLF
-        auto writable =
-            writeBuf.preallocate(lineLen, std::max(lineLen, size_t(2000)));
-        char* dst = (char*)writable.first;
-        memcpy(dst, header.data(), header.length());
-        dst += header.length();
-        *dst++ = ':';
-        *dst++ = ' ';
-        memcpy(dst, value.data(), value.length());
-        dst += value.length();
-        *dst++ = '\r';
-        *dst = '\n';
-        DCHECK_EQ(size_t(++dst - (char*)writable.first), lineLen);
-        writeBuf.postallocate(lineLen);
-        len += lineLen;
-      });
+        if (caseInsensitiveEqual(token, kClose)) {
+          keepalive_ = false;
+        } else if (!caseInsensitiveEqual(token, kKeepAlive)) {
+          connectionTokens[lastConnectionToken++] = token;
+        } // else eat the keep-alive token
+      }
+      connectionTokens.resize(lastConnectionToken);
+      // We'll generate a new Connection header based on the keepalive_
+      // state
+      return;
+    } else if (code == HTTP_HEADER_UPGRADE && txn == 1) {
+      hasUpgradeHeader = true;
+      if (upstream) {
+        // save in case we get a 101 Switching Protocols
+        upgradeHeader_ = value.str();
+      }
+    } else if (!hasTransferEncodingChunked &&
+               code == HTTP_HEADER_TRANSFER_ENCODING) {
+      if (!caseInsensitiveEqual(value, kChunked)) {
+        return;
+      }
+      hasTransferEncodingChunked = true;
+      if (!mayChunkEgress_) {
+        return;
+      }
+    } else if (!hasDateHeader && code == HTTP_HEADER_DATE) {
+      hasDateHeader = true;
+    } else if (egressWebsocketUpgrade &&
+               code == HTTP_HEADER_SEC_WEBSOCKET_KEY) {
+      // will generate our own key per hop, not client's.
+      return;
+    } else if (egressWebsocketUpgrade &&
+               code == HTTP_HEADER_SEC_WEBSOCKET_ACCEPT) {
+      // will generate our own accept per hop, not client's.
+      return;
+    }
+    if (value.find_first_of("\r\n") != std::string::npos) {
+      return;
+    }
+    size_t lineLen = header.size() + value.size() + 4; // 4 for ": " + CRLF
+    auto writable =
+        writeBuf.preallocate(lineLen, std::max(lineLen, size_t(2000)));
+    char* dst = (char*)writable.first;
+    memcpy(dst, header.data(), header.size());
+    dst += header.size();
+    *dst++ = ':';
+    *dst++ = ' ';
+    memcpy(dst, value.data(), value.size());
+    dst += value.size();
+    *dst++ = '\r';
+    *dst = '\n';
+    DCHECK_EQ(size_t(++dst - (char*)writable.first), lineLen);
+    writeBuf.postallocate(lineLen);
+    len += lineLen;
+  };
+  msg.getHeaders().forEachWithCode(headerEncoder);
+  if (extraHeaders) {
+    extraHeaders->forEachWithCode(headerEncoder);
+  }
+
   bool bodyCheck =
       (downstream && keepalive_ && !expectNoResponseBody_ && !egressUpgrade_) ||
       // auto chunk POSTs and any request that came to us chunked
@@ -573,7 +620,7 @@ void HTTP1xCodec::generateHeader(IOBufQueue& writeBuf,
   // TODO: 400 a 1.0 POST with no content-length
   // clear egressChunked_ if the header wasn't actually set
   egressChunked_ &= hasTransferEncodingChunked;
-  if (bodyCheck && !egressChunked_ && !deferredContentLength) {
+  if (bodyCheck && !egressChunked_ && deferredContentLength.empty()) {
     // On a connection that would otherwise be eligible for keep-alive,
     // we're being asked to send a response message with no Content-Length,
     // no chunked encoding, and no special circumstances that would eliminate
@@ -629,9 +676,9 @@ void HTTP1xCodec::generateHeader(IOBufQueue& writeBuf,
     appendLiteral(writeBuf, len, "\r\n");
   }
 
-  if (deferredContentLength) {
+  if (!deferredContentLength.empty()) {
     appendLiteral(writeBuf, len, "Content-Length: ");
-    appendString(writeBuf, len, *deferredContentLength);
+    appendString(writeBuf, len, deferredContentLength);
     appendLiteral(writeBuf, len, CRLF);
   }
   appendLiteral(writeBuf, len, CRLF);
@@ -844,7 +891,23 @@ int HTTP1xCodec::onReason(const char* buf, size_t len) {
   return 0;
 }
 
-void HTTP1xCodec::pushHeaderNameAndValue(HTTPHeaders& hdrs) {
+bool HTTP1xCodec::pushHeaderNameAndValue(HTTPHeaders& hdrs) {
+  // Header names are strictly validated by http_parser, however, it allows
+  // quoted+escaped CTLs so run our stricter check here.
+  if (strictValidation_) {
+    folly::StringPiece headerName(currentHeaderName_.empty()
+                                      ? currentHeaderNameStringPiece_
+                                      : currentHeaderName_);
+    bool compatValidate = false;
+    if (!CodecUtil::validateHeaderValue(
+            folly::StringPiece(currentHeaderValue_),
+            compatValidate ? CodecUtil::CtlEscapeMode::STRICT_COMPAT
+                           : CodecUtil::CtlEscapeMode::STRICT)) {
+      LOG(ERROR) << "Invalid header name=" << headerName;
+      std::cerr << " value=" << currentHeaderValue_ << std::endl;
+      return false;
+    }
+  }
   if (LIKELY(currentHeaderName_.empty())) {
     hdrs.addFromCodec(currentHeaderNameStringPiece_.begin(),
                       currentHeaderNameStringPiece_.size(),
@@ -855,16 +918,21 @@ void HTTP1xCodec::pushHeaderNameAndValue(HTTPHeaders& hdrs) {
   }
   currentHeaderNameStringPiece_.clear();
   currentHeaderValue_.clear();
+  return true;
 }
 
 int HTTP1xCodec::onHeaderField(const char* buf, size_t len) {
+  bool valid = true;
   if (headerParseState_ == HeaderParseState::kParsingHeaderValue) {
-    pushHeaderNameAndValue(msg_->getHeaders());
+    valid = pushHeaderNameAndValue(msg_->getHeaders());
   } else if (headerParseState_ == HeaderParseState::kParsingTrailerValue) {
     if (!trailers_) {
       trailers_.reset(new HTTPHeaders());
     }
-    pushHeaderNameAndValue(*trailers_);
+    valid = pushHeaderNameAndValue(*trailers_);
+  }
+  if (!valid) {
+    return -1;
   }
 
   if (isParsingHeaderOrTrailerName()) {
@@ -917,7 +985,9 @@ int HTTP1xCodec::onHeaderValue(const char* buf, size_t len) {
 
 int HTTP1xCodec::onHeadersComplete(size_t len) {
   if (headerParseState_ == HeaderParseState::kParsingHeaderValue) {
-    pushHeaderNameAndValue(msg_->getHeaders());
+    if (!pushHeaderNameAndValue(msg_->getHeaders())) {
+      return -1;
+    }
   }
 
   // discard messages with folded or multiple valued Transfer-Encoding headers
@@ -964,7 +1034,11 @@ int HTTP1xCodec::onHeadersComplete(size_t len) {
     // an entity-body in the response.
     headRequest_ = (msg_->getMethod() == HTTPMethod::HEAD);
 
-    ParseURL parseUrl = msg_->setURL(std::move(url_));
+    ParseURL parseUrl = msg_->setURL(std::move(url_), strictValidation_);
+    if (strictValidation_ && !parseUrl.valid()) {
+      LOG(ERROR) << "Invalid URL: " << msg_->getURL();
+      return -1;
+    }
     url_.clear();
 
     if (parseUrl.hasHost()) {
@@ -1078,8 +1152,19 @@ int HTTP1xCodec::onHeadersComplete(size_t len) {
       }
     } else {
       // request.
+      // If the websockAcceptKey is already set, we error out.
+      // Currently, websockAcceptKey is never cleared, which means
+      // that only one Websocket upgrade attempt can be made on the
+      // connection. If that upgrade request is not successful for any
+      // reason, the connection is no longer usable. At some point, we
+      // may want to change this to clear the websockAcceptKey if
+      // the request doesn't succeed keeping the connection usable.
+      if (!websockAcceptKey_.empty()) {
+        LOG(ERROR) << "ws accept key already set: '" << websockAcceptKey_
+                   << "'";
+        return -1;
+      }
       auto key = hdrs.getSingleOrEmpty(HTTP_HEADER_SEC_WEBSOCKET_KEY);
-      DCHECK(websockAcceptKey_.empty());
       websockAcceptKey_ = generateWebsocketAccept(key);
     }
   }
@@ -1174,7 +1259,9 @@ int HTTP1xCodec::onMessageComplete() {
     if (!trailers_) {
       trailers_.reset(new HTTPHeaders());
     }
-    pushHeaderNameAndValue(*trailers_);
+    if (!pushHeaderNameAndValue(*trailers_)) {
+      return -1;
+    }
   }
 
   headerParseState_ = HeaderParseState::kParsingHeaderIdle;
@@ -1200,6 +1287,10 @@ int HTTP1xCodec::onMessageComplete() {
     }
     case TransportDirection::UPSTREAM:
       responsePending_ = is1xxResponse_;
+      if (is1xxResponse_ && !nativeUpgrade_ && !ingressUpgrade_) {
+        // Some other 1xx status code, which doesn't terminate the message
+        return 0;
+      }
   }
 
   // For downstream, always call onMessageComplete. If native upgrade,
