@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -7,14 +7,15 @@
  */
 
 #include <proxygen/lib/http/codec/HTTP2Codec.h>
+
+#include <folly/base64.h>
 #include <proxygen/lib/http/codec/CodecUtil.h>
 #include <proxygen/lib/http/codec/HTTP2Constants.h>
-#include <proxygen/lib/utils/Base64.h>
 #include <proxygen/lib/utils/Logging.h>
 
 #include <folly/Conv.h>
 #include <folly/Random.h>
-#include <folly/ThreadLocal.h>
+#include <folly/Try.h>
 #include <folly/io/Cursor.h>
 #include <folly/tracing/ScopedTraceSection.h>
 #include <type_traits>
@@ -25,17 +26,9 @@ using namespace folly;
 using std::string;
 
 namespace {
-std::string base64url_encode(ByteRange range) {
-  return proxygen::Base64::urlEncode(range);
-}
-
-std::string base64url_decode(const std::string& str) {
-  return proxygen::Base64::urlDecode(str);
-}
-
 const size_t kDefaultGrowth = 4000;
-constexpr auto kOkhttp2 = "okhttp/2";
-constexpr int kOkhttp2GoawayLogFreq = 1000;
+constexpr auto kOkhttp = "okhttp/";
+constexpr int kOkhttpResetLogFreq = 1000;
 } // namespace
 
 namespace proxygen {
@@ -45,7 +38,7 @@ HTTP2Codec::HTTP2Codec(TransportDirection direction)
       headerCodec_(direction),
       frameState_(direction == TransportDirection::DOWNSTREAM
                       ? FrameState::UPSTREAM_CONNECTION_PREFACE
-                      : FrameState::DOWNSTREAM_CONNECTION_PREFACE) {
+                      : FrameState::EXPECT_FIRST_SETTINGS) {
 
   const auto maxHeaderListSize =
       egressSettings_.getSetting(SettingsId::MAX_HEADER_LIST_SIZE);
@@ -69,10 +62,11 @@ size_t HTTP2Codec::onIngress(const folly::IOBuf& buf) {
   Cursor cursor(&buf);
   size_t parsed = 0;
   ErrorCode connError = ErrorCode::NO_ERROR;
-  for (auto bufLen = cursor.totalLength(); connError == ErrorCode::NO_ERROR;
-       bufLen = cursor.totalLength()) {
+  auto bufLen = cursor.totalLength();
+  while (connError == ErrorCode::NO_ERROR) {
+    size_t remaining = bufLen - parsed;
     if (frameState_ == FrameState::UPSTREAM_CONNECTION_PREFACE) {
-      if (bufLen >= http2::kConnectionPreface.length()) {
+      if (remaining >= http2::kConnectionPreface.length()) {
         auto test = cursor.readFixedString(http2::kConnectionPreface.length());
         parsed += http2::kConnectionPreface.length();
         if (test != http2::kConnectionPreface) {
@@ -80,17 +74,17 @@ size_t HTTP2Codec::onIngress(const folly::IOBuf& buf) {
           VLOG(4) << goawayErrorMessage_;
           connError = ErrorCode::PROTOCOL_ERROR;
         }
-        frameState_ = FrameState::FRAME_HEADER;
+        frameState_ = FrameState::EXPECT_FIRST_SETTINGS;
       } else {
         break;
       }
     } else if (frameState_ == FrameState::FRAME_HEADER ||
-               frameState_ == FrameState::DOWNSTREAM_CONNECTION_PREFACE) {
+               frameState_ == FrameState::EXPECT_FIRST_SETTINGS) {
       // Waiting to parse the common frame header
-      if (bufLen >= http2::kFrameHeaderSize) {
+      if (remaining >= http2::kFrameHeaderSize) {
         connError = parseFrameHeader(cursor, curHeader_);
         parsed += http2::kFrameHeaderSize;
-        if (frameState_ == FrameState::DOWNSTREAM_CONNECTION_PREFACE &&
+        if (frameState_ == FrameState::EXPECT_FIRST_SETTINGS &&
             curHeader_.type != http2::FrameType::SETTINGS) {
           goawayErrorMessage_ = folly::to<string>(
               "GOAWAY error: got invalid connection preface frame type=",
@@ -115,23 +109,25 @@ size_t HTTP2Codec::onIngress(const folly::IOBuf& buf) {
                                    static_cast<uint8_t>(curHeader_.type));
         }
 
-        frameState_ = (curHeader_.type == http2::FrameType::DATA)
-                          ? FrameState::DATA_FRAME_DATA
-                          : FrameState::FRAME_DATA;
-        pendingDataFrameBytes_ = curHeader_.length;
-        pendingDataFramePaddingBytes_ = 0;
+        if (curHeader_.type == http2::FrameType::DATA) {
+          frameState_ = FrameState::DATA_FRAME_DATA;
+          pendingDataFrameBytes_ = curHeader_.length;
+          pendingDataFramePaddingBytes_ = 0;
+        } else {
+          frameState_ = FrameState::FRAME_DATA;
+        }
 #ifndef NDEBUG
         receivedFrameCount_++;
 #endif
       } else {
         break;
       }
-    } else if (frameState_ == FrameState::DATA_FRAME_DATA && bufLen > 0 &&
-               (bufLen < curHeader_.length ||
+    } else if (frameState_ == FrameState::DATA_FRAME_DATA && remaining > 0 &&
+               (remaining < curHeader_.length ||
                 pendingDataFrameBytes_ < curHeader_.length)) {
       // FrameState::DATA_FRAME_DATA with partial data only
       size_t dataParsed = 0;
-      connError = parseDataFrameData(cursor, bufLen, dataParsed);
+      connError = parseDataFrameData(cursor, remaining, dataParsed);
       if (dataParsed == 0 && pendingDataFrameBytes_ > 0) {
         // We received only the padding byte, we will wait for more
         break;
@@ -145,7 +141,7 @@ size_t HTTP2Codec::onIngress(const folly::IOBuf& buf) {
              // or FrameState::DATA_FRAME_DATA with all data available
       // Already parsed the common frame header
       const auto frameLen = curHeader_.length;
-      if (bufLen >= frameLen) {
+      if (remaining >= frameLen) {
         connError = parseFrame(cursor);
         parsed += curHeader_.length;
         frameState_ = FrameState::FRAME_HEADER;
@@ -460,7 +456,7 @@ ErrorCode HTTP2Codec::parseHeadersImpl(
 
   DeferredParseError parseError;
   if (curHeader_.flags & http2::END_HEADERS) {
-    auto parseRes = parseHeadersDecodeFrames(priority);
+    auto parseRes = parseHeadersDecodeFrames(priority, exAttributes);
     if (parseRes.hasError()) {
       parseError = std::move(parseRes.error());
       if (parseError.connectionError) {
@@ -539,19 +535,19 @@ ErrorCode HTTP2Codec::parseHeadersImpl(
 
 folly::Expected<std::unique_ptr<HTTPMessage>, HTTP2Codec::DeferredParseError>
 HTTP2Codec::parseHeadersDecodeFrames(
-    const folly::Optional<http2::PriorityUpdate>& priority) {
+    const folly::Optional<http2::PriorityUpdate>& priority,
+    const folly::Optional<ExAttributes>& exAttributes) {
   // decompress headers
   Cursor headerCursor(curHeaderBlock_.front());
 
-  // Validate circular dependencies.
-  if (priority && (curHeader_.stream == priority->streamDependency)) {
-    return folly::makeUnexpected(DeferredParseError(
-        ErrorCode::PROTOCOL_ERROR,
-        false,
-        folly::to<string>("Circular dependency for txn=", curHeader_.stream)));
-  }
-
-  decodeInfo_.init(parsingReq_, parsingDownstreamTrailers_, validateHeaders_);
+  // DO NOT return from this method until after the call to decodeStreaming
+  // unless you return a connection error.  Otherwise the HPACK state will
+  // get messed up.
+  decodeInfo_.init(parsingReq_,
+                   parsingDownstreamTrailers_,
+                   validateHeaders_,
+                   strictValidation_,
+                   exAttributes && exAttributes->controlStream != 0);
   if (priority) {
     decodeInfo_.msg->setHTTP2Priority(std::make_tuple(
         priority->streamDependency, priority->exclusive, priority->weight));
@@ -569,26 +565,45 @@ HTTP2Codec::parseHeadersDecodeFrames(
     // Avoid logging header blocks that have failed decoding due to being
     // excessively large.
     if (decodeInfo_.decodeError != HPACK::DecodeError::HEADERS_TOO_LARGE) {
-      LOG(ERROR) << decodeErrorMessage << curHeader_.stream << " header block=";
+      goawayErrorMessage_ =
+          folly::to<std::string>(decodeErrorMessage,
+                                 curHeader_.stream,
+                                 ": decompression error=",
+                                 uint32_t(decodeInfo_.decodeError));
+      LOG(ERROR) << goawayErrorMessage_
+                 << (VLOG_IS_ON(3) ? ", header block=" : "");
       VLOG(3) << IOBufPrinter::printHexFolly(curHeaderBlock_.front(), true);
     } else {
-      LOG(ERROR) << decodeErrorMessage << curHeader_.stream;
+      goawayErrorMessage_ = folly::to<std::string>(
+          decodeErrorMessage, curHeader_.stream, ": headers too large");
     }
 
     if (msg) {
       // print the partial message
       msg->dumpMessage(3);
     }
-    return folly::makeUnexpected(
-        DeferredParseError(ErrorCode::COMPRESSION_ERROR, true, empty_string));
+    return folly::makeUnexpected(DeferredParseError(
+        ErrorCode::COMPRESSION_ERROR, true, empty_string, std::move(msg)));
+  }
+
+  // Validate circular dependencies.
+  if (priority && (curHeader_.stream == priority->streamDependency)) {
+    return folly::makeUnexpected(DeferredParseError(
+        ErrorCode::PROTOCOL_ERROR,
+        false,
+        folly::to<string>("Circular dependency for txn=", curHeader_.stream)));
   }
 
   // Check parsing error
-  if (decodeInfo_.parsingError != "") {
+  if (!decodeInfo_.parsingError.empty()) {
     // This is "malformed" per the RFC
     LOG(ERROR) << "Failed parsing header list for stream=" << curHeader_.stream
-               << ", error=" << decodeInfo_.parsingError << ", header block=";
-    VLOG(3) << IOBufPrinter::printHexFolly(curHeaderBlock_.front(), true);
+               << ", error=" << decodeInfo_.parsingError;
+    if (!decodeInfo_.headerErrorValue.empty()) {
+      std::cerr << " value=" << decodeInfo_.headerErrorValue << std::endl;
+    }
+    VLOG(3) << "Header block="
+            << IOBufPrinter::printHexFolly(curHeaderBlock_.front(), true);
     if (transportDirection_ == TransportDirection::DOWNSTREAM &&
         parsingHeaders() && !parsingTrailers()) {
       return folly::makeUnexpected(
@@ -600,7 +615,8 @@ HTTP2Codec::parseHeadersDecodeFrames(
                                                     " status=",
                                                     400,
                                                     " error: ",
-                                                    decodeInfo_.parsingError)));
+                                                    decodeInfo_.parsingError),
+                             std::move(msg)));
     } else {
       // Upstream, PUSH_PROMISE, EX_HEADERS or trailers parsing failed:
       // PROTOCOL_ERROR
@@ -608,7 +624,8 @@ HTTP2Codec::parseHeadersDecodeFrames(
           ErrorCode::PROTOCOL_ERROR,
           false,
           folly::to<string>("Field section parsing failed txn=",
-                            curHeader_.stream)));
+                            curHeader_.stream),
+          std::move(msg)));
     }
   }
 
@@ -619,8 +636,11 @@ void HTTP2Codec::deliverDeferredParseError(
     const DeferredParseError& parseError) {
   CHECK(!parseError.connectionError);
   if (parseError.errorCode != ErrorCode::NO_ERROR) {
-    streamError(
-        parseError.errorMessage, parseError.errorCode, parsingHeaders());
+    streamError(parseError.errorMessage,
+                parseError.errorCode,
+                parsingHeaders(),
+                folly::none,
+                std::move(parseError.partialMessage));
     if (promisedStream_) {
       streamError("Malformed PUSH_PROMISE",
                   ErrorCode::REFUSED_STREAM,
@@ -632,6 +652,8 @@ void HTTP2Codec::deliverDeferredParseError(
     HTTPException err(HTTPException::Direction::INGRESS,
                       parseError.errorMessage);
     err.setHttpStatusCode(400);
+    err.setProxygenError(kErrorParseHeader);
+    err.setPartialMsg(std::move(parseError.partialMessage));
     deliverCallbackIfAllowed(&HTTPCodec::Callback::onError,
                              "onError",
                              curHeader_.stream,
@@ -719,8 +741,13 @@ ErrorCode HTTP2Codec::parsePriority(Cursor& cursor) {
         false);
     return ErrorCode::NO_ERROR;
   }
+  // Now we have two onPriority overloads, this function pointer has to be
+  // explicitly specified via a cast:
+  auto onPriFunc = static_cast<void (HTTPCodec::Callback::*)(
+      StreamID, const HTTPMessage::HTTP2Priority&)>(
+      &HTTPCodec::Callback::onPriority);
   deliverCallbackIfAllowed(
-      &HTTPCodec::Callback::onPriority,
+      onPriFunc,
       "onPriority",
       curHeader_.stream,
       std::make_tuple(pri.streamDependency, pri.exclusive, pri.weight));
@@ -751,17 +778,15 @@ ErrorCode HTTP2Codec::parseRstStream(Cursor& cursor) {
   auto err = http2::parseRstStream(cursor, curHeader_, statusCode);
   RETURN_IF_ERROR(err);
   if (statusCode == ErrorCode::PROTOCOL_ERROR) {
-    goawayErrorMessage_ =
-        folly::to<string>("GOAWAY error: RST_STREAM with code=",
-                          getErrorCodeString(statusCode),
-                          " for streamID=",
-                          curHeader_.stream,
-                          " user-agent=",
-                          userAgent_);
-    int logFreq = userAgent_.find(kOkhttp2) == std::string::npos
-                      ? 1
-                      : kOkhttp2GoawayLogFreq;
-    VLOG_EVERY_N(2, logFreq) << goawayErrorMessage_;
+    auto errorMsg = folly::to<string>("RST_STREAM with code=",
+                                      getErrorCodeString(statusCode),
+                                      " for streamID=",
+                                      curHeader_.stream,
+                                      " user-agent=",
+                                      userAgent_);
+    int logFreq =
+        userAgent_.find(kOkhttp) == std::string::npos ? 1 : kOkhttpResetLogFreq;
+    VLOG_EVERY_N(2, logFreq) << errorMsg;
   }
   deliverCallbackIfAllowed(
       &HTTPCodec::Callback::onAbort, "onAbort", curHeader_.stream, statusCode);
@@ -995,15 +1020,11 @@ ErrorCode HTTP2Codec::parseWindowUpdate(Cursor& cursor) {
     } else {
       // Parsing a zero delta window update should cause a protocol error
       // and send a rst stream
-      goawayErrorMessage_ =
-          folly::to<string>("parseWindowUpdate Invalid 0 length");
+      goawayErrorMessage_ = folly::to<std::string>(
+          "streamID=", curHeader_.stream, " with window update delta=", delta);
       VLOG(4) << goawayErrorMessage_;
-      streamError(folly::to<std::string>("streamID=",
-                                         curHeader_.stream,
-                                         " with HTTP2Codec stream error: ",
-                                         "window update delta=",
-                                         delta),
-                  ErrorCode::PROTOCOL_ERROR);
+      streamError(goawayErrorMessage_, ErrorCode::PROTOCOL_ERROR);
+      // Stream error and protocol error
       return ErrorCode::PROTOCOL_ERROR;
     }
   }
@@ -1052,7 +1073,7 @@ ErrorCode HTTP2Codec::parseCertificate(Cursor& cursor) {
     if (callback_) {
       callback_->onCertificate(certId, std::move(authenticator));
     } else {
-      curAuthenticatorBlock_.clear();
+      curAuthenticatorBlock_.reset();
     }
   }
   return ErrorCode::NO_ERROR;
@@ -1113,7 +1134,9 @@ bool HTTP2Codec::onIngressUpgradeMessage(const HTTPMessage& msg) {
     return true;
   }
 
-  auto decoded = base64url_decode(settingsHeader);
+  auto decoded = folly::makeTryWith([&settingsHeader] {
+                   return folly::base64URLDecode(settingsHeader);
+                 }).value_or(std::string());
 
   // Must be well formed Base64Url and not too large
   if (decoded.empty() || decoded.length() > http2::kMaxFramePayloadLength) {
@@ -1146,18 +1169,21 @@ bool HTTP2Codec::onIngressUpgradeMessage(const HTTPMessage& msg) {
   return true;
 }
 
-void HTTP2Codec::generateHeader(folly::IOBufQueue& writeBuf,
-                                StreamID stream,
-                                const HTTPMessage& msg,
-                                bool eom,
-                                HTTPHeaderSize* size) {
+void HTTP2Codec::generateHeader(
+    folly::IOBufQueue& writeBuf,
+    StreamID stream,
+    const HTTPMessage& msg,
+    bool eom,
+    HTTPHeaderSize* size,
+    const folly::Optional<HTTPHeaders>& extraHeaders) {
   generateHeaderImpl(writeBuf,
                      stream,
                      msg,
                      folly::none, /* assocStream */
                      folly::none, /* controlStream */
                      eom,
-                     size);
+                     size,
+                     extraHeaders);
 }
 
 void HTTP2Codec::generatePushPromise(folly::IOBufQueue& writeBuf,
@@ -1172,7 +1198,8 @@ void HTTP2Codec::generatePushPromise(folly::IOBufQueue& writeBuf,
                      assocStream,
                      folly::none, /* controlStream */
                      eom,
-                     size);
+                     size,
+                     folly::none /* extraHeaders */);
 }
 
 void HTTP2Codec::generateExHeader(folly::IOBufQueue& writeBuf,
@@ -1187,7 +1214,8 @@ void HTTP2Codec::generateExHeader(folly::IOBufQueue& writeBuf,
                      folly::none, /* assocStream */
                      exAttributes,
                      eom,
-                     size);
+                     size,
+                     folly::none /* extraHeaders */);
 }
 
 size_t HTTP2Codec::splitCompressed(size_t compressed,
@@ -1214,7 +1242,8 @@ void HTTP2Codec::generateHeaderImpl(
     const folly::Optional<StreamID>& assocStream,
     const folly::Optional<HTTPCodec::ExAttributes>& exAttributes,
     bool eom,
-    HTTPHeaderSize* size) {
+    HTTPHeaderSize* size,
+    const folly::Optional<HTTPHeaders>& extraHeaders) {
   HTTPHeaderSize localSize;
   if (!size) {
     size = &localSize;
@@ -1271,7 +1300,7 @@ void HTTP2Codec::generateHeaderImpl(
       maxFrameSize - headerSize + http2::kFrameHeaderSize;
   auto frameHeader = writeBuf.preallocate(headerSize, kDefaultGrowth);
   writeBuf.postallocate(headerSize);
-  headerCodec_.encodeHTTP(msg, writeBuf, addDateToResponse_);
+  headerCodec_.encodeHTTP(msg, writeBuf, addDateToResponse_, extraHeaders);
   *size = headerCodec_.getEncodedSize();
 
   IOBufQueue queue(IOBufQueue::cacheChainLength());
@@ -1434,6 +1463,10 @@ size_t HTTP2Codec::generateChunkTerminator(folly::IOBufQueue& /*writeBuf*/,
 size_t HTTP2Codec::generateTrailers(folly::IOBufQueue& writeBuf,
                                     StreamID stream,
                                     const HTTPHeaders& trailers) {
+  if (trailers.size() == 0) {
+    // No point in sending an empty trailer block, convert to EOM.
+    return generateEOM(writeBuf, stream);
+  }
   VLOG(4) << "generating TRAILERS for stream=" << stream;
   std::vector<compress::Header> allHeaders;
   CodecUtil::appendHeaders(trailers, allHeaders, HTTP_HEADER_NONE);
@@ -1508,11 +1541,10 @@ size_t HTTP2Codec::generateRstStream(folly::IOBufQueue& writeBuf,
     VLOG(2) << "sending RST_STREAM with code=" << getErrorCodeString(statusCode)
             << " for stream=" << stream << " user-agent=" << userAgent_;
   }
-  auto code = http2::errorCodeToReset(statusCode);
   return generateHeaderCallbackWrapper(
       stream,
       http2::FrameType::RST_STREAM,
-      http2::writeRstStream(writeBuf, stream, code));
+      http2::writeRstStream(writeBuf, stream, statusCode));
 }
 
 size_t HTTP2Codec::generateGoaway(folly::IOBufQueue& writeBuf,
@@ -1564,11 +1596,11 @@ size_t HTTP2Codec::generateGoaway(folly::IOBufQueue& writeBuf,
             << " user-agent=" << userAgent_;
   }
 
-  auto code = http2::errorCodeToGoaway(statusCode);
   return generateHeaderCallbackWrapper(
       0,
       http2::FrameType::GOAWAY,
-      http2::writeGoaway(writeBuf, lastStream, code, std::move(debugData)));
+      http2::writeGoaway(
+          writeBuf, lastStream, statusCode, std::move(debugData)));
 }
 
 size_t HTTP2Codec::generatePingRequest(folly::IOBufQueue& writeBuf,
@@ -1654,20 +1686,16 @@ size_t HTTP2Codec::generateSettings(folly::IOBufQueue& writeBuf) {
 }
 
 void HTTP2Codec::requestUpgrade(HTTPMessage& request) {
-  HTTP2Codec defaultCodec{TransportDirection::UPSTREAM};
-
   auto& headers = request.getHeaders();
   headers.set(HTTP_HEADER_UPGRADE, http2::kProtocolCleartextString);
   bool addUpgrade =
       !request.checkForHeaderToken(HTTP_HEADER_CONNECTION, "Upgrade", false);
   IOBufQueue writeBuf{IOBufQueue::cacheChainLength()};
-  // TODO staatic generateSettings
-  defaultCodec.generateSettings(writeBuf);
+  generateDefaultSettings(writeBuf);
   writeBuf.trimStart(http2::kFrameHeaderSize);
-  auto buf = writeBuf.move();
-  buf->coalesce();
+  auto binarySettings = writeBuf.move()->to<std::string>();
   headers.set(http2::kProtocolSettingsHeader,
-              base64url_encode(folly::ByteRange(buf->data(), buf->length())));
+              folly::base64URLEncode(binarySettings));
   bool addSettings = !request.checkForHeaderToken(
       HTTP_HEADER_CONNECTION, http2::kProtocolSettingsHeader.c_str(), false);
   if (addUpgrade && addSettings) {
@@ -1680,6 +1708,11 @@ void HTTP2Codec::requestUpgrade(HTTPMessage& request) {
   }
 }
 
+size_t HTTP2Codec::generateDefaultSettings(folly::IOBufQueue& writeBuf) {
+  static HTTP2Codec defaultCodec{TransportDirection::UPSTREAM};
+  return defaultCodec.generateSettings(writeBuf);
+}
+
 size_t HTTP2Codec::generateSettingsAck(folly::IOBufQueue& writeBuf) {
   VLOG(4) << getTransportDirectionString(getTransportDirection())
           << " generating settings ack";
@@ -1705,7 +1738,7 @@ size_t HTTP2Codec::generateWindowUpdate(folly::IOBufQueue& writeBuf,
 
 size_t HTTP2Codec::generatePriority(folly::IOBufQueue& writeBuf,
                                     StreamID stream,
-                                    const HTTPMessage::HTTPPriority& pri) {
+                                    const HTTPMessage::HTTP2Priority& pri) {
   VLOG(4) << "generating priority for stream=" << stream;
   if (!isStreamIngressEgressAllowed(stream)) {
     VLOG(2) << "suppressed PRIORITY for stream=" << stream
@@ -1721,6 +1754,12 @@ size_t HTTP2Codec::generatePriority(folly::IOBufQueue& writeBuf,
           {std::get<0>(pri), std::get<1>(pri), std::get<2>(pri)}));
 }
 
+size_t HTTP2Codec::generatePriority(folly::IOBufQueue& /* writeBuf */,
+                                    StreamID /* stream */,
+                                    HTTPPriority /* priority */) {
+  return 0;
+}
+
 size_t HTTP2Codec::generateCertificateRequest(
     folly::IOBufQueue& writeBuf,
     uint16_t requestId,
@@ -1771,9 +1810,13 @@ bool HTTP2Codec::checkConnectionError(ErrorCode err, const folly::IOBuf* buf) {
 void HTTP2Codec::streamError(const std::string& msg,
                              ErrorCode code,
                              bool newTxn,
-                             folly::Optional<HTTPCodec::StreamID> streamId) {
+                             folly::Optional<HTTPCodec::StreamID> streamId,
+                             std::unique_ptr<HTTPMessage> partialMessage) {
   HTTPException error(HTTPException::Direction::INGRESS_AND_EGRESS, msg);
   error.setCodecStatusCode(code);
+  if (partialMessage) {
+    error.setPartialMsg(std::move(partialMessage));
+  }
   deliverCallbackIfAllowed(&HTTPCodec::Callback::onError,
                            "onError",
                            streamId ? *streamId : curHeader_.stream,
