@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -10,6 +10,7 @@
 
 #include <folly/Portability.h>
 #include <folly/io/IOBufQueue.h>
+#include <folly/lang/Assume.h>
 #include <proxygen/lib/http/HTTPException.h>
 #include <proxygen/lib/http/HTTPHeaderSize.h>
 #include <proxygen/lib/http/codec/CodecProtocol.h>
@@ -133,16 +134,6 @@ class HTTPCodec {
                         std::unique_ptr<folly::IOBuf> chain,
                         uint16_t padding) = 0;
 
-    /**
-     * Called when DATA frame with length 0 headers arrives.
-     *
-     * @param stream        The stream ID
-     * @param streamOffset  Stream offset at which the body starts.
-     */
-    virtual void onUnframedBodyStarted(StreamID /* stream */,
-                                       uint64_t /* streamOffset */) {
-    }
-
     /**
      * Called for each HTTP chunk header.
      *
@@ -237,6 +228,16 @@ class HTTPCodec {
         std::unique_ptr<folly::IOBuf> /* debugData */ = nullptr) {
     }
 
+    /**
+     * Called upon receipt of an unknown frame.
+     * @param streamID          stream ID
+     * @param frameType         frame type
+     * @note only used for quic testing
+     */
+    virtual void onUnknownFrame(uint64_t /* streamID */,
+                                uint64_t /* frameType */) {
+    }
+
     /**
      * Called upon receipt of a ping request
      * @param data attached to the ping request
@@ -281,7 +282,21 @@ class HTTPCodec {
      * dynamic priority
      */
     virtual void onPriority(StreamID /* stream */,
-                            const HTTPMessage::HTTPPriority& /* pri */) {
+                            const HTTPMessage::HTTP2Priority& /* pri */) {
+    }
+
+    /**
+     * Experimental: this is the new HTTP Priority draft format of priority
+     * update. This is called when a PRIORITY_UPDATE frame is received.
+     */
+    virtual void onPriority(StreamID, const HTTPPriority& /* pri */) {
+    }
+
+    /**
+     * Experimental: this is the new HTTP Priority draft format of priority
+     * update. This is called when a PUSH_PRIORITY_UPDATE frame is received.
+     */
+    virtual void onPushPriority(StreamID, const HTTPPriority& /* pri */) {
     }
 
     /**
@@ -357,6 +372,28 @@ class HTTPCodec {
   virtual ~HTTPCodec() {
   }
 
+  /**
+   * Maps a stream id to its sequence number using the underlying protocol as
+   * context.
+   */
+  static size_t streamIDToSeqNo(CodecProtocol protocol,
+                                HTTPCodec::StreamID id) {
+    switch (protocol) {
+      case CodecProtocol::HTTP_1_1:
+        DCHECK_NE(id, 0);
+        return id - 1;
+      case CodecProtocol::HTTP_2:
+        return id / 2;
+      case CodecProtocol::HQ:
+      case CodecProtocol::HTTP_3:
+        // This doesn't factor out of order stream arrival...
+        return id / 4;
+      default:
+        LOG(FATAL) << "Unreachable";
+        return std::numeric_limits<size_t>::max();
+    }
+  }
+
   /**
    * Gets both the egress and ingress header table size, bytes stored in header
    * table, and the number of headers stored in the header table
@@ -425,6 +462,11 @@ class HTTPCodec {
    */
   virtual void setParserPaused(bool paused) = 0;
 
+  /**
+   * Return true if the parser is paused
+   */
+  virtual bool isParserPaused() const = 0;
+
   /**
    * Parse ingress data.
    * @param  buf   A single IOBuf of data to parse
@@ -505,15 +547,19 @@ class HTTPCodec {
   /**
    * Write an egress message header.  For pushed streams, you must specify
    * the assocStream.
+   * @param extraHeaders Optional extra headers to be generated togetger with
+   *                     the msg.
    * @retval size the size of the generated message, both the actual size
    *              and the size of the uncompressed data.
    * @return None
    */
-  virtual void generateHeader(folly::IOBufQueue& writeBuf,
-                              StreamID stream,
-                              const HTTPMessage& msg,
-                              bool eom = false,
-                              HTTPHeaderSize* size = nullptr) = 0;
+  virtual void generateHeader(
+      folly::IOBufQueue& writeBuf,
+      StreamID stream,
+      const HTTPMessage& msg,
+      bool eom = false,
+      HTTPHeaderSize* size = nullptr,
+      const folly::Optional<HTTPHeaders>& extraHeaders = folly::none) = 0;
 
   virtual void generatePushPromise(folly::IOBufQueue& /* writeBuf */,
                                    StreamID /* stream */,
@@ -549,6 +595,23 @@ class HTTPCodec {
                               folly::Optional<uint8_t> padding,
                               bool eom) = 0;
 
+  /**
+   * Write egress message body with fixed length. This is used in DSR where the
+   * actual body bytes are held elsewhere and take an express path.
+   *
+   * @param padding Optionally add padding bytes to the body if possible
+   * @param eom implicitly generate the EOM marker with this body frame
+   *
+   * @return number of bytes written
+   */
+  virtual size_t generateBodyDSR(StreamID,
+                                 size_t,
+                                 folly::Optional<uint8_t>,
+                                 bool) {
+    LOG(FATAL) << __func__ << " not supported on this codec";
+    folly::assume_unreachable();
+  }
+
   /**
    * Write a body chunk header, if relevant.
    */
@@ -662,7 +725,27 @@ class HTTPCodec {
    */
   virtual size_t generatePriority(folly::IOBufQueue& /* writeBuf */,
                                   StreamID /* stream */,
-                                  const HTTPMessage::HTTPPriority& /* pri */) {
+                                  const HTTPMessage::HTTP2Priority& /* pri */) {
+    return 0;
+  }
+
+  /**
+   * Generate a PRIORITY_UPDATE frame, according to the new HTTP priority
+   * draft, if supported.
+   */
+  virtual size_t generatePriority(folly::IOBufQueue& /* writeBuf */,
+                                  StreamID /* stream */,
+                                  HTTPPriority /* priority */) {
+    return 0;
+  }
+
+  /**
+   * Generate a PUSH_PRIORITY_UPDATE frame for non push stream, according to
+   * the new HTTP priority draft, if supported.
+   */
+  virtual size_t generatePushPriority(folly::IOBufQueue& /* writeBuf */,
+                                      StreamID /* stream */,
+                                      HTTPPriority /* priority */) {
     return 0;
   }
 
