@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -7,6 +7,7 @@
  */
 
 #include <proxygen/lib/http/codec/compress/QPACKEncoder.h>
+
 #include <proxygen/lib/http/codec/compress/HPACKDecodeBuffer.h>
 
 using std::vector;
@@ -393,6 +394,13 @@ HPACK::DecodeError QPACKEncoder::decodeDecoderStream(
   return err;
 }
 
+HPACK::DecodeError QPACKEncoder::decoderStreamEnd() {
+  if (!decoderIngress_.empty()) {
+    return HPACK::DecodeError::BUFFER_UNDERFLOW;
+  }
+  return HPACK::DecodeError::NONE;
+}
+
 HPACK::DecodeError QPACKEncoder::decodeHeaderAck(HPACKDecodeBuffer& dbuf,
                                                  uint8_t prefixLength,
                                                  bool all) {
