@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -7,12 +7,14 @@
  */
 
 #include <proxygen/lib/http/codec/HTTP1xCodec.h>
+
+#include <folly/String.h>
+#include <folly/base64.h>
 #include <folly/portability/GTest.h>
 #include <proxygen/lib/http/HTTPHeaderSize.h>
 #include <proxygen/lib/http/HTTPMessage.h>
 #include <proxygen/lib/http/codec/test/MockHTTPCodec.h>
 #include <proxygen/lib/http/codec/test/TestUtils.h>
-#include <proxygen/lib/utils/Base64.h>
 
 using namespace proxygen;
 using namespace std;
@@ -77,6 +79,15 @@ unique_ptr<folly::IOBuf> getSimpleRequestData() {
   return folly::IOBuf::copyBuffer(req);
 }
 
+unique_ptr<folly::IOBuf> getSimpleRequestDataQueue() {
+  string req1("GET /yeah HTTP/1.1\nHost: www.facebook.com\n\n");
+  auto buf1 = folly::IOBuf::copyBuffer(req1);
+  string req2("GET /yeah2 HTTP/1.1\nHost: www.facebook.com\n\n");
+  auto buf2 = folly::IOBuf::copyBuffer(req2);
+  buf1->appendToChain(std::move(buf2));
+  return buf1;
+}
+
 TEST(HTTP1xCodecTest, TestSimpleHeaders) {
   HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
   HTTP1xCodecCallback callbacks;
@@ -90,6 +101,32 @@ TEST(HTTP1xCodecTest, TestSimpleHeaders) {
   EXPECT_EQ(callbacks.headerSize.compressed, callbacks.headerSize.uncompressed);
 }
 
+TEST(HTTP1xCodecTest, TestSimpleHeadersQueue) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  HTTP1xCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer = getSimpleRequestDataQueue();
+  codec.onIngress(*buffer);
+  EXPECT_EQ(callbacks.headersComplete, 2);
+  EXPECT_EQ(callbacks.headerSize.compressed, callbacks.headerSize.uncompressed);
+}
+
+TEST(HTTP1xCodecTest, TestSimpleHeadersQueueWithPause) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer = getSimpleRequestDataQueue();
+
+  EXPECT_CALL(callbacks, onMessageComplete(_, _))
+      .Times(1)
+      .WillOnce(InvokeWithoutArgs([&] { codec.setParserPaused(true); }));
+  size_t bytesParsed = codec.onIngress(*buffer);
+  codec.setParserPaused(false);
+  buffer->trimStart(bytesParsed);
+  EXPECT_CALL(callbacks, onMessageComplete(_, _)).Times(1);
+  codec.onIngress(*buffer);
+}
+
 TEST(HTTP1xCodecTest, Test09Req) {
   HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
   HTTP1xCodecCallback callbacks;
@@ -140,6 +177,30 @@ TEST(HTTP1xCodecTest, Test09Resp) {
   EXPECT_EQ(callbacks.messageComplete, 1);
 }
 
+TEST(HTTP1xCodecTest, TestResponseSplit) {
+  HTTP1xCodec downcodec(TransportDirection::DOWNSTREAM);
+  HTTP1xCodec upcodec(TransportDirection::UPSTREAM);
+
+  HTTP1xCodecCallback callbacks;
+  downcodec.setCallback(&callbacks);
+  auto id = upcodec.createStream();
+
+  HTTPMessage req;
+  req.setHTTPVersion(1, 1);
+  req.setMethod("GET");
+  req.setURL("/foo");
+  req.getHeaders().set("InjectHeader", "Value\r\nx: new");
+  folly::IOBufQueue buf;
+  upcodec.generateHeader(buf, id, req);
+  downcodec.onIngress(*buf.front());
+  EXPECT_EQ(callbacks.headersComplete, 1);
+  EXPECT_EQ(callbacks.messageComplete, 1);
+  // make sure that the header is not sent on the wire.
+  // Otherwise it would be parsed as two different headers with HTTP <= 1.1
+  EXPECT_EQ(callbacks.msg_->getHeaders().getSingleOrEmpty("InjectHeader"), "");
+  EXPECT_EQ(callbacks.msg_->getHeaders().getSingleOrEmpty("x"), "");
+}
+
 TEST(HTTP1xCodecTest, TestO9NoVersion) {
   HTTP1xCodec codec(TransportDirection::UPSTREAM);
   HTTPMessage req;
@@ -192,7 +253,7 @@ TEST(HTTP1xCodecTest, TestKeepalive09_10) {
                                 "Connection: close\r\n"
                                 "Content-Length: 0\r\n\r\n")));
   EXPECT_FALSE(codec2.isReusable());
-  buf.move();
+  buf.reset();
 
   HTTP1xCodec codec3(TransportDirection::DOWNSTREAM, true);
   HTTP1xCodecCallback callbacks3;
@@ -212,7 +273,7 @@ TEST(HTTP1xCodecTest, TestKeepalive09_10) {
                                 "Connection: keep-alive\r\n"
                                 "Content-Length: 0\r\n\r\n")));
   EXPECT_TRUE(codec3.isReusable());
-  buf.move();
+  buf.reset();
 
   HTTP1xCodec codec4(TransportDirection::DOWNSTREAM, true);
   HTTP1xCodecCallback callbacks4;
@@ -232,7 +293,7 @@ TEST(HTTP1xCodecTest, TestKeepalive09_10) {
                                 "Date: \r\n"
                                 "Connection: close\r\n\r\n")));
   EXPECT_FALSE(codec4.isReusable());
-  buf.move();
+  buf.reset();
 }
 
 TEST(HTTP1xCodecTest, TestBadHeaders) {
@@ -247,10 +308,64 @@ TEST(HTTP1xCodecTest, TestBadHeaders) {
       .WillOnce(Invoke(
           [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
             EXPECT_EQ(error->getHttpStatusCode(), 400);
+            EXPECT_EQ(error->getProxygenError(), kErrorParseHeader);
           }));
   codec.onIngress(*buffer);
 }
 
+TEST(HTTP1xCodecTest, TestHighAsciiUA) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM,
+                    /*force1_1=*/true,
+                    /*strictValidation=*/true);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer = folly::IOBuf::copyBuffer(
+      string("GET /yeah HTTP/1.1\r\nUser-Agent: ꪶ𝛸ꫂ_𝐹𝛩𝑅𝐶𝛯_𝑉2\r\n\r\n"));
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onError(1, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+            EXPECT_EQ(error->getProxygenError(), kErrorParseHeader);
+          }));
+  codec.onIngress(*buffer);
+}
+
+TEST(HTTP1xCodecTest, TestBadURL) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM,
+                    /*force1_1=*/true,
+                    /*strictValidation=*/true);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer = folly::IOBuf::copyBuffer(string("GET /Ñoño HTTP/1.1\r\n\r\n"));
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onError(1, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+            EXPECT_EQ(error->getProxygenError(), kErrorParseHeader);
+          }));
+  codec.onIngress(*buffer);
+}
+
+TEST(HTTP1xCodecTest, TestUnderscoreAllowedInHost) {
+  // The strict mode of http_parser used to couple strict URL parsing with
+  // disallowing _ in hostnames.  We decoupled those behaviors so this should
+  // pass even in strict mode.
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM,
+                    /*force1_1=*/true,
+                    /*strictValidation=*/true);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer = folly::IOBuf::copyBuffer(
+      "CONNECT face_book.facebook.com:443 HTTP/1.1\r\n\r\n");
+
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onHeadersComplete(1, _));
+  EXPECT_CALL(callbacks, onMessageComplete(1, _));
+  codec.onIngress(*buffer);
+}
+
 TEST(HTTP1xCodecTest, TestHeadRequestChunkedResponse) {
   HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
   HTTP1xCodecCallback callbacks;
@@ -538,7 +653,7 @@ TEST(HTTP1xCodecTest, Test1xxConnectionHeader) {
   resp.setStatusCode(200);
   resp.getHeaders().remove(HTTP_HEADER_CONNECTION);
   resp.getHeaders().add(HTTP_HEADER_CONTENT_LENGTH, "0");
-  writeBuf.move();
+  writeBuf.reset();
   downstream.generateHeader(writeBuf, streamID, resp);
   upstream.onIngress(*writeBuf.front());
   EXPECT_EQ(callbacks.headersComplete, 2);
@@ -564,15 +679,7 @@ TEST(HTTP1xCodecTest, TestChainedBody) {
       "POST /test.php HTTP/1.1\r\nHost: www.test.com\r\n"
       "Content-Length: 10\r\n\r\nabcde"));
   reqQueue.append(folly::IOBuf::copyBuffer("fghij"));
-
-  while (!reqQueue.empty()) {
-    auto processed = codec.onIngress(*reqQueue.front());
-    if (processed == 0) {
-      break;
-    }
-    reqQueue.trimStart(processed);
-  }
-
+  codec.onIngress(*reqQueue.front());
   EXPECT_TRUE(folly::IOBufEqualTo()(*bodyQueue.front(),
                                     *folly::IOBuf::copyBuffer("abcdefghij")));
 }
@@ -624,7 +731,7 @@ TEST(HTTP1xCodecTest, WebsocketUpgrade) {
   resp.setHTTPVersion(1, 1);
   resp.setStatusCode(101);
   resp.setEgressWebsocketUpgrade();
-  buf.clear();
+  buf.reset();
   downstreamCodec.generateHeader(buf, streamID, resp);
   upstreamCodec.onIngress(*buf.front());
   EXPECT_EQ(upstreamCallbacks.headersComplete, 1);
@@ -700,12 +807,8 @@ TEST(HTTP1xCodecTest, WebsocketConnectionHeader) {
   EXPECT_NE(ws_sec_key, empty_string);
   EXPECT_NE(ws_sec_key, "key should change");
 
-  // We know the key is length 16
-  // https://tools.ietf.org/html/rfc6455#section-4.2.1.5
-  // for base64 % 3 leaves 1 byte so we expect padding of '=='
-  // hence this offset of 2 explicitly
-  EXPECT_NO_THROW(Base64::decode(ws_sec_key, 2));
-  EXPECT_EQ(16, Base64::decode(ws_sec_key, 2).length());
+  EXPECT_NO_THROW(folly::base64Decode(ws_sec_key));
+  EXPECT_EQ(16, folly::base64Decode(ws_sec_key).length());
 
   EXPECT_EQ(headers.getSingleOrEmpty(HTTP_HEADER_SEC_WEBSOCKET_ACCEPT),
             empty_string);
@@ -893,7 +996,7 @@ TEST(HTTP1xCodecTest, CloseOnEgressCompleteUpstreamConnect) {
   codec.generateHeader(writeBuf, id, req, false);
   EXPECT_FALSE(codec.closeOnEgressComplete());
   codec.generateEOM(writeBuf, id);
-  EXPECT_FALSE(codec.closeOnEgressComplete());
+  EXPECT_TRUE(codec.closeOnEgressComplete());
   auto ingress = folly::IOBuf::copyBuffer(std::string("HTTP/1.1 200 OK\r\r\n"));
   HTTP1xCodecCallback callbacks;
   codec.setCallback(&callbacks);
@@ -926,6 +1029,453 @@ TEST(HTTP1xCodecTest, CloseOnEgressCompleteDownstreamResponse) {
   EXPECT_TRUE(codec.closeOnEgressComplete());
 }
 
+TEST(HTTP1xCodecTest, GenerateExtraHeaders) {
+  HTTP1xCodec upstream(TransportDirection::UPSTREAM);
+  HTTP1xCodec downstream(TransportDirection::DOWNSTREAM);
+  HTTP1xCodecCallback callbacks;
+  upstream.setCallback(&callbacks);
+
+  HTTPMessage resp;
+  resp.setHTTPVersion(1, 1);
+  resp.setStatusCode(200);
+  resp.getHeaders().set(HTTP_HEADER_CONTENT_LENGTH, "1000");
+
+  folly::IOBufQueue writeBuf(folly::IOBufQueue::cacheChainLength());
+  auto id = downstream.createStream();
+  HTTPHeaders extraHeaders;
+  extraHeaders.add(HTTP_HEADER_PRIORITY, "u=1");
+  downstream.generateHeader(writeBuf,
+                            id,
+                            resp,
+                            false,
+                            nullptr /* HTTPHeaderSize */,
+                            std::move(extraHeaders));
+
+  upstream.onIngress(*writeBuf.front());
+  EXPECT_EQ(callbacks.headersComplete, 1);
+  EXPECT_EQ(
+      "u=1",
+      callbacks.msg_->getHeaders().getSingleOrEmpty(HTTP_HEADER_PRIORITY));
+  EXPECT_EQ("1000",
+            callbacks.msg_->getHeaders().getSingleOrEmpty(
+                HTTP_HEADER_CONTENT_LENGTH));
+}
+
+TEST(HTTP1xCodecTest, WebsocketUpgradeDuplicate) {
+  HTTP1xCodec downstreamCodec(TransportDirection::DOWNSTREAM);
+  MockHTTPCodecCallback downStreamCallbacks;
+  downstreamCodec.setCallback(&downStreamCallbacks);
+
+  auto reqBuf = folly::IOBuf::copyBuffer(
+      "GET / HTTP/1.1\r\nConnection:\r\nUpgrade:websocket\r\n\r\n");
+  EXPECT_CALL(downStreamCallbacks, onError(_, _, _)).Times(1);
+
+  EXPECT_NO_THROW(downstreamCodec.onIngress(*reqBuf));
+  EXPECT_NO_THROW(downstreamCodec.onIngress(*reqBuf));
+}
+
+TEST(HTTP1xCodecTest, UpgradeHeaderCaseInsensitive) {
+  auto result = checkForProtocolUpgrade("h2c,WebSocket", "websocket", false);
+  ASSERT_TRUE(result.has_value());
+  // We always return the server string so compare against that.
+  EXPECT_EQ("websocket", result->second);
+}
+
+TEST(HTTP1xCodecTest, HugeURL) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  std::string request = folly::to<std::string>("GET /echo?q=",
+                                               std::string(84 * 1024, 'a'),
+                                               " HTTP/1.1\r\n"
+                                               "Host: foo\r\n\r\n");
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onError(1, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+          }));
+  folly::IOBufQueue input{folly::IOBufQueue::cacheChainLength()};
+  input.append(folly::IOBuf::wrapBuffer(request.data(), request.size()));
+  // TODO: seems we only check size when we're unfinished at the end of an IOBuf
+  while (input.chainLength() > 0) {
+    auto head = input.splitAtMost(4096);
+    codec.onIngress(*head);
+  }
+}
+
+TEST(HTTP1xCodecTest, UTF8Chars) {
+  // Non-ascii characters should be pct-encoded according to RFC3986. S220852
+  // was caused partly due to UTF-8 characters found in the URL.
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM,
+                    /*force1_1=*/true,
+                    /*strictValidation=*/true);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto utf8_string = std::string("Ñoño");
+  auto badRequest =
+      folly::to<std::string>("GET /echo?dl=", utf8_string, " HTTP/1.1\r\n\r\n");
+  std::string uriEscapedStr;
+  folly::uriEscape(utf8_string, uriEscapedStr);
+  auto goodRequest = folly::to<std::string>(
+      "GET /echo?dl=", uriEscapedStr, " HTTP/1.1\r\n\r\n");
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onHeadersComplete(1, _));
+  EXPECT_CALL(callbacks, onMessageComplete(1, _));
+  codec.onIngress(
+      *folly::IOBuf::wrapBuffer(goodRequest.data(), goodRequest.size()));
+
+  EXPECT_CALL(callbacks, onMessageBegin(2, _));
+  EXPECT_CALL(callbacks, onError(2, _, _));
+
+  codec.onIngress(
+      *folly::IOBuf::wrapBuffer(badRequest.data(), badRequest.size()));
+}
+
+TEST(HTTP1xCodecTest, ExtraCRLF) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  std::string requests = folly::to<std::string>(
+      "\r"
+      "GET /echo HTTP/1.1\r\n\r\n",
+      "\n"
+      "GET /echo HTTP/1.1\r\n\r\n",
+      "\r\n"
+      "GET /echo HTTP/1.1\r\n\r\n",
+      "\r\n\r\n"
+      "GET /echo HTTP/1.1\r\n\r\n",
+      "\r\n\r\n");
+  EXPECT_CALL(callbacks, onMessageBegin(_, _)).Times(4);
+  EXPECT_CALL(callbacks, onHeadersComplete(_, _)).Times(4);
+  EXPECT_CALL(callbacks, onMessageComplete(_, _)).Times(4);
+  codec.onIngress(*folly::IOBuf::wrapBuffer(requests.data(), requests.size()));
+}
+
+TEST(HTTP1xCodecTest, ContentLengthLast) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  HTTP1xCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  std::string request = "GET /echo HTTP/1.1\r\n\r\n";
+  codec.onIngress(*folly::IOBuf::wrapBuffer(request.data(), request.size()));
+  HTTPCodec::StreamID id = 1;
+  HTTPMessage resp;
+  resp.setStatusCode(200);
+  resp.getHeaders().set(HTTP_HEADER_CONTENT_LENGTH, "100");
+  resp.getHeaders().set("a", "b");
+  resp.getHeaders().set("z", "x");
+  folly::IOBufQueue buf{folly::IOBufQueue::cacheChainLength()};
+  codec.generateHeader(buf, id, resp, true);
+  auto length = buf.chainLength();
+  std::string clHeader = "Content-Length: 100";
+  auto expectedPos = length - clHeader.size() - 4 /* \r\n\r\n */;
+  EXPECT_EQ(buf.move()->moveToFbString().find(clHeader), expectedPos);
+}
+
+TEST(HTTP1xCodecTest, HugeChunkLength) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  std::string request = folly::to<std::string>(
+      "POST /echo HTTP/1.1\r\n"
+      "Transfer-Encoding: chunked\r\n"
+      "\r\n"
+      "ffffffffffffffff\r\n");
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onError(1, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            // TODO: It would probably be nicer to return a 413 code.
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+          }));
+  codec.onIngress(*folly::IOBuf::wrapBuffer(request.data(), request.size()));
+}
+
+TEST(HTTP1xCodecTest, HugeContentLength) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  std::string request = folly::to<std::string>(
+      "POST /echo HTTP/1.1\r\n"
+      "Content-Length: ffffffffffffffff\r\n");
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onError(1, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            // TODO: It would probably be nicer to return a 413 code.
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+          }));
+  codec.onIngress(*folly::IOBuf::wrapBuffer(request.data(), request.size()));
+}
+
+TEST(HTTP1xCodecTest, Dechunk) {
+  // Send a 1.0 request and a 1.1 chunked response.  The codec should dechunk
+  HTTP1xCodec upCodec(TransportDirection::UPSTREAM, false);
+  folly::IOBufQueue buf{folly::IOBufQueue::cacheChainLength()};
+  HTTPMessage req;
+  req.setHTTPVersion(1, 0);
+  req.setURL("/");
+  req.setMethod(HTTPMethod::GET);
+  HTTPCodec::StreamID id = upCodec.createStream();
+  upCodec.generateHeader(buf, id, req, false);
+
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM);
+  HTTP1xCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  codec.onIngress(*buf.front());
+  buf.reset();
+
+  HTTPMessage resp;
+  resp.setStatusCode(200);
+  resp.setHTTPVersion(1, 1);
+  resp.setIsChunked(true);
+  resp.getHeaders().add(HTTP_HEADER_TRANSFER_ENCODING, "chunked");
+  std::string body(10, 'a');
+  codec.generateHeader(buf, id, resp, false);
+  codec.generateChunkHeader(buf, id, 1);
+  codec.generateBody(buf,
+                     id,
+                     folly::IOBuf::wrapBuffer(body.data(), body.size()),
+                     folly::none,
+                     false);
+  codec.generateChunkTerminator(buf, id);
+  codec.generateEOM(buf, id);
+
+  StrictMock<MockHTTPCodecCallback> upCallbacks;
+  upCodec.setCallback(&upCallbacks);
+  // No chunk header callbacks
+  EXPECT_CALL(upCallbacks, onMessageBegin(1, _));
+  EXPECT_CALL(upCallbacks, onHeadersComplete(1, _))
+      .WillOnce(Invoke([](HTTPCodec::StreamID,
+                          std::shared_ptr<HTTPMessage> resp) {
+        EXPECT_FALSE(resp->getHeaders().exists(HTTP_HEADER_TRANSFER_ENCODING));
+        EXPECT_EQ(resp->getHeaders().getSingleOrEmpty(HTTP_HEADER_CONNECTION),
+                  "close");
+      }));
+  EXPECT_CALL(upCallbacks, onBody(1, _, _));
+  upCodec.onIngress(*buf.front());
+  EXPECT_CALL(upCallbacks, onMessageComplete(1, _));
+  upCodec.onIngressEOF();
+}
+
+TEST(HTTP1xCodecTest, Chunkify) {
+  // Send a 1.1 request and a 1.0 non-chunked response.  The codec should chunk
+  HTTP1xCodec upCodec(TransportDirection::UPSTREAM, false);
+  folly::IOBufQueue buf{folly::IOBufQueue::cacheChainLength()};
+  HTTPMessage req;
+  req.setHTTPVersion(1, 1);
+  req.setURL("/");
+  req.setMethod(HTTPMethod::GET);
+  HTTPCodec::StreamID id = upCodec.createStream();
+  upCodec.generateHeader(buf, id, req, false);
+
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM, /*force11*/ true);
+  HTTP1xCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  codec.onIngress(*buf.front());
+  buf.reset();
+
+  HTTPMessage resp;
+  resp.setStatusCode(200);
+  resp.setHTTPVersion(1, 0);
+  resp.setIsChunked(false);
+  std::string body(10, 'a');
+  codec.generateHeader(buf, id, resp, false);
+  codec.generateBody(buf,
+                     id,
+                     folly::IOBuf::wrapBuffer(body.data(), body.size()),
+                     folly::none,
+                     false);
+  codec.generateEOM(buf, id);
+
+  StrictMock<MockHTTPCodecCallback> upCallbacks;
+  upCodec.setCallback(&upCallbacks);
+  // No chunk header callbacks
+  EXPECT_CALL(upCallbacks, onMessageBegin(1, _));
+  EXPECT_CALL(upCallbacks, onHeadersComplete(1, _))
+      .WillOnce(Invoke([](HTTPCodec::StreamID,
+                          std::shared_ptr<HTTPMessage> resp) {
+        EXPECT_TRUE(resp->getHeaders().exists(HTTP_HEADER_TRANSFER_ENCODING));
+        EXPECT_EQ(resp->getHeaders().getSingleOrEmpty(HTTP_HEADER_CONNECTION),
+                  "keep-alive");
+      }));
+  EXPECT_CALL(upCallbacks, onChunkHeader(1, _));
+  EXPECT_CALL(upCallbacks, onBody(1, _, _));
+  EXPECT_CALL(upCallbacks, onChunkComplete(1));
+  EXPECT_CALL(upCallbacks, onMessageComplete(1, _));
+  upCodec.onIngress(*buf.front());
+}
+
+TEST(HTTP1xCodecTest, Chunkify100) {
+  // Send a 1.1 request and a 1.0 non-chunked response.  The codec should chunk
+  HTTP1xCodec upCodec(TransportDirection::UPSTREAM, false);
+  folly::IOBufQueue buf{folly::IOBufQueue::cacheChainLength()};
+  HTTPMessage req;
+  req.setHTTPVersion(1, 1);
+  req.setURL("/");
+  req.setMethod(HTTPMethod::GET);
+  req.getHeaders().add(HTTP_HEADER_EXPECT, "100-continue");
+  HTTPCodec::StreamID id = upCodec.createStream();
+  upCodec.generateHeader(buf, id, req, false);
+
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM, /*force11*/ true);
+  HTTP1xCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  codec.onIngress(*buf.front());
+  buf.reset();
+
+  HTTPMessage resp;
+  resp.setStatusCode(100);
+  resp.setIsChunked(false);
+  codec.generateHeader(buf, id, resp, false);
+
+  resp.setStatusCode(200);
+  resp.setHTTPVersion(1, 0);
+  resp.setIsChunked(false);
+  std::string body(10, 'a');
+  codec.generateHeader(buf, id, resp, false);
+  codec.generateBody(buf,
+                     id,
+                     folly::IOBuf::wrapBuffer(body.data(), body.size()),
+                     folly::none,
+                     false);
+  codec.generateEOM(buf, id);
+
+  StrictMock<MockHTTPCodecCallback> upCallbacks;
+  upCodec.setCallback(&upCallbacks);
+  // No chunk header callbacks
+  EXPECT_CALL(upCallbacks, onMessageBegin(1, _)).Times(2);
+  EXPECT_CALL(upCallbacks, onHeadersComplete(1, _))
+      .WillOnce(
+          Invoke([](HTTPCodec::StreamID, std::shared_ptr<HTTPMessage> resp) {
+            EXPECT_EQ(resp->getStatusCode(), 100);
+            EXPECT_FALSE(resp->getHeaders().exists(HTTP_HEADER_CONNECTION));
+          }))
+      .WillOnce(Invoke([](HTTPCodec::StreamID,
+                          std::shared_ptr<HTTPMessage> resp) {
+        EXPECT_TRUE(resp->getHeaders().exists(HTTP_HEADER_TRANSFER_ENCODING));
+        EXPECT_EQ(resp->getHeaders().getSingleOrEmpty(HTTP_HEADER_CONNECTION),
+                  "keep-alive");
+      }));
+  EXPECT_CALL(upCallbacks, onChunkHeader(1, _));
+  EXPECT_CALL(upCallbacks, onBody(1, _, _));
+  EXPECT_CALL(upCallbacks, onChunkComplete(1));
+  EXPECT_CALL(upCallbacks, onMessageComplete(1, _));
+  upCodec.onIngress(*buf.front());
+}
+
+TEST(HTTP1xCodecTest, TrailersNonChunked) {
+  HTTP1xCodec codec(TransportDirection::UPSTREAM);
+  StrictMock<MockHTTPCodecCallback> callbacks;
+  codec.setCallback(&callbacks);
+  HTTPMessage req;
+  auto id = codec.createStream();
+  req.setHTTPVersion(1, 1);
+  req.setMethod(HTTPMethod::GET);
+  req.setURL("/");
+  codec.setCallback(&callbacks);
+  folly::IOBufQueue buf;
+  codec.generateHeader(buf, id, req, true);
+
+  std::string response = folly::to<std::string>(
+      "HTTP/1.1 200 Ok\r\n"
+      "Content-Length: 10\r\n"
+      "\r\n"
+      "aaaaaaaaaa0\r\n"
+      "Trailer-1: trailer\r\n\r\n");
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onHeadersComplete(1, _));
+  EXPECT_CALL(callbacks, onBody(1, _, _));
+  EXPECT_CALL(callbacks, onMessageComplete(1, _));
+  // Trailers interpreted as another message, and error
+  EXPECT_CALL(callbacks, onMessageBegin(2, _));
+  EXPECT_CALL(callbacks, onError(2, _, _));
+
+  codec.onIngress(*folly::IOBuf::wrapBuffer(response.data(), response.size()));
+}
+
+TEST(HTTP1xCodecTest, HeaderCtls) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM,
+                    /*force1_1=*/true,
+                    /*strictValidation=*/false);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer =
+      folly::IOBuf::copyBuffer(string("GET / HTTP/1.1\r\n"
+                                      "Foo: \"\\\r\\\n\"\r\n"
+                                      "\r\n"));
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onHeadersComplete(1, _))
+      .WillOnce(Invoke([](HTTPCodec::StreamID, std::shared_ptr<HTTPMessage> m) {
+        EXPECT_EQ(m->getHeaders().getSingleOrEmpty("Foo"), "\"\\\r\\\n\"");
+      }));
+  EXPECT_CALL(callbacks, onMessageComplete(1, _));
+  codec.onIngress(*buffer);
+
+  codec.setStrictValidation(true);
+  EXPECT_CALL(callbacks, onMessageBegin(2, _));
+  EXPECT_CALL(callbacks, onError(2, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+          }));
+  codec.onIngress(*buffer);
+}
+
+TEST(HTTP1xCodecTest, HeaderCtlsMiddle) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM,
+                    /*force1_1=*/true,
+                    /*strictValidation=*/true);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer =
+      folly::IOBuf::copyBuffer(string("GET / HTTP/1.1\r\n"
+                                      "Foo: \"\\\r\\\n\"\r\n"
+                                      "Bar: b\r\n"
+                                      "\r\n"));
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onError(1, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+          }));
+  codec.onIngress(*buffer);
+}
+
+TEST(HTTP1xCodecTest, TrailerCtls) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM,
+                    /*force1_1=*/true,
+                    /*strictValidation=*/true);
+  MockHTTPCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer =
+      folly::IOBuf::copyBuffer(string("GET / HTTP/1.1\r\n"
+                                      "Transfer-Encoding: chunked\r\n"
+                                      "\r\n"
+                                      "0\r\n"
+                                      "Foo: \"\\\r\\\n\"\r\n"
+                                      "\r\n"));
+  EXPECT_CALL(callbacks, onMessageBegin(1, _));
+  EXPECT_CALL(callbacks, onHeadersComplete(1, _));
+  EXPECT_CALL(callbacks, onError(1, _, _))
+      .WillOnce(Invoke(
+          [&](HTTPCodec::StreamID, std::shared_ptr<HTTPException> error, bool) {
+            EXPECT_EQ(error->getHttpStatusCode(), 400);
+          }));
+  codec.onIngress(*buffer);
+}
+
+TEST(HTTP1xCodecTest, AbsoluteURLNoPath) {
+  HTTP1xCodec codec(TransportDirection::DOWNSTREAM, true);
+  HTTP1xCodecCallback callbacks;
+  codec.setCallback(&callbacks);
+  auto buffer = folly::IOBuf::copyBuffer(
+      string("GET http://www.foo.com HTTP/1.0\r\n\r\n"));
+  codec.onIngress(*buffer);
+  EXPECT_EQ(callbacks.headersComplete, 1);
+  EXPECT_EQ(callbacks.msg_->getPathAsStringPiece(), string("/"));
+}
+
 class ConnectionHeaderTest
     : public TestWithParam<std::pair<std::list<string>, string>> {
  public:
@@ -952,7 +1502,7 @@ TEST_P(ConnectionHeaderTest, TestConnectionHeaders) {
   EXPECT_EQ(headers.getSingleOrEmpty(HTTP_HEADER_CONNECTION), val.second);
 }
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     HTTP1xCodec,
     ConnectionHeaderTest,
     ::testing::Values(
