@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -7,6 +7,7 @@
  */
 
 #include <proxygen/lib/http/codec/HTTP2Codec.h>
+
 #include <folly/io/Cursor.h>
 #include <proxygen/lib/http/HTTPHeaderSize.h>
 #include <proxygen/lib/http/HTTPMessage.h>
@@ -51,6 +52,10 @@ class HTTP2CodecTest : public HTTPParallelCodecTest {
 
   void SetUp() override {
     HTTPParallelCodecTest::SetUp();
+    // make it transparent to the tests that we've received a settings frame
+    upstreamCodec_.generateSettings(output_);
+    parse();
+    callbacks_.reset();
   }
   void testHeaderListSize(bool oversized);
   void testFrameSizeLimit(bool oversized);
@@ -99,6 +104,14 @@ TEST_F(HTTP2CodecTest, IgnoreUnknownSettings) {
             downstreamCodec_.getIngressSettings()->getNumSettings());
 }
 
+// Some tests rely on the fact that we haven't flushed and parsed the preface
+// and settings frame.
+class HTTP2CodecTestOmitParsePreface : public HTTP2CodecTest {
+  void SetUp() override {
+    HTTPParallelCodecTest::SetUp();
+  }
+};
+
 TEST_F(HTTP2CodecTest, NoExHeaders) {
   // do not emit ENABLE_EX_HEADERS setting, if disabled
   SetUpUpstreamTest();
@@ -225,6 +238,22 @@ TEST_F(HTTP2CodecTest, BasicHeader) {
   EXPECT_EQ("www.foo.com", headers.getSingleOrEmpty(HTTP_HEADER_HOST));
 }
 
+TEST_F(HTTP2CodecTest, GenerateExtraHeaders) {
+  HTTPMessage req = getGetRequest("/fish_taco");
+  req.getHeaders().add(HTTP_HEADER_CONTENT_LENGTH, "157");
+  HTTPHeaders extraHeaders;
+  extraHeaders.add(HTTP_HEADER_PRIORITY, "u=0");
+  upstreamCodec_.generateHeader(
+      output_, 1, req, true, nullptr /* headerSize */, std::move(extraHeaders));
+
+  parse();
+  // There is also a HOST header
+  callbacks_.expectMessage(true, 3, "/fish_taco");
+  const auto& headers = callbacks_.msg->getHeaders();
+  EXPECT_EQ("157", headers.getSingleOrEmpty(HTTP_HEADER_CONTENT_LENGTH));
+  EXPECT_EQ("u=0", headers.getSingleOrEmpty(HTTP_HEADER_PRIORITY));
+}
+
 TEST_F(HTTP2CodecTest, RequestFromServer) {
   // this is to test EX_HEADERS frame, which carrys the HTTP request initiated
   // by server side
@@ -257,6 +286,20 @@ TEST_F(HTTP2CodecTest, RequestFromServer) {
   EXPECT_EQ("www.foo.com", headers.getSingleOrEmpty(HTTP_HEADER_HOST));
 }
 
+TEST_F(HTTP2CodecTestOmitParsePreface, OmitSettingsAfterConnPrefaceError) {
+  HTTPMessage req = getGetRequest("/test");
+  req.getHeaders().add(HTTP_HEADER_USER_AGENT, "rand-user");
+  req.setSecure(true);
+  upstreamCodec_.generateHeader(output_, 1, req, true, /*headerSize=*/nullptr);
+
+  parse();
+  EXPECT_EQ(callbacks_.settings, 0);
+  EXPECT_EQ(callbacks_.numSettings, 0);
+  EXPECT_EQ(callbacks_.sessionErrors, 1);
+  EXPECT_EQ(callbacks_.lastParseError->getCodecStatusCode(),
+            ErrorCode::PROTOCOL_ERROR);
+}
+
 TEST_F(HTTP2CodecTest, ResponseFromClient) {
   // this is to test EX_HEADERS frame, which carrys the HTTP response replied by
   // client side
@@ -297,7 +340,9 @@ TEST_F(HTTP2CodecTest, ExHeadersWithPriority) {
       output_, {{proxygen::SettingsId::ENABLE_EX_HEADERS, 1}});
 
   auto req = getGetRequest();
-  auto pri = HTTPMessage::HTTPPriority(0, false, 7);
+  // Test empty path
+  req.setURL("");
+  auto pri = HTTPMessage::HTTP2Priority(0, false, 7);
   req.setHTTP2Priority(pri);
   upstreamCodec_.generateExHeader(
       output_, 3, req, HTTPCodec::ExAttributes(1, true));
@@ -420,6 +465,7 @@ TEST_F(HTTP2CodecTest, BadPseudoHeaders) {
   EXPECT_EQ(callbacks_.headersComplete, 0);
   EXPECT_EQ(callbacks_.messageComplete, 0);
   EXPECT_EQ(callbacks_.streamErrors, 1);
+  EXPECT_EQ(callbacks_.lastParseError->getProxygenError(), kErrorParseHeader);
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
 
@@ -455,6 +501,106 @@ TEST_F(HTTP2CodecTest, BadHeaderValues) {
   EXPECT_EQ(callbacks_.headersComplete, 0);
   EXPECT_EQ(callbacks_.messageComplete, 0);
   EXPECT_EQ(callbacks_.streamErrors, 4);
+  EXPECT_EQ(callbacks_.lastParseError->getProxygenError(), kErrorParseHeader);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
+}
+
+TEST_F(HTTP2CodecTest, HostAuthority) {
+  static const std::string v1("GET");
+  static const std::string v2("/");
+  static const std::string v3("http");
+  static const std::string v4("foo.com");
+
+  static const vector<proxygen::compress::Header> reqHeaders = {
+      Header::makeHeaderForTest(headers::kMethod, v1),
+      Header::makeHeaderForTest(headers::kPath, v2),
+      Header::makeHeaderForTest(headers::kScheme, v3),
+      Header::makeHeaderForTest(headers::kAuthority, v4),
+  };
+
+  HTTPCodec::StreamID stream = 1;
+  for (auto i = 0; i < 2; i++, stream += 2) {
+    auto allHeaders = reqHeaders;
+    std::string v5(i == 0 ? v4 : "nope");
+    allHeaders.emplace_back(HTTP_HEADER_HOST, v5);
+    HPACKCodec headerCodec(TransportDirection::UPSTREAM);
+    auto encodedHeaders = headerCodec.encode(allHeaders);
+    writeHeaders(output_,
+                 std::move(encodedHeaders),
+                 stream,
+                 folly::none,
+                 http2::kNoPadding,
+                 true,
+                 true);
+    parse();
+    if (i == 0) {
+      // same value, ok
+      callbacks_.expectMessage(true, stream, "/");
+      const auto& headers = callbacks_.msg->getHeaders();
+      EXPECT_EQ(v4, headers.getSingleOrEmpty(HTTP_HEADER_HOST));
+    } else {
+      // different values, error
+      EXPECT_EQ(callbacks_.streamErrors, 1);
+      EXPECT_EQ(callbacks_.lastParseError->getHttpStatusCode(), 400);
+    }
+  }
+}
+
+TEST_F(HTTP2CodecTest, HighAscii) {
+  auto g =
+      folly::makeGuard([this] { downstreamCodec_.setStrictValidation(false); });
+  downstreamCodec_.setStrictValidation(true);
+  HTTPMessage req1 = getGetRequest("/guacamole\xff");
+  upstreamCodec_.generateHeader(
+      output_, 1, req1, true, nullptr /* headerSize */);
+  HTTPMessage req2 = getGetRequest("/guacamole");
+  req2.getHeaders().set(HTTP_HEADER_HOST, std::string("foo.com\xff"));
+  upstreamCodec_.generateHeader(
+      output_, 3, req2, true, nullptr /* headerSize */);
+  HTTPMessage req3 = getGetRequest("/guacamole");
+  req3.getHeaders().set(folly::StringPiece("Foo\xff"), "bar");
+  upstreamCodec_.generateHeader(
+      output_, 5, req3, true, nullptr /* headerSize */);
+  HTTPMessage req4 = getGetRequest("/guacamole");
+  req4.getHeaders().set("Foo", std::string("bar\xff"));
+  upstreamCodec_.generateHeader(
+      output_, 7, req4, true, nullptr /* headerSize */);
+
+  parse();
+  EXPECT_EQ(callbacks_.messageBegin, 4);
+  EXPECT_EQ(callbacks_.headersComplete, 0);
+  EXPECT_EQ(callbacks_.messageComplete, 0);
+  EXPECT_EQ(callbacks_.streamErrors, 4);
+  EXPECT_EQ(callbacks_.lastParseError->getProxygenError(), kErrorParseHeader);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
+
+  HTTPMessage req5 = getGetRequest("/guacamole");
+  req5.getHeaders().set(HTTP_HEADER_USER_AGENT, "ꪶ𝛸ꫂ_𝐹𝛩𝑅𝐶𝛯_𝑉2");
+  upstreamCodec_.generateHeader(
+      output_, 9, req5, true, nullptr /* headerSize */);
+  callbacks_.reset();
+  parse();
+  EXPECT_EQ(callbacks_.messageBegin, 1);
+  EXPECT_EQ(callbacks_.headersComplete, 0);
+  EXPECT_EQ(callbacks_.messageComplete, 0);
+  EXPECT_EQ(callbacks_.streamErrors, 1);
+  EXPECT_EQ(callbacks_.lastParseError->getProxygenError(), kErrorParseHeader);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
+}
+
+TEST_F(HTTP2CodecTest, EmptyPath) {
+  auto g =
+      folly::makeGuard([this] { downstreamCodec_.setStrictValidation(false); });
+  downstreamCodec_.setStrictValidation(true);
+  HTTPMessage req1 = getGetRequest("");
+  upstreamCodec_.generateHeader(
+      output_, 1, req1, true, nullptr /* headerSize */);
+  parse();
+  EXPECT_EQ(callbacks_.messageBegin, 1);
+  EXPECT_EQ(callbacks_.headersComplete, 0);
+  EXPECT_EQ(callbacks_.messageComplete, 0);
+  EXPECT_EQ(callbacks_.streamErrors, 1);
+  EXPECT_EQ(callbacks_.lastParseError->getProxygenError(), kErrorParseHeader);
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
 
@@ -474,6 +620,7 @@ TEST_F(HTTP2CodecTest, EmptyHeaderName) {
   EXPECT_EQ(callbacks_.headersComplete, 0);
   EXPECT_EQ(callbacks_.messageComplete, 0);
   EXPECT_EQ(callbacks_.streamErrors, 1);
+  EXPECT_EQ(callbacks_.lastParseError->getProxygenError(), kErrorParseHeader);
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
 
@@ -555,7 +702,6 @@ void HTTP2CodecTest::testFrameSizeLimit(bool oversized) {
   HTTPMessage req = getBigGetRequest("/guacamole");
   auto settings = downstreamCodec_.getEgressSettings();
 
-  parse(); // consume preface
   if (oversized) {
     // trick upstream for sending a 2x bigger HEADERS frame
     settings->setSetting(SettingsId::MAX_FRAME_SIZE,
@@ -600,7 +746,6 @@ TEST_F(HTTP2CodecTest, BigHeaderCompressed) {
   downstreamCodec_.generateSettings(output_);
   parseUpstream();
 
-  SetUp();
   HTTPMessage req = getGetRequest("/guacamole");
   req.getHeaders().add(HTTP_HEADER_USER_AGENT, "coolio");
   upstreamCodec_.generateHeader(output_, 1, req, true /* eom */);
@@ -783,7 +928,8 @@ TEST_F(HTTP2CodecTest, MissingContinuation) {
   EXPECT_EQ(callbacks_.streamErrors, 0);
   EXPECT_EQ(callbacks_.sessionErrors, 1);
 #ifndef NDEBUG
-  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 2);
+  // frames = 3 to account for settings frame after H2 conn preface
+  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 3);
 #endif
 }
 
@@ -807,7 +953,8 @@ TEST_F(HTTP2CodecTest, MissingContinuationBadFrame) {
   EXPECT_EQ(callbacks_.streamErrors, 0);
   EXPECT_EQ(callbacks_.sessionErrors, 1);
 #ifndef NDEBUG
-  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 2);
+  // frames = 3 to account for settings frame after H2 conn preface
+  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 3);
 #endif
 }
 
@@ -827,7 +974,8 @@ TEST_F(HTTP2CodecTest, BadContinuationStream) {
   EXPECT_EQ(callbacks_.streamErrors, 0);
   EXPECT_EQ(callbacks_.sessionErrors, 1);
 #ifndef NDEBUG
-  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 2);
+  // frames = 3 to account for settings frame after H2 conn preface
+  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 3);
 #endif
 }
 
@@ -845,6 +993,27 @@ TEST_F(HTTP2CodecTest, FrameTooLarge) {
             ErrorCode::FRAME_SIZE_ERROR);
 }
 
+TEST_F(HTTP2CodecTest, DataFrameZeroLengthWithEOM) {
+  HTTPMessage req = getGetRequest();
+  req.getHeaders().add(HTTP_HEADER_USER_AGENT, "coolio");
+
+  upstreamCodec_.generateHeader(output_, 1, req);
+  // Write Zero length Data frame (just a Frame header) with end of stream set
+  writeFrameHeaderManual(output_,
+                         0,
+                         (uint8_t)http2::FrameType::DATA,
+                         (uint8_t)http2::Flags::END_STREAM,
+                         1);
+
+  parse();
+
+  EXPECT_EQ(callbacks_.messageBegin, 1);
+  EXPECT_EQ(callbacks_.headersComplete, 1);
+  EXPECT_EQ(callbacks_.messageComplete, 1);
+  EXPECT_EQ(callbacks_.streamErrors, 0);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
+}
+
 TEST_F(HTTP2CodecTest, UnknownFrameType) {
   HTTPMessage req = getGetRequest();
   req.getHeaders().add(HTTP_HEADER_USER_AGENT, "coolio");
@@ -1001,7 +1170,7 @@ TEST_F(HTTP2CodecTest, MalformedPaddingLength) {
                               0x35,
                               0xa7,
                               0xd7};
-  output_.clear();
+  output_.reset();
   output_.append(badInput, sizeof(badInput));
   EXPECT_EQ(output_.chainLength(), sizeof(badInput));
 
@@ -1017,16 +1186,19 @@ TEST_F(HTTP2CodecTest, MalformedPadding) {
   EXPECT_FALSE(parse());
 }
 
-TEST_F(HTTP2CodecTest, NoAppByte) {
+TEST_F(HTTP2CodecTestOmitParsePreface, NoAppByte) {
   const uint8_t noAppByte[] = {
-      0x50, 0x52, 0x49, 0x20, 0x2a, 0x20, 0x48, 0x54, 0x54, 0x50, 0x2f, 0x32,
-      0x2e, 0x30, 0x0d, 0x0a, 0x0d, 0x0a, 0x53, 0x4d, 0x0d, 0x0a, 0x0d, 0x0a,
+      0x50, 0x52, 0x49, 0x20, 0x2a, 0x20, 0x48, 0x54, 0x54, 0x50, 0x2f,
+      0x32, 0x2e, 0x30, 0x0d, 0x0a, 0x0d, 0x0a, 0x53, 0x4d, 0x0d, 0x0a,
+      0x0d, 0x0a, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
       0x00, 0x00, 0x56, 0x00, 0x5d, 0x00, 0x00, 0x00, 0x01, 0x55, 0x00};
-  output_.clear();
+  output_.reset();
   output_.append(noAppByte, sizeof(noAppByte));
   EXPECT_EQ(output_.chainLength(), sizeof(noAppByte));
 
   EXPECT_TRUE(parse());
+  EXPECT_EQ(callbacks_.settings, 1);
+  EXPECT_EQ(callbacks_.numSettings, 0);
   EXPECT_EQ(callbacks_.messageBegin, 0);
   EXPECT_EQ(callbacks_.headersComplete, 0);
   EXPECT_EQ(callbacks_.messageComplete, 0);
@@ -1045,7 +1217,8 @@ TEST_F(HTTP2CodecTest, DataFramePartialDataOnFrameHeaderCall) {
   auto buf = makeBuf(bufSize);
   const size_t padding = 10;
   upstreamCodec_.generateBody(output_, 1, buf->clone(), padding, true);
-  EXPECT_EQ(output_.chainLength(), 54);
+  // 9 (frame header) + 1 (padding length) + 10 (body) + 10 (padding)
+  EXPECT_EQ(output_.chainLength(), 30);
 
   downstreamCodec_.setCallback(&mockCallback);
 
@@ -1061,17 +1234,18 @@ TEST_F(HTTP2CodecTest, DataFramePartialDataWithNoAppByte) {
   auto buf = makeBuf(bufSize);
   const size_t padding = 10;
   upstreamCodec_.generateBody(output_, 1, buf->clone(), padding, true);
-  EXPECT_EQ(output_.chainLength(), 54);
+  // 9 (frame header) + 1 (padding length) + 10 (body) + 10 (padding)
+  EXPECT_EQ(output_.chainLength(), 30);
 
   auto ingress = output_.move();
   ingress->coalesce();
   // Copy up to the padding length byte to a new buffer
-  auto ingress1 = IOBuf::copyBuffer(ingress->data(), 34);
+  auto ingress1 = IOBuf::copyBuffer(ingress->data(), 10);
   size_t parsed = downstreamCodec_.onIngress(*ingress1);
-  // The 34th byte is the padding length byte which should not be parsed
-  EXPECT_EQ(parsed, 33);
+  // The 10th byte is the padding length byte which should not be parsed
+  EXPECT_EQ(parsed, 9);
   // Copy from the padding length byte to the end
-  auto ingress2 = IOBuf::copyBuffer(ingress->data() + 33, 21);
+  auto ingress2 = IOBuf::copyBuffer(ingress->data() + 9, 21);
   parsed = downstreamCodec_.onIngress(*ingress2);
   // The padding length byte should be parsed this time along with 10 bytes of
   // application data and 10 bytes of padding
@@ -1102,7 +1276,7 @@ TEST_F(HTTP2CodecTest, BasicRst) {
 }
 
 TEST_F(HTTP2CodecTest, BasicRstInvalidCode) {
-  upstreamCodec_.generateRstStream(output_, 2, ErrorCode::_SPDY_INVALID_STREAM);
+  upstreamCodec_.generateRstStream(output_, 2, ErrorCode::STREAM_CLOSED);
   parse();
   EXPECT_EQ(callbacks_.messageBegin, 0);
   EXPECT_EQ(callbacks_.headersComplete, 0);
@@ -1120,7 +1294,7 @@ TEST_F(HTTP2CodecTest, BasicPing) {
   uint64_t pingReq;
   parse([&](IOBuf* ingress) {
     folly::io::Cursor c(ingress);
-    c.skip(http2::kFrameHeaderSize + http2::kConnectionPreface.length());
+    c.skip(http2::kFrameHeaderSize);
     pingReq = c.read<uint64_t>();
   });
 
@@ -1192,8 +1366,24 @@ TEST_F(HTTP2CodecTest, BadGoaway) {
   EXPECT_EQ(bytes, 0);
 }
 
+TEST_F(HTTP2CodecTest, RstStreamNoError) {
+  SetUpUpstreamTest();
+  HTTPMessage resp;
+  resp.setStatusCode(200);
+  resp.setStatusMessage("OK");
+  downstreamCodec_.generateHeader(output_, 1, resp);
+  downstreamCodec_.generateBody(
+      output_, 1, makeBuf(10), HTTPCodec::NoPadding, true);
+  downstreamCodec_.generateRstStream(output_, 1, ErrorCode::NO_ERROR);
+  parseUpstream();
+  EXPECT_EQ(callbacks_.messageBegin, 1);
+  EXPECT_EQ(callbacks_.headersComplete, 1);
+  EXPECT_EQ(callbacks_.bodyCalls, 1);
+  EXPECT_EQ(callbacks_.messageComplete, 1);
+  EXPECT_EQ(callbacks_.aborts, 1);
+}
+
 TEST_F(HTTP2CodecTest, DoubleGoaway) {
-  parse();
   SetUpUpstreamTest();
   downstreamCodec_.generateGoaway(output_);
   EXPECT_TRUE(downstreamCodec_.isWaitingToDrain());
@@ -1201,6 +1391,21 @@ TEST_F(HTTP2CodecTest, DoubleGoaway) {
   EXPECT_TRUE(downstreamCodec_.isStreamIngressEgressAllowed(0));
   EXPECT_TRUE(downstreamCodec_.isStreamIngressEgressAllowed(1));
   EXPECT_TRUE(downstreamCodec_.isStreamIngressEgressAllowed(2));
+
+  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(0));
+  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(1));
+  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(2));
+  EXPECT_TRUE(upstreamCodec_.isReusable());
+
+  parseUpstream();
+  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(0));
+  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(1));
+  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(2));
+  EXPECT_FALSE(upstreamCodec_.isReusable());
+  EXPECT_EQ(callbacks_.goaways, 1);
+  EXPECT_EQ(callbacks_.streamErrors, 0);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
+
   downstreamCodec_.generateGoaway(output_, 0, ErrorCode::NO_ERROR);
   EXPECT_FALSE(downstreamCodec_.isWaitingToDrain());
   EXPECT_FALSE(downstreamCodec_.isReusable());
@@ -1208,9 +1413,6 @@ TEST_F(HTTP2CodecTest, DoubleGoaway) {
   EXPECT_FALSE(downstreamCodec_.isStreamIngressEgressAllowed(1));
   EXPECT_TRUE(downstreamCodec_.isStreamIngressEgressAllowed(2));
 
-  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(0));
-  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(1));
-  EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(2));
   parseUpstream();
   EXPECT_TRUE(upstreamCodec_.isStreamIngressEgressAllowed(0));
   EXPECT_FALSE(upstreamCodec_.isStreamIngressEgressAllowed(1));
@@ -1275,7 +1477,7 @@ TEST_F(HTTP2CodecTest, GoawayHandling) {
   upstreamCodec_.generateBody(
       output_, 3, makeBuf(10), HTTPCodec::NoPadding, false);
   upstreamCodec_.generatePriority(
-      output_, 3, HTTPMessage::HTTPPriority(0, true, 1));
+      output_, 3, HTTPMessage::HTTP2Priority(0, true, 1));
   upstreamCodec_.generateEOM(output_, 3);
   upstreamCodec_.generateRstStream(output_, 3, ErrorCode::CANCEL);
   EXPECT_EQ(output_.chainLength(), 0);
@@ -1302,7 +1504,7 @@ TEST_F(HTTP2CodecTest, GoawayHandling) {
 
   output_.append(makeBuf(10));
   downstreamCodec_.generatePriority(
-      output_, 2, HTTPMessage::HTTPPriority(0, true, 1));
+      output_, 2, HTTPMessage::HTTP2Priority(0, true, 1));
   downstreamCodec_.generateEOM(output_, 2);
   downstreamCodec_.generateRstStream(output_, 2, ErrorCode::CANCEL);
 
@@ -1369,7 +1571,7 @@ TEST_F(HTTP2CodecTest, BadSettings) {
   EXPECT_EQ(callbacks_.sessionErrors, 1);
 }
 
-TEST_F(HTTP2CodecTest, BadPushSettings) {
+TEST_F(HTTP2CodecTestOmitParsePreface, BadPushSettings) {
   auto settings = downstreamCodec_.getEgressSettings();
   settings->clearSettings();
   settings->setSetting(SettingsId::ENABLE_PUSH, 0);
@@ -1415,7 +1617,7 @@ TEST_F(HTTP2CodecTest, SettingsTableSize) {
   EXPECT_EQ("x-coolio", headers.getSingleOrEmpty(HTTP_HEADER_CONTENT_TYPE));
 }
 
-TEST_F(HTTP2CodecTest, BadSettingsTableSize) {
+TEST_F(HTTP2CodecTestOmitParsePreface, BadSettingsTableSize) {
   auto settings = upstreamCodec_.getEgressSettings();
   settings->setSetting(SettingsId::HEADER_TABLE_SIZE, 8192);
   // This sets the max decoder table size to 8k
@@ -1523,7 +1725,7 @@ TEST_F(HTTP2CodecTest, ConcurrentStreams) {
 }
 
 TEST_F(HTTP2CodecTest, BasicPriority) {
-  auto pri = HTTPMessage::HTTPPriority(0, true, 1);
+  auto pri = HTTPMessage::HTTP2Priority(0, true, 1);
   upstreamCodec_.generatePriority(output_, 1, pri);
 
   EXPECT_TRUE(parse());
@@ -1534,13 +1736,13 @@ TEST_F(HTTP2CodecTest, BasicPriority) {
 
 TEST_F(HTTP2CodecTest, BadHeaderPriority) {
   HTTPMessage req = getGetRequest();
-  req.setHTTP2Priority(HTTPMessage::HTTPPriority(0, false, 7));
+  req.setHTTP2Priority(HTTPMessage::HTTP2Priority(0, false, 7));
   upstreamCodec_.generateHeader(output_, 1, req, true /* eom */);
 
   // hack ingress with cirular dep
   EXPECT_TRUE(parse([&](IOBuf* ingress) {
     folly::io::RWPrivateCursor c(ingress);
-    c.skip(http2::kFrameHeaderSize + http2::kConnectionPreface.length());
+    c.skip(http2::kFrameHeaderSize);
     c.writeBE<uint32_t>(1);
   }));
 
@@ -1550,42 +1752,52 @@ TEST_F(HTTP2CodecTest, BadHeaderPriority) {
 
 TEST_F(HTTP2CodecTest, CircularHeaderPriority) {
   HTTPMessage req = getGetRequest();
-  req.setHTTP2Priority(HTTPMessage::HTTPPriority(1, false, 7));
+  req.setHTTP2Priority(HTTPMessage::HTTP2Priority(1, false, 7));
   upstreamCodec_.generateHeader(output_, 1, req, true /* eom */);
 }
 
 TEST_F(HTTP2CodecTest, DuplicateBadHeaderPriority) {
   // Sent an initial header with a circular dependency
   HTTPMessage req = getGetRequest();
-  req.setHTTP2Priority(HTTPMessage::HTTPPriority(0, false, 7));
+  req.setHTTP2Priority(HTTPMessage::HTTP2Priority(0, false, 7));
   upstreamCodec_.generateHeader(output_, 1, req, true /* eom */);
 
   // Hack ingress with circular dependency.
   EXPECT_TRUE(parse([&](IOBuf* ingress) {
     folly::io::RWPrivateCursor c(ingress);
-    c.skip(http2::kFrameHeaderSize + http2::kConnectionPreface.length());
+    c.skip(http2::kFrameHeaderSize);
     c.writeBE<uint32_t>(1);
   }));
 
   EXPECT_EQ(callbacks_.streamErrors, 1);
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 
-  // On the same stream, send another request.
+  // On the same stream, send another request, interpreted as trailers with
+  // a pseudo header which is illegal
   HTTPMessage nextRequest = getGetRequest();
   upstreamCodec_.generateHeader(output_, 1, nextRequest, true /* eom */);
   parse();
   EXPECT_EQ(callbacks_.streamErrors, 2);
   EXPECT_EQ(callbacks_.sessionErrors, 0);
+
+  callbacks_.reset();
+
+  // Now send a legit request
+  upstreamCodec_.generateHeader(output_, 3, nextRequest, true /* eom */);
+  parse();
+  callbacks_.expectMessage(true, 1, "/");
+  EXPECT_EQ(callbacks_.streamErrors, 0);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
 
 TEST_F(HTTP2CodecTest, BadPriority) {
-  auto pri = HTTPMessage::HTTPPriority(0, true, 1);
+  auto pri = HTTPMessage::HTTP2Priority(0, true, 1);
   upstreamCodec_.generatePriority(output_, 1, pri);
 
   // hack ingress with cirular dep
   EXPECT_TRUE(parse([&](IOBuf* ingress) {
     folly::io::RWPrivateCursor c(ingress);
-    c.skip(http2::kFrameHeaderSize + http2::kConnectionPreface.length());
+    c.skip(http2::kFrameHeaderSize);
     c.writeBE<uint32_t>(1);
   }));
 
@@ -1760,7 +1972,7 @@ TEST_F(HTTP2CodecTest, BasicCertificate) {
 }
 
 TEST_F(HTTP2CodecTest, BadServerPreface) {
-  output_.move();
+  output_.reset();
   downstreamCodec_.generateWindowUpdate(output_, 0, 10);
   parseUpstream();
   EXPECT_EQ(callbacks_.messageBegin, 0);
@@ -1776,7 +1988,7 @@ TEST_F(HTTP2CodecTest, Normal1024Continuation) {
   string bigval(8691, '!');
   bigval.append(8691, '@');
   req.getHeaders().add("x-headr", bigval);
-  req.setHTTP2Priority(HTTPMessage::HTTPPriority(0, false, 7));
+  req.setHTTP2Priority(HTTPMessage::HTTP2Priority(0, false, 7));
   upstreamCodec_.generateHeader(output_, 1, req);
 
   parse();
@@ -2007,7 +2219,7 @@ TEST_F(HTTP2CodecTest, WebsocketDupProtocol) {
 }
 
 TEST_F(HTTP2CodecTest, WebsocketIncorrectResponse) {
-  output_.clear();
+  output_.reset();
   HTTPMessage resp;
   resp.setStatusCode(400);
   resp.setStatusMessage("Bad Request");
@@ -2068,7 +2280,7 @@ TEST_F(HTTP2CodecTest, TestAllEgressFrameTypeCallbacks) {
   upstreamCodec_.generateHeader(output_, 1, req, true, &size);
 
   upstreamCodec_.generatePriority(
-      output_, 3, HTTPMessage::HTTPPriority(0, true, 1));
+      output_, 3, HTTPMessage::HTTP2Priority(0, true, 1));
   upstreamCodec_.generateRstStream(output_, 2, ErrorCode::ENHANCE_YOUR_CALM);
   upstreamCodec_.generateSettings(output_);
   downstreamCodec_.generatePushPromise(output_, 2, req, 1);
@@ -2121,7 +2333,8 @@ TEST_F(HTTP2CodecTest, Trailers) {
   EXPECT_EQ(callbacks_.streamErrors, 0);
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 #ifndef NDEBUG
-  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 3);
+  // frames = 4 to account for settings frame after H2 conn preface
+  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 4);
 #endif
 }
 
@@ -2183,7 +2396,8 @@ TEST_F(HTTP2CodecTest, TrailersNoBody) {
   EXPECT_EQ(callbacks_.streamErrors, 0);
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 #ifndef NDEBUG
-  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 2);
+  // frames = 3 to account for settings frame after H2 conn preface
+  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 3);
 #endif
 }
 
@@ -2212,7 +2426,8 @@ TEST_F(HTTP2CodecTest, TrailersContinuation) {
   EXPECT_EQ(std::string(http2::kMaxFramePayloadLengthMin, '!'),
             callbacks_.msg->getTrailers()->getSingleOrEmpty("x-huge-trailer"));
 #ifndef NDEBUG
-  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 3);
+  // frames = 4 to account for settings frame after H2 conn preface
+  EXPECT_EQ(downstreamCodec_.getReceivedFrameCount(), 4);
 #endif
 }
 
@@ -2253,6 +2468,39 @@ TEST_F(HTTP2CodecTest, TrailersReply) {
 #endif
 }
 
+TEST_F(HTTP2CodecTest, TrailersReplyEmpty) {
+  SetUpUpstreamTest();
+  HTTPMessage resp;
+  resp.setStatusCode(200);
+  resp.setStatusMessage("nifty-nice");
+  resp.getHeaders().add(HTTP_HEADER_CONTENT_TYPE, "x-coolio");
+  downstreamCodec_.generateHeader(output_, 1, resp);
+
+  string data("abcde");
+  auto buf = folly::IOBuf::copyBuffer(data.data(), data.length());
+  downstreamCodec_.generateBody(
+      output_, 1, std::move(buf), HTTPCodec::NoPadding, false);
+
+  HTTPHeaders trailers;
+  auto trailerSz = downstreamCodec_.generateTrailers(output_, 1, trailers);
+  // Just a frame header (DATA + END_STREAM).
+  EXPECT_EQ(trailerSz, 9);
+
+  parseUpstream();
+
+  callbacks_.expectMessage(true, 2, 200);
+  EXPECT_EQ(callbacks_.bodyCalls, 1);
+  EXPECT_EQ(callbacks_.bodyLength, 5);
+  const auto& headers = callbacks_.msg->getHeaders();
+  EXPECT_TRUE(callbacks_.msg->getHeaders().exists(HTTP_HEADER_DATE));
+  EXPECT_EQ("x-coolio", headers.getSingleOrEmpty(HTTP_HEADER_CONTENT_TYPE));
+  EXPECT_EQ(0, callbacks_.trailers);
+  EXPECT_EQ(nullptr, callbacks_.msg->getTrailers());
+#ifndef NDEBUG
+  EXPECT_EQ(upstreamCodec_.getReceivedFrameCount(), 4);
+#endif
+}
+
 TEST_F(HTTP2CodecTest, TrailersReplyWithNoData) {
   SetUpUpstreamTest();
   HTTPMessage resp;
@@ -2405,79 +2653,28 @@ TEST_F(HTTP2CodecTest, TrailersNotLatest) {
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
 
-TEST_F(HTTP2CodecTest, PriorityHeaderUrgencyOnly) {
-  HTTPMessage req = getGetRequest("/pittston");
-  req.getHeaders().add(HTTP_HEADER_PRIORITY, "u=5");
-  upstreamCodec_.generateHeader(output_, 1, req);
+class HTTP2CodecDoubleGoawayTest : public HTTPParallelCodecTest {
+ public:
+  HTTP2CodecDoubleGoawayTest()
+      : HTTPParallelCodecTest(upstreamCodec_, downstreamCodec_) {
+  }
 
-  parse();
-  const auto& msg = callbacks_.msg;
-  EXPECT_EQ(msg->getPriority(), 5);
-  EXPECT_FALSE(msg->getIncremental());
-}
-
-TEST_F(HTTP2CodecTest, PriorityHeaderUrgencyAndIncremental) {
-  HTTPMessage req = getGetRequest("/pittston");
-  req.getHeaders().add(HTTP_HEADER_PRIORITY, "u=4, i");
-  upstreamCodec_.generateHeader(output_, 1, req);
-
-  parse();
-  const auto& msg = callbacks_.msg;
-  EXPECT_EQ(msg->getPriority(), 4);
-  EXPECT_TRUE(msg->getIncremental());
-}
-
-TEST_F(HTTP2CodecTest, PriorityHeaderUrgencyAndIncrementalUppercase) {
-  HTTPMessage req = getGetRequest("/pittston");
-  req.getHeaders().add(HTTP_HEADER_PRIORITY, "u=4, i");
-  upstreamCodec_.generateHeader(output_, 1, req);
-
-  parse();
-  const auto& msg = callbacks_.msg;
-  EXPECT_EQ(msg->getPriority(), 4);
-  EXPECT_TRUE(msg->getIncremental());
-}
-
-TEST_F(HTTP2CodecTest, PriorityHeaderUrgencyAndIncrementalTrimSpaces) {
-  HTTPMessage req = getGetRequest("/pittston");
-  req.getHeaders().add(HTTP_HEADER_PRIORITY, "u=4,  i ");
-  upstreamCodec_.generateHeader(output_, 1, req);
-
-  parse();
-  const auto& msg = callbacks_.msg;
-  EXPECT_EQ(msg->getPriority(), 4);
-  EXPECT_TRUE(msg->getIncremental());
-}
-
-TEST_F(HTTP2CodecTest, PriorityHeaderBadUrgency) {
-  HTTPMessage req = getGetRequest("/pittston");
-  req.getHeaders().add(HTTP_HEADER_PRIORITY, "p=3");
-  upstreamCodec_.generateHeader(output_, 1, req);
-
-  parse();
-  const auto& msg = callbacks_.msg;
-  EXPECT_EQ(msg->getPriority(), kDefaultHttpPriorityUrgency);
-  EXPECT_TRUE(msg->getIncremental());
-}
-
-TEST_F(HTTP2CodecTest, PriorityHeaderBadUrgencyWithIncremental) {
-  HTTPMessage req = getGetRequest("/pittston");
-  req.getHeaders().add(HTTP_HEADER_PRIORITY, "p=3, i");
-  upstreamCodec_.generateHeader(output_, 1, req);
-
-  parse();
-  const auto& msg = callbacks_.msg;
-  EXPECT_EQ(msg->getPriority(), kDefaultHttpPriorityUrgency);
-  EXPECT_TRUE(msg->getIncremental());
-}
-
-TEST_F(HTTP2CodecTest, PriorityHeaderBadIncremental) {
-  HTTPMessage req = getGetRequest("/pittston");
-  req.getHeaders().add(HTTP_HEADER_PRIORITY, "u=3, true");
-  upstreamCodec_.generateHeader(output_, 1, req);
-
-  parse();
-  const auto& msg = callbacks_.msg;
-  EXPECT_EQ(msg->getPriority(), kDefaultHttpPriorityUrgency);
-  EXPECT_TRUE(msg->getIncremental());
+  void SetUp() override {
+    HTTPParallelCodecTest::SetUp();
+  }
+
+ protected:
+  HTTP2Codec upstreamCodec_{TransportDirection::UPSTREAM};
+  HTTP2Codec downstreamCodec_{TransportDirection::DOWNSTREAM};
+};
+
+TEST_F(HTTP2CodecDoubleGoawayTest, EnableDoubleGoawayAfterClose) {
+  SetUpUpstreamTest();
+  upstreamCodec_.generateGoaway(output_);
+  // Now a no-op
+  upstreamCodec_.enableDoubleGoawayDrain();
+  parseUpstream();
+  EXPECT_EQ(callbacks_.goaways, 1);
+  EXPECT_EQ(callbacks_.streamErrors, 0);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
