@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -27,6 +27,7 @@
 #include <proxygen/lib/http/codec/HTTPCodecFilter.h>
 #include <proxygen/lib/http/session/ByteEventTracker.h>
 #include <proxygen/lib/http/session/HTTPEvent.h>
+#include <proxygen/lib/http/session/HTTPSessionActivityTracker.h>
 #include <proxygen/lib/http/session/HTTPSessionBase.h>
 #include <proxygen/lib/http/session/HTTPTransaction.h>
 #include <proxygen/lib/http/session/SecondaryAuthManagerBase.h>
@@ -63,6 +64,15 @@ class HTTPSession
     return HTTPTransaction::Transport::Type::TCP;
   }
 
+  void setHTTPSessionActivityTracker(std::unique_ptr<HTTPSessionActivityTracker>
+                                         httpSessionActivityTracker) override {
+    if (!getByteEventTracker()) {
+      setByteEventTracker(std::make_shared<ByteEventTracker>(this));
+    }
+    HTTPSessionBase::setHTTPSessionActivityTracker(
+        std::move(httpSessionActivityTracker));
+  }
+
   folly::AsyncTransport* getTransport() override {
     return sock_.get();
   }
@@ -133,8 +143,8 @@ class HTTPSession
     HTTPSessionBase::setHTTP2PrioritiesEnabled(enabled);
   }
 
-  folly::Optional<HTTPTransaction::ConnectionToken> getConnectionToken() const
-      noexcept override {
+  folly::Optional<HTTPTransaction::ConnectionToken> getConnectionToken()
+      const noexcept override {
     return connectionToken_;
   }
 
@@ -284,8 +294,6 @@ class HTTPSession
     return draining_;
   }
 
-  virtual void injectTraceEventIntoAllTransactions(TraceEvent& event) override;
-
   bool readsUnpaused() const {
     return reads_ == SocketState::UNPAUSED;
   }
@@ -417,13 +425,12 @@ class HTTPSession
                                    std::unique_ptr<HTTPCodec> codec,
                                    const std::string& protocolString);
 
-  virtual folly::Optional<const HTTPMessage::HTTPPriority> getHTTPPriority(
+  using HTTPTransaction::Transport::getHTTPPriority;
+  virtual folly::Optional<const HTTPMessage::HTTP2Priority> getHTTPPriority(
       uint8_t) override {
     return folly::none;
   }
 
-  void setNewTransactionPauseState(HTTPTransaction* txn) override;
-
   void readTimeoutExpired() noexcept;
   void writeTimeoutExpired() noexcept;
   void flowControlTimeoutExpired() noexcept;
@@ -473,7 +480,8 @@ class HTTPSession
   void onSettings(const SettingsList& settings) override;
   void onSettingsAck() override;
   void onPriority(HTTPCodec::StreamID stream,
-                  const HTTPMessage::HTTPPriority&) override;
+                  const HTTPMessage::HTTP2Priority&) override;
+  void onPriority(HTTPCodec::StreamID, const HTTPPriority&) override;
   void onCertificateRequest(uint16_t requestId,
                             std::unique_ptr<folly::IOBuf> authRequest) override;
   void onCertificate(uint16_t certId,
@@ -497,6 +505,11 @@ class HTTPSession
                   std::unique_ptr<folly::IOBuf>,
                   bool includeEOM,
                   bool trackLastByteFlushed) noexcept override;
+  size_t sendBody(HTTPTransaction*,
+                  const HTTPTransaction::BufferMeta&,
+                  bool /* eom */) noexcept override {
+    return 0;
+  }
   size_t sendChunkHeader(HTTPTransaction* txn, size_t length) noexcept override;
   size_t sendChunkTerminator(HTTPTransaction* txn) noexcept override;
   size_t sendEOM(HTTPTransaction* txn,
@@ -505,6 +518,8 @@ class HTTPSession
                    ErrorCode statusCode) noexcept override;
   size_t sendPriority(HTTPTransaction* txn,
                       const http2::PriorityUpdate& pri) noexcept override;
+  size_t changePriority(HTTPTransaction* /*txn*/,
+                        HTTPPriority /* pri */) noexcept override;
   void detach(HTTPTransaction* txn) noexcept override;
   size_t sendWindowUpdate(HTTPTransaction* txn,
                           uint32_t bytes) noexcept override;
@@ -528,8 +543,8 @@ class HTTPSession
    * Returns the underlying AsyncTransport.
    * Overrides HTTPTransaction::Transport::getUnderlyingTransport().
    */
-  const folly::AsyncTransport* getUnderlyingTransport() const
-      noexcept override {
+  const folly::AsyncTransport* getUnderlyingTransport()
+      const noexcept override {
     return sock_.get();
   }
 
@@ -554,7 +569,8 @@ class HTTPSession
    */
   void shutdownTransport(bool shutdownReads = true,
                          bool shutdownWrites = true,
-                         const std::string& errorMsg = "");
+                         const std::string& errorMsg = "",
+                         ProxygenError error = kErrorEOF);
 
   /**
    * Immediately close the socket in both directions, discarding any
@@ -671,10 +687,6 @@ class HTTPSession
    */
   void invokeOnAllTransactions(folly::Function<void(HTTPTransaction*)> fn);
 
-  void pauseTransactions() override {
-    invokeOnAllTransactions([](HTTPTransaction* txn) { txn->pauseEgress(); });
-  }
-
   /**
    * This function invokes a callback on all transactions. It is safe,
    * but runs in O(n*log n) and if the callback *adds* transactions,
@@ -816,7 +828,7 @@ class HTTPSession
    * longer tracking.
    */
   void invalidStream(HTTPCodec::StreamID stream,
-                     ErrorCode code = ErrorCode::_SPDY_INVALID_STREAM);
+                     ErrorCode code = ErrorCode::STREAM_CLOSED);
 
   http2::PriorityUpdate getMessagePriority(const HTTPMessage* msg);
 
@@ -970,6 +982,8 @@ class HTTPSession
 
   std::shared_ptr<ByteEventTracker> byteEventTracker_{nullptr};
 
+  std::unique_ptr<HTTPSessionActivityTracker> httpSessionActivityTracker_;
+
   HTTPTransaction* lastTxn_{nullptr};
 
   /**
@@ -1096,7 +1110,6 @@ class HTTPSession
   bool resetSocketOnShutdown_ : 1;
   // indicates a fatal error that prevents further ingress data processing
   bool inLoopCallback_ : 1;
-  bool inResume_ : 1;
   bool pendingPause_ : 1;
   bool writeBufSplit_ : 1;
 
