@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -13,6 +13,7 @@
 #include <folly/io/async/AsyncUDPSocket.h>
 #include <folly/io/async/SSLContext.h>
 #include <proxygen/lib/http/codec/HTTPCodecFilter.h>
+#include <proxygen/lib/http/session/HTTPSessionActivityTracker.h>
 #include <proxygen/lib/http/session/HTTPTransaction.h>
 #include <proxygen/lib/utils/Time.h>
 #include <wangle/acceptor/ManagedConnection.h>
@@ -123,9 +124,17 @@ class HTTPSessionBase : public wangle::ManagedConnection {
                   const WheelTimerInstance& wheelTimer,
                   HTTPCodec::StreamID rootNodeId);
 
-  virtual ~HTTPSessionBase() {
+  virtual ~HTTPSessionBase() override;
+
+  virtual void setHTTPSessionActivityTracker(
+      std::unique_ptr<HTTPSessionActivityTracker> httpSessionActivityTracker) {
+    httpSessionActivityTracker_ = std::move(httpSessionActivityTracker);
   }
 
+  [[nodiscard]] HTTPSessionActivityTracker* getHTTPSessionActivityTracker()
+      const {
+    return httpSessionActivityTracker_.get();
+  }
   /**
    * Set the read buffer limit to be used for all new HTTPSessionBase objects.
    */
@@ -162,9 +171,7 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     return infoCallback_;
   }
 
-  virtual void setSessionStats(HTTPSessionStats* stats) {
-    sessionStats_ = stats;
-  }
+  virtual void setSessionStats(HTTPSessionStats* stats);
 
   virtual HTTPTransaction::Transport::Type getType() const noexcept = 0;
 
@@ -186,7 +193,7 @@ class HTTPSessionBase : public wangle::ManagedConnection {
   /**
    * Returns true iff a new outgoing transaction can be made on this session
    */
-  bool supportsMoreTransactions() const {
+  virtual bool supportsMoreTransactions() const {
     return (getNumOutgoingStreams() < getMaxConcurrentOutgoingStreams());
   }
 
@@ -345,7 +352,8 @@ class HTTPSessionBase : public wangle::ManagedConnection {
   }
 
   std::chrono::seconds getLatestIdleTime() const {
-    DCHECK_GT(transactionSeqNo_, 0) << "No idle time for the first transaction";
+    DCHECK_GT(transactionSeqNo_, 0u)
+        << "No idle time for the first transaction";
     DCHECK(latestActive_ > TimePoint::min());
     return latestIdleDuration_;
   }
@@ -422,7 +430,7 @@ class HTTPSessionBase : public wangle::ManagedConnection {
   // Implments a map from generic priority level to HTTP/2 priority.
   class PriorityAdapter {
    public:
-    virtual folly::Optional<const HTTPMessage::HTTPPriority> getHTTPPriority(
+    virtual folly::Optional<const HTTPMessage::HTTP2Priority> getHTTPPriority(
         uint8_t level) = 0;
     virtual ~PriorityAdapter() = default;
   };
@@ -480,7 +488,7 @@ class HTTPSessionBase : public wangle::ManagedConnection {
    */
   virtual void drain() = 0;
 
-  virtual folly::Optional<const HTTPMessage::HTTPPriority> getHTTPPriority(
+  virtual folly::Optional<const HTTPMessage::HTTP2Priority> getHTTPPriority(
       uint8_t level) = 0;
 
   /**
@@ -492,8 +500,6 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     return exHeadersEnabled_;
   }
 
-  virtual void injectTraceEventIntoAllTransactions(TraceEvent&) = 0;
-
   void setConnectionToken(
       const HTTPTransaction::ConnectionToken& token) noexcept {
     connectionToken_ = token;
@@ -511,6 +517,10 @@ class HTTPSessionBase : public wangle::ManagedConnection {
                                 bool extendIntervalOnIngress,
                                 bool immediate = false) = 0;
 
+  void setIngressTimeoutAfterEom(bool setIngressTimeoutAfterEom) noexcept {
+    setIngressTimeoutAfterEom_ = setIngressTimeoutAfterEom;
+  }
+
  protected:
   bool notifyEgressBodyBuffered(int64_t bytes, bool update);
 
@@ -518,11 +528,9 @@ class HTTPSessionBase : public wangle::ManagedConnection {
 
   void updatePendingWrites();
 
-  virtual void pauseTransactions() = 0;
-
-  virtual void resumeTransactions();
-
-  virtual void setNewTransactionPauseState(HTTPTransaction* txn) = 0;
+  bool hasPendingEgress() {
+    return pendingWriteSize_ + pendingWriteSizeDelta_ > 0;
+  }
 
   /**
    * Install a direct response handler for the transaction based on the
@@ -649,6 +657,13 @@ class HTTPSessionBase : public wangle::ManagedConnection {
    */
   folly::Optional<HTTPTransaction::ConnectionToken> connectionToken_;
 
+  /**
+   * Indicates whether ingress timeout has to be scheduled after EOM is sent.
+   */
+  bool setIngressTimeoutAfterEom_{false};
+
+  std::unique_ptr<HTTPSessionActivityTracker> httpSessionActivityTracker_;
+
  private:
   // Underlying controller_ is marked as private so that callers must utilize
   // getController/setController protected methods.  This ensures we have a
