@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -28,6 +28,8 @@ class HTTPSessionStats : public TTLBAStats {
   }
   virtual void recordTransactionStalled() noexcept = 0;
   virtual void recordSessionStalled() noexcept = 0;
+  virtual void recordPendingBufferedReadBytes(int64_t) noexcept = 0;
+  virtual void recordEgressContentLengthMismatches() noexcept = 0;
 };
 
 } // namespace proxygen
