@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -36,6 +36,24 @@
 
 namespace proxygen {
 
+/**
+ * Experimental
+ *
+ * A sender interface to send out DSR delegated packetization requests.
+ */
+class DSRRequestSender {
+ public:
+  virtual ~DSRRequestSender() = default;
+
+  // This is called back when the underlying session has generated the header
+  // bytes for the transaction. At this point it is the responsibility of the
+  // DSRRequestSender to call addBufferMeta and sendEOM so that the buffer meta
+  // data can start flowing through the transport. The parameter is the offset
+  // at which the DSR data begins.
+  virtual void onHeaderBytesGenerated(size_t /*dsrDataStartingOffset*/) {
+  }
+};
+
 /**
  * An HTTPTransaction represents a single request/response pair
  * for some HTTP-like protocol.  It works with a Transport that
@@ -229,6 +247,15 @@ class HTTPTransactionHandler : public TraceEventObserver {
    */
   virtual void onError(const HTTPException& error) noexcept = 0;
 
+  /**
+   * Can be called at any time before detachTransaction(). This callback is
+   * invoked in cases that violate an internal invariant that is fatal to the
+   * transaction but can be recoverable for the session or library.  One such
+   * example is mis-use of the egress APIs (sendBody() before sendHeaders()).
+   */
+  virtual void onInvariantViolation(const HTTPException& error) noexcept {
+    LOG(FATAL) << error.what();
+  }
   /**
    * If the remote side's receive buffer fills up, this callback will be
    * invoked so you can attempt to stop sending to the remote side.
@@ -269,33 +296,16 @@ class HTTPTransactionHandler : public TraceEventObserver {
   }
 
   /**
-   * Inform the handler that unframed body is starting.
+   * Can be called multiple times per transaction after onHeadersComplete and
+   * before detachTransaction()
+   *
+   * It does not obey pauseIngress/resumeIngress it is up to the handler
+   * to decide whether to buffer/drop datagrams
    */
-  virtual void onUnframedBodyStarted(uint64_t /* offset */) noexcept {
+  virtual void onDatagram(std::unique_ptr<folly::IOBuf> /*datagram*/) noexcept {
   }
 
-  /**
-   * Inform the handler that data arrived into underlying transport's read
-   * buffer.
-   */
-  virtual void onBodyPeek(uint64_t /* offset */,
-                          const folly::IOBuf& /* chain */) noexcept {
-  }
-
-  /**
-   * Inform the handler that the sender skipped data below certain offset.
-   */
-  virtual void onBodySkipped(uint64_t /* offset */) noexcept {
-  }
-
-  /**
-   * Inform the handler that the receiver doesn't expect data under certain
-   * offset anymore.
-   */
-  virtual void onBodyRejected(uint64_t /* offset */) noexcept {
-  }
-
-  virtual ~HTTPTransactionHandler() {
+  virtual ~HTTPTransactionHandler() override {
   }
 };
 
@@ -371,6 +381,7 @@ class HTTPTransactionTransportCallback {
 
   virtual void headerBytesReceived(const HTTPHeaderSize& size) noexcept = 0;
 
+  // May include extra bytes for EOF/trailers
   virtual void bodyBytesGenerated(size_t nbytes) noexcept = 0;
 
   virtual void bodyBytesReceived(size_t size) noexcept = 0;
@@ -378,6 +389,9 @@ class HTTPTransactionTransportCallback {
   virtual void lastEgressHeaderByteAcked() noexcept {
   }
 
+  virtual void bodyBytesTx(uint64_t /* bodyOffset */) noexcept {
+  }
+
   virtual void bodyBytesDelivered(uint64_t /* bodyOffset */) noexcept {
   }
 
@@ -387,6 +401,11 @@ class HTTPTransactionTransportCallback {
   virtual void transportAppRateLimited() noexcept {
   }
 
+  virtual void datagramBytesGenerated(size_t /* nbytes */) noexcept {
+  }
+  virtual void datagramBytesReceived(size_t /* size */) noexcept {
+  }
+
   virtual ~HTTPTransactionTransportCallback() {
   }
 };
@@ -399,10 +418,6 @@ class HTTPTransaction
   using Handler = HTTPTransactionHandler;
   using PushHandler = HTTPPushTransactionHandler;
 
-  using PeekCallback =
-      const folly::Function<void(HTTPCodec::StreamID streamId,
-                                 uint64_t /* bodyOffset */,
-                                 const folly::IOBuf& /* chain */) const>&;
   struct FlowControlInfo {
     bool flowControlEnabled_{false};
     int64_t sessionSendWindow_{-1};
@@ -415,6 +430,26 @@ class HTTPTransaction
     int64_t streamRecvOutstanding_{-1};
   };
 
+  /**
+   * Experimental.
+   *
+   * BufferMeta represents a buffer. The real data will be sourced from another
+   * place.
+   */
+  struct BufferMeta {
+    size_t length{0};
+
+    BufferMeta() = default;
+    explicit BufferMeta(size_t lengthIn) : length(lengthIn) {
+    }
+
+    BufferMeta split(size_t splitLen) {
+      CHECK_GE(length, splitLen);
+      length -= splitLen;
+      return BufferMeta(splitLen);
+    }
+  };
+
   /**
    * Opaque token that identifies the underlying connection.
    * Transaction handlers can use this token to group different
@@ -441,8 +476,32 @@ class HTTPTransaction
                              HTTPHeaderSize* size,
                              bool eom) noexcept = 0;
 
+    /**
+     * Experimental API
+     *
+     * Send headers and a DSRRequestSender to Transport.
+     * The Transport will generate header for DATA frame.
+     *
+     * dataFrameHeaderSize: An output parameter to get the size of DATA frame
+     * header.
+     */
+    virtual bool sendHeadersWithDelegate(
+        HTTPTransaction*,
+        const HTTPMessage&,
+        HTTPHeaderSize*,
+        size_t* /* dataFrameHeaderSize */,
+        uint64_t /* contentLength */,
+        std::unique_ptr<DSRRequestSender>) noexcept {
+      return false;
+    }
+
+    // Experimental
+    virtual size_t sendBody(HTTPTransaction*,
+                            const BufferMeta&,
+                            bool /* eom */) noexcept = 0;
+
     virtual size_t sendBody(HTTPTransaction* txn,
-                            std::unique_ptr<folly::IOBuf>,
+                            std::unique_ptr<folly::IOBuf> iobuf,
                             bool eom,
                             bool trackLastByteFlushed) noexcept = 0;
 
@@ -459,6 +518,12 @@ class HTTPTransaction
 
     virtual size_t sendPriority(HTTPTransaction* txn,
                                 const http2::PriorityUpdate& pri) noexcept = 0;
+    /*
+     * Updates the Local priority for the transaction.
+     * For an upstream transaction it also sends the priority update to the peer
+     */
+    virtual size_t changePriority(HTTPTransaction* txn,
+                                  HTTPPriority pri) noexcept = 0;
 
     virtual size_t sendWindowUpdate(HTTPTransaction* txn,
                                     uint32_t bytes) noexcept = 0;
@@ -477,15 +542,15 @@ class HTTPTransaction
 
     virtual void describe(std::ostream&) const = 0;
 
-    virtual const wangle::TransportInfo& getSetupTransportInfo() const
-        noexcept = 0;
+    virtual const wangle::TransportInfo& getSetupTransportInfo()
+        const noexcept = 0;
 
     virtual bool getCurrentTransportInfo(wangle::TransportInfo* tinfo) = 0;
 
     virtual void getFlowControlInfo(FlowControlInfo* info) = 0;
 
-    virtual HTTPTransaction::Transport::Type getSessionType() const
-        noexcept = 0;
+    virtual HTTPTransaction::Transport::Type getSessionType()
+        const noexcept = 0;
 
     virtual const HTTPCodec& getCodec() const noexcept = 0;
 
@@ -517,8 +582,8 @@ class HTTPTransaction
 
     virtual bool needToBlockForReplaySafety() const = 0;
 
-    virtual const folly::AsyncTransport* getUnderlyingTransport() const
-        noexcept = 0;
+    virtual const folly::AsyncTransport* getUnderlyingTransport()
+        const noexcept = 0;
 
     /**
      * Returns true if the underlying transport has completed full handshake.
@@ -530,35 +595,18 @@ class HTTPTransaction
 
     virtual HTTPSessionBase* getHTTPSessionBase() = 0;
 
-    virtual folly::Optional<const HTTPMessage::HTTPPriority> getHTTPPriority(
+    virtual folly::Optional<const HTTPMessage::HTTP2Priority> getHTTPPriority(
         uint8_t level) = 0;
 
-    virtual folly::Expected<folly::Unit, ErrorCode> peek(
-        PeekCallback /* peekCallback */) {
-      LOG(FATAL) << __func__ << " not supported";
-      folly::assume_unreachable();
+    virtual folly::Optional<HTTPPriority> getHTTPPriority() {
+      return folly::none;
     }
 
-    virtual folly::Expected<folly::Unit, ErrorCode> consume(
-        size_t /* amount */) {
-      LOG(FATAL) << __func__ << " not supported";
-      folly::assume_unreachable();
+    virtual uint16_t getDatagramSizeLimit() const noexcept {
+      return 0;
     }
 
-    /**
-     * Notify peer that the data below the offset isn't going to be sent.
-     */
-    virtual folly::Expected<folly::Optional<uint64_t>, ErrorCode> skipBodyTo(
-        HTTPTransaction* /* txn */, uint64_t /* nextBodyOffset */) {
-      LOG(FATAL) << __func__ << " not supported";
-      folly::assume_unreachable();
-    }
-
-    /**
-     * Notify peer that the data below the offset is not needed anymore.
-     */
-    virtual folly::Expected<folly::Optional<uint64_t>, ErrorCode> rejectBodyTo(
-        HTTPTransaction* /* txn */, uint64_t /* nextBodyOffset */) {
+    virtual bool sendDatagram(std::unique_ptr<folly::IOBuf> /*datagram*/) {
       LOG(FATAL) << __func__ << " not supported";
       folly::assume_unreachable();
     }
@@ -566,7 +614,8 @@ class HTTPTransaction
     /**
      * Ask transport to track and ack body delivery.
      */
-    virtual void trackEgressBodyDelivery(uint64_t /* bodyOffset */) {
+    virtual void trackEgressBodyOffset(uint64_t /* bodyOffset */,
+                                       ByteEvent::EventFlags /*flags*/) {
       LOG(FATAL) << __func__ << " not supported";
       folly::assume_unreachable();
     }
@@ -602,7 +651,8 @@ class HTTPTransaction
       http2::PriorityUpdate = http2::DefaultPriority,
       folly::Optional<HTTPCodec::StreamID> assocStreamId = HTTPCodec::NoStream,
       folly::Optional<HTTPCodec::ExAttributes> exAttributes =
-          HTTPCodec::NoExAttributes);
+          HTTPCodec::NoExAttributes,
+      bool setIngressTimeoutAfterEom = false);
 
   ~HTTPTransaction() override;
 
@@ -647,10 +697,15 @@ class HTTPTransaction
   }
 
   std::tuple<uint64_t, uint64_t, double> getPrioritySummary() const {
-    return std::make_tuple(insertDepth_,
-                           currentDepth_,
-                           egressCalls_ > 0 ? cumulativeRatio_ / egressCalls_
-                                            : 0);
+    return std::make_tuple(
+        insertDepth_,
+        currentDepth_,
+        egressCalls_ > 0 ? cumulativeRatio_ / static_cast<double>(egressCalls_)
+                         : 0);
+  }
+
+  folly::Optional<HTTPPriority> getHTTPPriority() const {
+    return transport_.getHTTPPriority();
   }
 
   bool getPriorityFallback() const {
@@ -819,16 +874,6 @@ class HTTPTransaction
    */
   void onIngressSetSendWindow(uint32_t newWindowSize);
 
-  /**
-   * Ivoked by the session when it gets the start of the unframed body.
-   */
-  void onIngressUnframedBodyStarted(uint64_t offset) {
-    partiallyReliable_ = true;
-    if (handler_) {
-      handler_->onUnframedBodyStarted(offset);
-    }
-  }
-
   /**
    * Notify this transaction that it is ok to egress.  Returns true if there
    * is additional pending egress
@@ -874,6 +919,12 @@ class HTTPTransaction
    */
   void onLastEgressHeaderByteAcked();
 
+  /**
+   * Invoked by the session when egress body has been transmitted to the
+   * peer. Called for each sendBody() call if body bytes tracking is enabled.
+   */
+  void onEgressBodyBytesTx(uint64_t bodyOffset);
+
   /**
    * Invoked by the session when egress body has been acked by the
    * peer. Called for each sendBody() call if body bytes tracking is enabled.
@@ -881,8 +932,8 @@ class HTTPTransaction
   void onEgressBodyBytesAcked(uint64_t bodyOffset);
 
   /**
-   * Invoked by the session when egress body delivery has been cancelled by the
-   * peer.
+   * Invoked by the session when egress body tx or delivery has been cancelled
+   * by the peer.
    */
   void onEgressBodyDeliveryCanceled(uint64_t bodyOffset);
 
@@ -906,30 +957,19 @@ class HTTPTransaction
   void onEgressTransportAppRateLimited();
 
   /**
-   * Invoked by the session when data to peek into is available on trasport
-   * layer.
-   */
-  void onIngressBodyPeek(uint64_t bodyOffset, const folly::IOBuf& chain);
-
-  /**
-   * Invoked by the session when transaction receives a skip from the peer.
+   * Can be called multiple times per transaction after onHeadersComplete and
+   * before detachTransaction()
    *
-   * @param nextBodyOffset      Next body offset set by the sender.
+   * It does not obey pauseIngress/resumeIngress it is up to the handler
+   * to decide whether to buffer/drop datagrams
    */
-  void onIngressBodySkipped(uint64_t nextBodyOffset);
-
-  /**
-   * Invoked by the session when transaction receives a reject from the peer.
-   *
-   * @param nextBodyOffset  Next body offset set by the receiver.
-   */
-  void onIngressBodyRejected(uint64_t nextBodyOffset);
+  void onDatagram(std::unique_ptr<folly::IOBuf> datagram) noexcept;
 
   /**
    * Invoked by the handlers that are interested in tracking
    * performance stats.
    */
-  void setTransportCallback(TransportCallback* cb) {
+  virtual void setTransportCallback(TransportCallback* cb) {
     transportCallback_ = cb;
   }
 
@@ -1031,6 +1071,15 @@ class HTTPTransaction
   virtual void sendHeadersWithEOM(const HTTPMessage& headers);
   virtual void sendHeadersWithOptionalEOM(const HTTPMessage& headers, bool eom);
 
+  /**
+   * Experimental API
+   *
+   * Send the egress message header and a DSRRequestSender to the Transport.
+   * Handler does NOT have to explicitly sendBody and sendEOM after this.
+   */
+  virtual bool sendHeadersWithDelegate(const HTTPMessage& headers,
+                                       std::unique_ptr<DSRRequestSender>);
+
   /**
    * Send part or all of the egress message body to the Transport. If flow
    * control is enabled, the chunk boundaries may not be respected.
@@ -1045,6 +1094,13 @@ class HTTPTransaction
    */
   virtual void sendBody(std::unique_ptr<folly::IOBuf> body);
 
+  /**
+   * Returns the cumulative size of body passed to sendBody so far
+   */
+  size_t bodyBytesSent() const {
+    return actualResponseLength_.value_or(0);
+  }
+
   /**
    * Write any protocol framing required for the subsequent call(s)
    * to sendBody(). This method does not actually write the message out on
@@ -1053,13 +1109,15 @@ class HTTPTransaction
    * @param length  Length in bytes of the body data to follow.
    */
   virtual void sendChunkHeader(size_t length) {
-    CHECK(HTTPTransactionEgressSM::transit(
-        egressState_, HTTPTransactionEgressSM::Event::sendChunkHeader));
-    CHECK(!partiallyReliable_)
-        << __func__ << ": chunking not supported in partially reliable mode.";
+    if (!validateEgressStateTransition(
+            HTTPTransactionEgressSM::Event::sendChunkHeader)) {
+      return;
+    }
+    CHECK_EQ(deferredBufferMeta_.length, 0)
+        << "Chunked-encoding doesn't support BufferMeta write";
     // TODO: move this logic down to session/codec
     if (!transport_.getCodec().supportsParallelRequests()) {
-      chunkHeaders_.emplace_back(Chunk(length));
+      chunkHeaders_.emplace_back(length);
     }
   }
 
@@ -1070,10 +1128,10 @@ class HTTPTransaction
    * Frame begun by the last call to sendChunkHeader().
    */
   virtual void sendChunkTerminator() {
-    CHECK(HTTPTransactionEgressSM::transit(
-        egressState_, HTTPTransactionEgressSM::Event::sendChunkTerminator));
-    CHECK(!partiallyReliable_)
-        << __func__ << ": chunking not supported in partially reliable mode.";
+    validateEgressStateTransition(
+        HTTPTransactionEgressSM::Event::sendChunkTerminator);
+    CHECK_EQ(deferredBufferMeta_.length, 0)
+        << "Chunked-encoding doesn't support BufferMeta write";
   }
 
   /**
@@ -1085,11 +1143,10 @@ class HTTPTransaction
    * @param trailers  Message trailers.
    */
   virtual void sendTrailers(const HTTPHeaders& trailers) {
-    CHECK(HTTPTransactionEgressSM::transit(
-        egressState_, HTTPTransactionEgressSM::Event::sendTrailers));
-    CHECK(!partiallyReliable_)
-        << __func__
-        << ": trailers are not supported in partially reliable mode.";
+    if (!validateEgressStateTransition(
+            HTTPTransactionEgressSM::Event::sendTrailers)) {
+      return;
+    }
     trailers_.reset(new HTTPHeaders(trailers));
   }
 
@@ -1207,11 +1264,13 @@ class HTTPTransaction
    */
   virtual HTTPTransaction* newPushedTransaction(
       HTTPPushTransactionHandler* handler, ProxygenError* error = nullptr) {
-    // Pushed transactions do support partially reliable mode, however push
-    // promises should be only generated on a fully reliable transaction.
-    CHECK(!partiallyReliable_)
-        << __func__
-        << ": push promises not supported in partially reliable mode.";
+    if (isDelegated_) {
+      LOG(ERROR)
+          << "Creating Pushed transaction on a delegated HTTPTransaction "
+          << "is not supported.";
+      return nullptr;
+    }
+
     if (isEgressEOMSeen()) {
       SET_PROXYGEN_ERROR_IF(error,
                             ProxygenError::kErrorEgressEOMSeenOnParentStream);
@@ -1233,6 +1292,11 @@ class HTTPTransaction
    */
   virtual HTTPTransaction* newExTransaction(HTTPTransactionHandler* handler,
                                             bool unidirectional = false) {
+    if (isDelegated_) {
+      LOG(ERROR) << "Creating ExTransaction on a delegated HTTPTransaction is "
+                 << "not supported.";
+      return nullptr;
+    }
     auto txn = transport_.newExTransaction(handler, id_, unidirectional);
     if (txn) {
       exTransactions_.insert(txn->getID());
@@ -1353,7 +1417,9 @@ class HTTPTransaction
    * Schedule or refresh the idle timeout for this transaction
    */
   void refreshTimeout() {
-    if (timer_ && hasIdleTimeout()) {
+    // TODO(T121147568): Remove the zero-check after the experiment is complete.
+    if (timer_ && hasIdleTimeout() &&
+        idleTimeout_.value() != std::chrono::milliseconds::zero()) {
       timer_->scheduleTimeout(this, idleTimeout_.value());
     }
   }
@@ -1423,10 +1489,13 @@ class HTTPTransaction
   void describe(std::ostream& os) const;
 
   /**
-   * Change the priority of this transaction, may generate a PRIORITY frame
+   * Change the priority of this transaction, may generate a PRIORITY frame.
+   * The first variant is SPDY priority. The second is HTTP/2 priority. The
+   * third one is a new proposal in a draft for both HTTP/2 and HTTP/3.
    */
   void updateAndSendPriority(int8_t newPriority);
   void updateAndSendPriority(const http2::PriorityUpdate& pri);
+  virtual void updateAndSendPriority(uint8_t urgency, bool incremental);
 
   /**
    * Notify of priority change, will not generate a PRIORITY frame
@@ -1483,88 +1552,54 @@ class HTTPTransaction
   const CompressionInfo& getCompressionInfo() const;
 
   bool hasPendingBody() const {
-    return deferredEgressBody_.chainLength() > 0;
+    return getOutstandingEgressBodyBytes() > 0;
   }
 
   size_t getOutstandingEgressBodyBytes() const {
-    return deferredEgressBody_.chainLength();
+    return deferredEgressBody_.chainLength() + deferredBufferMeta_.length;
   }
 
   void setLastByteFlushedTrackingEnabled(bool enabled) {
     enableLastByteFlushedTracking_ = enabled;
   }
 
-  bool setBodyLastByteDeliveryTrackingEnabled(bool enabled) {
-    if (transport_.getSessionType() != Transport::Type::QUIC) {
-      return false;
-    }
+  // Use this API to track TX or Ack for a particular offset of the HTTP body,
+  // if the underlying transport is capable of tracking.
+  // It will generate a callback to HTTPTransactionTransportCallback either
+  // trackedByteEventTx or trackedByteEventAck.  The the event does not happen,
+  // there is no cancellation callback.
+  //
+  // You can call this API before you have egressed the given bodyOffset.
+  // Last byte ack is already tracked implicity and delivered via lastByteAcked.
+  bool trackEgressBodyOffset(
+      uint64_t bodyOffset,
+      ByteEvent::EventFlags flags = ByteEvent::EventFlags::ACK);
 
-    enableBodyLastByteDeliveryTracking_ = enabled;
-    return true;
-  }
-
-  /**
-   * Allows the caller to peek into underlying transport's read buffer.
-   * This, together with consume(), forms a scatter/gather API.
-   *
-   * @param peekCallback  A callback that will be executed on each contiguous
-   *                      byte range in transport's read buffer. Number of byte
-   *                      ranges is determined by the number of gaps in the
-   *                      read buffer.
-   */
-  folly::Expected<folly::Unit, ErrorCode> peek(PeekCallback peekCallback);
-
-  /**
-   * Allows the caller to consume bytes from the beginning of the read buffer in
-   * the underlying transport's read buffer.
-   * This is useful when e.g. we know that the transaction is
-   * head-of-line-blocked and we are willing to get rid of existing bytes in the
-   * buffer to allow the transaction to proceed. For example:
-   *    - read buffer pointer is at 3
-   *    - we have bytes range [5, 18] ready in the buffer
-   * If we don't want to wait to receive bytes [3, 4], we can call consume(2)
-   * and the read buffer pointer would be moved to 5 allowing read operations to
-   * proceed.
-   *
-   * @param amount  Number of bytes to consume from transport's read buffer.
-   *                Gaps will be consumed together with received bytes in the
-   *                buffer. Bytes will be always consumed from current read
-   *                buffer front pointer.
-   */
-  folly::Expected<folly::Unit, ErrorCode> consume(size_t amount);
-
-  /**
-   * Allows the sender to skip part of the egress body. Calls can be interleaved
-   * with sendBody() calls.
-   * Upon receipt by the peer, this signals that the body up to bodyOffset shall
-   * not be expected. Note that some bytes before this new advertised offset
-   * might still be received by the peer if they were already in transmission
-   * between the peers.
-   *
-   * @param bodyOffset  New offset the sender is going to start sending the body
-   *                    from going forward.
-   */
-  folly::Expected<folly::Optional<uint64_t>, ErrorCode> skipBodyTo(
-      uint64_t nextBodyOffset);
-
-  /**
-   * Similar to skipBodyTo() above, rejectBodyTo() allows the receiver to signal
-   * to the sender that body bytes below bodyOffset are not expected anymore and
-   * the sender needs to start sending from the new offset.
-   * Note that the receiver may still receive some bytes below this new
-   * advertised offset if any were in flight between the two peers.
-   *
-   * @param bodyOffset  New offset the sender needs to start sending from.
-   */
-  folly::Expected<folly::Optional<uint64_t>, ErrorCode> rejectBodyTo(
-      uint64_t nextBodyOffset);
+  uint16_t getDatagramSizeLimit() const noexcept;
+  virtual bool sendDatagram(std::unique_ptr<folly::IOBuf> datagram);
 
   folly::Optional<ConnectionToken> getConnectionToken() const noexcept;
 
+  static void setEgressBufferLimit(uint64_t limit) {
+    egressBufferLimit_ = limit;
+  }
+
+  virtual bool addBufferMeta() noexcept;
+
  private:
   HTTPTransaction(const HTTPTransaction&) = delete;
   HTTPTransaction& operator=(const HTTPTransaction&) = delete;
 
+  void dequeue() {
+    CHECK(isEnqueued());
+    egressQueue_.clearPendingEgress(queueHandle_);
+  }
+
+  bool delegatedTransactionChecks(const HTTPMessage& headers) noexcept;
+  bool delegatedTransactionChecks() noexcept;
+
+  void abortAndDeliverError(ErrorCode codecErorr, const std::string& msg);
+
   void onDelayedDestroy(bool delayed) override;
 
   /**
@@ -1614,19 +1649,18 @@ class HTTPTransaction
 
   size_t sendDeferredBody(uint32_t maxEgress);
 
+  size_t sendDeferredBufferMeta(uint32_t maxEgress);
+
   bool maybeDelayForRateLimit();
 
   bool isEnqueued() const {
-    return queueHandle_->isEnqueued();
-  }
-
-  void dequeue() {
-    DCHECK(isEnqueued());
-    egressQueue_.clearPendingEgress(queueHandle_);
+    return queueHandle_ ? queueHandle_->isEnqueued() : false;
   }
 
+  // Whther the txn has a pending EOM that can be send out (i.e., no more body
+  // bytes need to go before it.)
   bool hasPendingEOM() const {
-    return isEgressEOMQueued() && deferredEgressBody_.chainLength() == 0;
+    return isEgressEOMQueued() && getOutstandingEgressBodyBytes() == 0;
   }
 
   bool isExpectingIngress() const;
@@ -1649,11 +1683,22 @@ class HTTPTransaction
 
   /**
    * Validates the ingress state transition. Returns false and sends an
-   * abort with PROTOCOL_ERROR if the transition fails. Otherwise it
+   * abort with INTERNAL_ERROR if the transition fails. Otherwise it
    * returns true.
    */
   bool validateIngressStateTransition(HTTPTransactionIngressSM::Event);
 
+  /**
+   * Validates the egress state transition.
+   *
+   * If the transition fails, it will invoke onInvariantViolation, and the
+   * default implementation is to CHECK/crash.  If you have a custom
+   * onInvariantViolation handler, this function can return false.
+   */
+  bool validateEgressStateTransition(HTTPTransactionEgressSM::Event);
+
+  void invariantViolation(HTTPException ex);
+
   /**
    * Flushes any pending window updates.  This can happen from setReceiveWindow
    * or sendHeaders depending on transaction state.
@@ -1664,8 +1709,6 @@ class HTTPTransaction
 
   void rateLimitTimeoutExpired();
 
-  void trimDeferredEgressBody(uint64_t bodyOffset);
-
   class RateLimitCallback : public folly::HHWheelTimer::Callback {
    public:
     explicit RateLimitCallback(HTTPTransaction& txn) : txn_(txn) {
@@ -1696,6 +1739,11 @@ class HTTPTransaction
    */
   folly::IOBufQueue deferredEgressBody_{folly::IOBufQueue::cacheChainLength()};
 
+  /**
+   * BufferMeta queued at this HTTPTransaction to be sent to Transport.
+   */
+  BufferMeta deferredBufferMeta_;
+
   const TransportDirection direction_;
   HTTPTransactionEgressSM::State egressState_{
       HTTPTransactionEgressSM::getNewInstance()};
@@ -1751,7 +1799,7 @@ class HTTPTransaction
   /**
    * Handle to our position in the priority queue.
    */
-  HTTP2PriorityQueueBase::Handle queueHandle_;
+  HTTP2PriorityQueueBase::Handle queueHandle_{nullptr};
 
   /**
    * ID of request transaction (for pushed txns only)
@@ -1799,9 +1847,8 @@ class HTTPTransaction
   folly::Optional<uint64_t> expectedIngressContentLengthRemaining_;
   folly::Optional<uint64_t> expectedResponseLength_;
   folly::Optional<uint64_t> actualResponseLength_{0};
-  // Keeps track of how many bytes the transaction passed to the transport so
-  // far.
-  uint64_t egressBodyBytesCommittedToTransport_{0};
+  uint64_t bodyBytesEgressed_{0};
+  std::map<uint64_t, ByteEvent::EventFlags> egressBodyOffsetsToTrack_;
 
   bool ingressPaused_ : 1;
   bool egressPaused_ : 1;
@@ -1819,16 +1866,17 @@ class HTTPTransaction
   bool priorityFallback_ : 1;
   bool headRequest_ : 1;
   bool enableLastByteFlushedTracking_ : 1;
-  bool enableBodyLastByteDeliveryTracking_ : 1;
-  // Signals if the transaction is partially reliable.
-  // Set on first sendHeaders() call on egress or with setPartiallyReliable() on
-  // ingress.
-  bool partiallyReliable_ : 1;
 
   // Prevents the application from calling skipBodyTo() before egress
   // headers have been delivered.
   bool egressHeadersDelivered_ : 1;
 
+  // Whether the HTTPTransaction has sent out a 1xx response HTTPMessage.
+  bool has1xxResponse_ : 1;
+
+  // Whether this HTTPTransaction delegates body sending to another entity.
+  bool isDelegated_ : 1;
+
   /**
    * If this transaction represents a request (ie, it is backed by an
    * HTTPUpstreamSession) , this field indicates the last response status
@@ -1838,6 +1886,7 @@ class HTTPTransaction
    */
   uint16_t lastResponseStatus_{0};
 
+  // Maximum size of egress buffer before invoking onEgressPaused
   static uint64_t egressBufferLimit_;
 
   uint64_t egressLimitBytesPerMs_{0};
@@ -1852,8 +1901,9 @@ class HTTPTransaction
   std::unique_ptr<PrioritySample> prioritySample_;
 
   // Keeps track for body offset processed so far.
-  // Includes skipped bytes for partially reliable transactions.
   uint64_t ingressBodyOffset_{0};
+
+  bool setIngressTimeoutAfterEom_{false};
 };
 
 /**
