@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -18,6 +18,7 @@ class HTTPTransactionEgressSMData {
   enum class State : uint8_t {
     Start,
     HeadersSent,
+    DatagramSent,
     RegularBodySent,
     ChunkHeaderSent,
     ChunkBodySent,
@@ -33,6 +34,7 @@ class HTTPTransactionEgressSMData {
   enum class Event : uint8_t {
     // API accessible transitions
     sendHeaders,
+    sendDatagram,
     sendBody,
     sendChunkHeader,
     sendChunkTerminator,
