@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -18,7 +18,8 @@ class HTTPTransactionIngressSMData {
  public:
   enum class State : uint8_t {
     Start,
-    HeadersReceived,
+    NonFinalHeadersReceived,
+    FinalHeadersReceived,
     RegularBodyReceived,
     ChunkHeaderReceived,
     ChunkBodyReceived,
@@ -34,7 +35,9 @@ class HTTPTransactionIngressSMData {
 
   enum class Event : uint8_t {
     // API accessible transitions
-    onHeaders,
+    onNonFinalHeaders,
+    onFinalHeaders,
+    onDatagram,
     onBody,
     onChunkHeader,
     onChunkComplete,
