@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,7 +8,7 @@
 
 #include <folly/io/async/EventBase.h>
 #include <folly/io/async/test/MockAsyncTransport.h>
-#include <proxygen/lib/http/codec/SPDYConstants.h>
+#include <proxygen/lib/http/codec/HTTP2Constants.h>
 #include <proxygen/lib/http/codec/test/MockHTTPCodec.h>
 #include <proxygen/lib/http/codec/test/TestUtils.h>
 #include <proxygen/lib/http/session/test/HTTPSessionMocks.h>
@@ -32,13 +32,13 @@ class DownstreamTransactionTest : public testing::Test {
   void setupRequestResponseFlow(HTTPTransaction* txn,
                                 uint32_t size,
                                 bool delayResponse = false) {
-    EXPECT_CALL(handler_, setTransaction(txn));
-    EXPECT_CALL(handler_, detachTransaction());
+    EXPECT_CALL(handler_, _setTransaction(txn));
+    EXPECT_CALL(handler_, _detachTransaction());
     EXPECT_CALL(transport_, detach(txn));
     if (delayResponse) {
-      EXPECT_CALL(handler_, onHeadersComplete(_));
+      EXPECT_CALL(handler_, _onHeadersComplete(_));
     } else {
-      EXPECT_CALL(handler_, onHeadersComplete(_))
+      EXPECT_CALL(handler_, _onHeadersComplete(_))
           .WillOnce(Invoke([=](std::shared_ptr<HTTPMessage> /*msg*/) {
             auto response = makeResponse(200);
             txn->sendHeaders(*response.get());
@@ -69,12 +69,12 @@ class DownstreamTransactionTest : public testing::Test {
             return 5;
           }));
     }
-    EXPECT_CALL(handler_, onBodyWithOffset(_, _))
+    EXPECT_CALL(handler_, _onBodyWithOffset(_, _))
         .WillRepeatedly(
             Invoke([=](uint64_t, std::shared_ptr<folly::IOBuf> body) {
               received_ += body->computeChainDataLength();
             }));
-    EXPECT_CALL(handler_, onEOM()).WillOnce(InvokeWithoutArgs([=] {
+    EXPECT_CALL(handler_, _onEOM()).WillOnce(InvokeWithoutArgs([=] {
       CHECK_EQ(received_, size);
     }));
     EXPECT_CALL(transport_, notifyPendingEgress())
@@ -86,17 +86,47 @@ class DownstreamTransactionTest : public testing::Test {
 
  protected:
   folly::EventBase eventBase_;
-  folly::HHWheelTimer::UniquePtr transactionTimeouts_{
-      folly::HHWheelTimer::newTimer(
-          &eventBase_,
-          std::chrono::milliseconds(folly::HHWheelTimer::DEFAULT_TICK_INTERVAL),
-          folly::AsyncTimeout::InternalEnum::NORMAL,
-          std::chrono::milliseconds(500))};
   MockHTTPTransactionTransport transport_;
   StrictMock<MockHTTPHandler> handler_;
   HTTP2PriorityQueue txnEgressQueue_;
   uint32_t received_{0};
   uint32_t sent_{0};
+  std::unique_ptr<HTTPTransaction> txn_;
+
+  HTTPTransaction& makeTxn(bool useFlowControl = false,
+                           uint32_t receiveInitialWindowSize = 0,
+                           uint32_t sendInitialWindowSize = 0) {
+    txn_ = std::make_unique<HTTPTransaction>(TransportDirection::DOWNSTREAM,
+                                             HTTPCodec::StreamID(1),
+                                             1,
+                                             transport_,
+                                             txnEgressQueue_,
+                                             &eventBase_.timer(),
+                                             std::chrono::milliseconds(500),
+                                             nullptr,
+                                             useFlowControl,
+                                             receiveInitialWindowSize,
+                                             sendInitialWindowSize);
+    return *txn_;
+  }
+
+  HTTPTransaction& makeExTxn() {
+    txn_ = std::make_unique<HTTPTransaction>(TransportDirection::DOWNSTREAM,
+                                             HTTPCodec::StreamID(2),
+                                             1,
+                                             transport_,
+                                             txnEgressQueue_,
+                                             &eventBase_.timer(),
+                                             std::chrono::milliseconds(500),
+                                             nullptr,
+                                             false,
+                                             0,
+                                             0,
+                                             http2::DefaultPriority,
+                                             HTTPCodec::NoStream,
+                                             HTTPCodec::ExAttributes(1, true));
+    return *txn_;
+  }
 };
 
 /**
@@ -105,34 +135,43 @@ class DownstreamTransactionTest : public testing::Test {
  */
 TEST_F(DownstreamTransactionTest, SimpleCallbackForwarding) {
   // flow control is disabled
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500));
+  auto& txn = makeTxn();
   setupRequestResponseFlow(&txn, 100);
 
   txn.onIngressHeadersComplete(makeGetRequest());
   eventBase_.loop();
 }
 
+TEST_F(DownstreamTransactionTest, InvariantViolationHandler) {
+  auto& txn = makeTxn();
+
+  EXPECT_CALL(handler_, _setTransaction(&txn));
+  EXPECT_CALL(handler_, _onInvariantViolation(_))
+      .WillOnce(Invoke([](const HTTPException& ex) {
+        EXPECT_EQ(ex.getDirection(),
+                  HTTPException::Direction::INGRESS_AND_EGRESS);
+        EXPECT_EQ(ex.getErrno(), (int)HTTPException::Direction::EGRESS);
+        EXPECT_EQ(std::string(ex.what()),
+                  "Invalid egress state transition, state=Start, "
+                  "event=sendBody, streamID=1");
+      }));
+  EXPECT_CALL(transport_, sendAbort(_, _));
+  EXPECT_CALL(handler_, _detachTransaction());
+  EXPECT_CALL(transport_, detach(&txn));
+
+  txn.setHandler(&handler_);
+  // Send body before headers -- oops;
+  txn.sendBody(makeBuf(10));
+  eventBase_.loop();
+}
+
 /**
  * Testing that we're sending a window update for simple requests
  */
 TEST_F(DownstreamTransactionTest, RegularWindowUpdate) {
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500),
-                      nullptr,
-                      true, // flow control enabled
+  auto& txn = makeTxn(true, // flow control enabled
                       400,
-                      spdy::kInitialWindow);
+                      http2::kInitialWindow);
   uint32_t reqBodySize = 220;
   setupRequestResponseFlow(&txn, reqBodySize);
 
@@ -145,17 +184,9 @@ TEST_F(DownstreamTransactionTest, RegularWindowUpdate) {
 }
 
 TEST_F(DownstreamTransactionTest, NoWindowUpdate) {
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500),
-                      nullptr,
-                      true, // flow control enabled
+  auto& txn = makeTxn(true, // flow control enabled
                       450,  // more than 2x req size
-                      spdy::kInitialWindow);
+                      http2::kInitialWindow);
   uint32_t reqBodySize = 220;
   setupRequestResponseFlow(&txn, reqBodySize, true);
 
@@ -173,17 +204,7 @@ TEST_F(DownstreamTransactionTest, NoWindowUpdate) {
 }
 
 TEST_F(DownstreamTransactionTest, FlowControlInfoCorrect) {
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(157784760000),
-                      nullptr,
-                      true,
-                      450,
-                      100);
+  auto& txn = makeTxn(true, 450, 100);
 
   EXPECT_CALL(transport_, getFlowControlInfo(_))
       .WillOnce(Invoke([=](HTTPTransaction::FlowControlInfo* info) {
@@ -202,17 +223,7 @@ TEST_F(DownstreamTransactionTest, FlowControlInfoCorrect) {
 }
 
 TEST_F(DownstreamTransactionTest, ExpectingWindowUpdate) {
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(157784760000),
-                      nullptr,
-                      true,
-                      450,
-                      100);
+  auto& txn = makeTxn(true, 450, 100);
   uint32_t reqBodySize = 220;
 
   // Get a request, pause ingress, fill up the sendWindow, then expect for a
@@ -226,21 +237,11 @@ TEST_F(DownstreamTransactionTest, ExpectingWindowUpdate) {
   txn.sendBody(makeBuf(reqBodySize));
   txn.sendEOM();
   txn.onWriteReady(1000, 1);
-  EXPECT_EQ(transactionTimeouts_->count(), 1);
+  EXPECT_EQ(eventBase_.timer().count(), 1);
 }
 
 TEST_F(DownstreamTransactionTest, NoWindowUpdateAfterDoneSending) {
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(157784760000),
-                      nullptr,
-                      true,
-                      450,
-                      220);
+  auto& txn = makeTxn(true, 450, 220);
   uint32_t reqBodySize = 220;
 
   // Ensure that after flushing an EOM we are not expecting window update.
@@ -253,7 +254,7 @@ TEST_F(DownstreamTransactionTest, NoWindowUpdateAfterDoneSending) {
   txn.onWriteReady(1000, 1);
   txn.pauseIngress();
   txn.onIngressEOM();
-  EXPECT_EQ(transactionTimeouts_->count(), 0);
+  EXPECT_EQ(eventBase_.timer().count(), 0);
 }
 
 /**
@@ -262,24 +263,16 @@ TEST_F(DownstreamTransactionTest, NoWindowUpdateAfterDoneSending) {
  */
 TEST_F(DownstreamTransactionTest, WindowIncrease) {
   // set initial window size higher than per-stream window
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500),
-                      nullptr,
-                      true, // flow control enabled
-                      spdy::kInitialWindow,
-                      spdy::kInitialWindow);
+  auto& txn = makeTxn(true, // flow control enabled
+                      http2::kInitialWindow,
+                      http2::kInitialWindow);
   uint32_t reqSize = 500;
   setupRequestResponseFlow(&txn, reqSize);
 
   // we expect the difference from the per stream window and the initial window,
   // together with the bytes sent in the request
-  uint32_t perStreamWindow = spdy::kInitialWindow + 1024 * 1024;
-  uint32_t expectedWindowUpdate = perStreamWindow - spdy::kInitialWindow;
+  uint32_t perStreamWindow = http2::kInitialWindow + 1024 * 1024;
+  uint32_t expectedWindowUpdate = perStreamWindow - http2::kInitialWindow;
   EXPECT_CALL(transport_, sendWindowUpdate(_, expectedWindowUpdate));
 
   // use a higher window
@@ -295,17 +288,9 @@ TEST_F(DownstreamTransactionTest, WindowIncrease) {
  */
 TEST_F(DownstreamTransactionTest, WindowDecrease) {
   // set initial window size higher than per-stream window
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500),
-                      nullptr,
-                      true, // flow control enabled
-                      spdy::kInitialWindow,
-                      spdy::kInitialWindow);
+  auto& txn = makeTxn(true, // flow control enabled
+                      http2::kInitialWindow,
+                      http2::kInitialWindow);
   setupRequestResponseFlow(&txn, 500);
 
   // in this case, there should be no window update, as we decrease the window
@@ -313,7 +298,7 @@ TEST_F(DownstreamTransactionTest, WindowDecrease) {
   EXPECT_CALL(transport_, sendWindowUpdate(_, _)).Times(0);
 
   // use a smaller window
-  uint32_t perStreamWindow = spdy::kInitialWindow - 1000;
+  uint32_t perStreamWindow = http2::kInitialWindow - 1000;
   txn.setReceiveWindow(perStreamWindow);
 
   txn.onIngressHeadersComplete(makeGetRequest());
@@ -325,21 +310,15 @@ TEST_F(DownstreamTransactionTest, ParseErrorCbs) {
   // callback. This is possible because codecs are stateless between
   // frames.
 
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500));
+  auto& txn = makeTxn();
 
   HTTPException err(HTTPException::Direction::INGRESS, "test");
   err.setHttpStatusCode(400);
 
-  InSequence dummy;
+  InSequence enforceOrder;
 
-  EXPECT_CALL(handler_, setTransaction(&txn));
-  EXPECT_CALL(handler_, onError(_))
+  EXPECT_CALL(handler_, _setTransaction(&txn));
+  EXPECT_CALL(handler_, _onError(_))
       .WillOnce(Invoke([](const HTTPException& ex) {
         ASSERT_EQ(ex.getDirection(), HTTPException::Direction::INGRESS);
         ASSERT_EQ(std::string(ex.what()), "test");
@@ -347,7 +326,19 @@ TEST_F(DownstreamTransactionTest, ParseErrorCbs) {
   // onBody() is suppressed since ingress is complete after ingress onError()
   // onEOM() is suppressed since ingress is complete after ingress onError()
   EXPECT_CALL(transport_, sendAbort(_, _));
-  EXPECT_CALL(handler_, detachTransaction());
+
+  // New-ish: onError can be delivered more than once if the first error was
+  // unidirectional and the other direction isn't closed before the second error
+  // occurs.  Is this OK?
+  EXPECT_CALL(handler_, _onError(_))
+      .WillOnce(Invoke([](const HTTPException& ex) {
+        EXPECT_EQ(ex.getDirection(),
+                  HTTPException::Direction::INGRESS_AND_EGRESS);
+        EXPECT_NE(
+            std::string(ex.what()).find("onIngressBody after ingress closed"),
+            std::string::npos);
+      }));
+  EXPECT_CALL(handler_, _detachTransaction());
   EXPECT_CALL(transport_, detach(&txn));
 
   txn.setHandler(&handler_);
@@ -364,18 +355,12 @@ TEST_F(DownstreamTransactionTest, DetachFromNotify) {
   unique_ptr<StrictMock<MockHTTPHandler>> handler(
       new StrictMock<MockHTTPHandler>);
 
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500));
+  auto& txn = makeTxn();
 
-  InSequence dummy;
+  InSequence enforceOrder;
 
-  EXPECT_CALL(*handler, setTransaction(&txn));
-  EXPECT_CALL(*handler, onHeadersComplete(_))
+  EXPECT_CALL(*handler, _setTransaction(&txn));
+  EXPECT_CALL(*handler, _onHeadersComplete(_))
       .WillOnce(Invoke([&](std::shared_ptr<HTTPMessage> /*msg*/) {
         auto response = makeResponse(200);
         txn.sendHeaders(*response.get());
@@ -404,22 +389,12 @@ TEST_F(DownstreamTransactionTest, DeferredEgress) {
   EXPECT_CALL(transport_, describe(_)).WillRepeatedly(Return());
   EXPECT_CALL(transport_, notifyPendingEgress()).WillRepeatedly(Return());
 
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500),
-                      nullptr,
-                      true,
-                      10,
-                      10);
+  auto& txn = makeTxn(true, 10, 10);
 
-  InSequence dummy;
+  InSequence enforceOrder;
 
-  EXPECT_CALL(handler_, setTransaction(&txn));
-  EXPECT_CALL(handler_, onHeadersComplete(_))
+  EXPECT_CALL(handler_, _setTransaction(&txn));
+  EXPECT_CALL(handler_, _onHeadersComplete(_))
       .WillOnce(Invoke([&](std::shared_ptr<HTTPMessage> /*msg*/) {
         auto response = makeResponse(200);
         txn.sendHeaders(*response.get());
@@ -433,14 +408,14 @@ TEST_F(DownstreamTransactionTest, DeferredEgress) {
 
   // when enqueued
   EXPECT_CALL(transport_, notifyEgressBodyBuffered(10));
-  EXPECT_CALL(handler_, onEgressPaused());
+  EXPECT_CALL(handler_, _onEgressPaused());
   // sendBody
   EXPECT_CALL(transport_, notifyEgressBodyBuffered(20));
 
   txn.setHandler(&handler_);
   txn.onIngressHeadersComplete(makeGetRequest());
 
-  // onWriteReady, send, then dequeue (SPDY window now full)
+  // onWriteReady, send, then dequeue (window now full)
   EXPECT_CALL(transport_, notifyEgressBodyBuffered(-10));
   EXPECT_CALL(transport_, notifyEgressBodyBuffered(-20));
 
@@ -453,8 +428,8 @@ TEST_F(DownstreamTransactionTest, DeferredEgress) {
 
   // Buffer released on error
   EXPECT_CALL(transport_, notifyEgressBodyBuffered(-20));
-  EXPECT_CALL(handler_, onError(_));
-  EXPECT_CALL(handler_, detachTransaction());
+  EXPECT_CALL(handler_, _onError(_));
+  EXPECT_CALL(handler_, _detachTransaction());
   EXPECT_CALL(transport_, detach(&txn));
 
   HTTPException err(HTTPException::Direction::INGRESS_AND_EGRESS, "test");
@@ -465,18 +440,12 @@ TEST_F(DownstreamTransactionTest, InternalError) {
   unique_ptr<StrictMock<MockHTTPHandler>> handler(
       new StrictMock<MockHTTPHandler>);
 
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500));
+  auto& txn = makeTxn();
 
-  InSequence dummy;
+  InSequence enforceOrder;
 
-  EXPECT_CALL(*handler, setTransaction(&txn));
-  EXPECT_CALL(*handler, onHeadersComplete(_))
+  EXPECT_CALL(*handler, _setTransaction(&txn));
+  EXPECT_CALL(*handler, _onHeadersComplete(_))
       .WillOnce(Invoke([&](std::shared_ptr<HTTPMessage> /*msg*/) {
         auto response = makeResponse(200);
         txn.sendHeaders(*response.get());
@@ -486,7 +455,7 @@ TEST_F(DownstreamTransactionTest, InternalError) {
         EXPECT_EQ(headers.getStatusCode(), 200);
       }));
   EXPECT_CALL(transport_, sendAbort(&txn, ErrorCode::INTERNAL_ERROR));
-  EXPECT_CALL(*handler, detachTransaction());
+  EXPECT_CALL(*handler, _detachTransaction());
   EXPECT_CALL(transport_, detach(&txn));
 
   HTTPException err(HTTPException::Direction::INGRESS_AND_EGRESS, "test");
@@ -497,56 +466,39 @@ TEST_F(DownstreamTransactionTest, InternalError) {
 }
 
 TEST_F(DownstreamTransactionTest, UnpausedFlowControlViolation) {
-  StrictMock<MockHTTPHandler> handler;
-
   InSequence enforceOrder;
-  HTTPTransaction txn(TransportDirection::DOWNSTREAM,
-                      HTTPCodec::StreamID(1),
-                      1,
-                      transport_,
-                      txnEgressQueue_,
-                      transactionTimeouts_.get(),
-                      std::chrono::milliseconds(500),
-                      nullptr,
-                      true, // flow control enabled
+  auto& txn = makeTxn(true, // flow control enabled
                       400,
-                      spdy::kInitialWindow);
+                      http2::kInitialWindow);
 
-  EXPECT_CALL(handler, setTransaction(&txn));
-  EXPECT_CALL(handler, onHeadersComplete(_));
+  EXPECT_CALL(handler_, _setTransaction(&txn));
+  EXPECT_CALL(handler_, _onHeadersComplete(_));
   EXPECT_CALL(transport_, sendAbort(&txn, ErrorCode::FLOW_CONTROL_ERROR));
-  EXPECT_CALL(handler, detachTransaction());
+  EXPECT_CALL(handler_, _onError(_))
+      .WillOnce(Invoke([](const HTTPException& ex) {
+        EXPECT_EQ(ex.getDirection(),
+                  HTTPException::Direction::INGRESS_AND_EGRESS);
+        EXPECT_NE(std::string(ex.what()).find("reserve failed"),
+                  std::string::npos);
+      }));
+  EXPECT_CALL(handler_, _detachTransaction());
   EXPECT_CALL(transport_, detach(&txn));
 
-  txn.setHandler(&handler);
+  txn.setHandler(&handler_);
   txn.onIngressHeadersComplete(makePostRequest(401));
   txn.onIngressBody(makeBuf(401), 0);
 }
 
 TEST_F(DownstreamTransactionTest, ParseIngressErrorExTxnUnidirectional) {
   // Test where the ex transaction using QoS0 gets Ingress error
-  HTTPTransaction exTxn(TransportDirection::DOWNSTREAM,
-                        HTTPCodec::StreamID(2),
-                        1,
-                        transport_,
-                        txnEgressQueue_,
-                        transactionTimeouts_.get(),
-                        std::chrono::milliseconds(500),
-                        nullptr,
-                        false,
-                        0,
-                        0,
-                        http2::DefaultPriority,
-                        HTTPCodec::NoStream,
-                        HTTPCodec::ExAttributes(1, true));
-
+  auto& exTxn = makeExTxn();
   HTTPException err(HTTPException::Direction::INGRESS, "test");
   err.setHttpStatusCode(400);
 
-  InSequence dummy;
+  InSequence enforceOrder;
 
-  EXPECT_CALL(handler_, setTransaction(&exTxn));
-  EXPECT_CALL(handler_, onError(_))
+  EXPECT_CALL(handler_, _setTransaction(&exTxn));
+  EXPECT_CALL(handler_, _onError(_))
       .WillOnce(Invoke([](const HTTPException& ex) {
         ASSERT_EQ(ex.getDirection(), HTTPException::Direction::INGRESS);
         ASSERT_EQ(std::string(ex.what()), "test");
@@ -554,7 +506,15 @@ TEST_F(DownstreamTransactionTest, ParseIngressErrorExTxnUnidirectional) {
   // onBody() is suppressed since ingress is complete after ingress onError()
   // onEOM() is suppressed since ingress is complete after ingress onError()
   EXPECT_CALL(transport_, sendAbort(_, _));
-  EXPECT_CALL(handler_, detachTransaction());
+  EXPECT_CALL(handler_, _onError(_))
+      .WillOnce(Invoke([](const HTTPException& ex) {
+        EXPECT_EQ(ex.getDirection(),
+                  HTTPException::Direction::INGRESS_AND_EGRESS);
+        EXPECT_NE(
+            std::string(ex.what()).find("onIngressBody after ingress closed"),
+            std::string::npos);
+      }));
+  EXPECT_CALL(handler_, _detachTransaction());
   EXPECT_CALL(transport_, detach(&exTxn));
 
   exTxn.setHandler(&handler_);
@@ -569,37 +529,31 @@ TEST_F(DownstreamTransactionTest, ParseIngressErrorExTxnUnidirectional) {
 
 TEST_F(DownstreamTransactionTest, ParseIngressErrorExTxnNonUnidirectional) {
   // Test where the ex transaction using QoS0 gets Ingress error
-  HTTPTransaction exTxn(TransportDirection::DOWNSTREAM,
-                        HTTPCodec::StreamID(2),
-                        1,
-                        transport_,
-                        txnEgressQueue_,
-                        transactionTimeouts_.get(),
-                        std::chrono::milliseconds(500),
-                        nullptr,
-                        false,
-                        0,
-                        0,
-                        http2::DefaultPriority,
-                        HTTPCodec::NoStream,
-                        HTTPCodec::ExAttributes(1, true));
-
+  auto& exTxn = makeExTxn();
   HTTPException err(HTTPException::Direction::INGRESS, "test");
   err.setHttpStatusCode(400);
 
-  InSequence dummy;
+  InSequence enforceOrder;
 
-  EXPECT_CALL(handler_, setTransaction(&exTxn));
+  EXPECT_CALL(handler_, _setTransaction(&exTxn));
   // Ingress error will propagate
   // even if INGRESS state is completed for unidrectional ex_txn
-  EXPECT_CALL(handler_, onError(_))
+  EXPECT_CALL(handler_, _onError(_))
       .WillOnce(Invoke([](const HTTPException& ex) {
         ASSERT_EQ(ex.getDirection(), HTTPException::Direction::INGRESS);
         ASSERT_EQ(std::string(ex.what()), "test");
       }));
 
   EXPECT_CALL(transport_, sendAbort(_, _));
-  EXPECT_CALL(handler_, detachTransaction());
+  EXPECT_CALL(handler_, _onError(_))
+      .WillOnce(Invoke([](const HTTPException& ex) {
+        EXPECT_EQ(ex.getDirection(),
+                  HTTPException::Direction::INGRESS_AND_EGRESS);
+        EXPECT_NE(
+            std::string(ex.what()).find("onIngressBody after ingress closed"),
+            std::string::npos);
+      }));
+  EXPECT_CALL(handler_, _detachTransaction());
   EXPECT_CALL(transport_, detach(&exTxn));
 
   exTxn.setHandler(&handler_);
@@ -611,3 +565,40 @@ TEST_F(DownstreamTransactionTest, ParseIngressErrorExTxnNonUnidirectional) {
 
   eventBase_.loop();
 }
+
+TEST_F(DownstreamTransactionTest, IngressStateViolationWithByteEvents) {
+  auto& txn = makeTxn();
+
+  EXPECT_CALL(handler_, _setTransaction(&txn));
+  txn.setHandler(&handler_);
+  EXPECT_CALL(handler_, _onHeadersComplete(_));
+  txn.onIngressHeadersComplete(makeGetRequest());
+  EXPECT_CALL(handler_, _onEOM());
+  txn.onIngressEOM();
+
+  // When headers are sent, hold a byte event
+  EXPECT_CALL(transport_, sendHeaders(&txn, _, _, _))
+      .WillOnce(
+          InvokeWithoutArgs([&txn] { txn.incrementPendingByteEvents(); }));
+  txn.sendHeaders(getResponse(200));
+
+  EXPECT_CALL(transport_, sendAbort(_, _));
+  // The error is delivered immediately after abort is sent
+  EXPECT_CALL(handler_, _onError(_))
+      .WillOnce(Invoke([](const HTTPException& ex) {
+        EXPECT_EQ(ex.getDirection(),
+                  HTTPException::Direction::INGRESS_AND_EGRESS);
+        EXPECT_NE(
+            std::string(ex.what()).find("onIngressEOM after ingress closed"),
+            std::string::npos);
+      }));
+
+  // Double EOM
+  txn.onIngressEOM();
+
+  // The transaction detaches after the byte event is released
+  EXPECT_CALL(handler_, _detachTransaction());
+  EXPECT_CALL(transport_, detach(&txn));
+  txn.decrementPendingByteEvents();
+  eventBase_.loop();
+}
