@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -14,6 +14,7 @@
 #include <proxygen/lib/http/session/HQUnidirectionalCallbacks.h>
 #include <proxygen/lib/http/session/test/HTTPSessionMocks.h>
 #include <proxygen/lib/http/session/test/HTTPTransactionMocks.h>
+#include <quic/dsr/test/Mocks.h>
 
 namespace proxygen {
 
@@ -98,13 +99,6 @@ class MockDispatcher : public HQUnidirStreamDispatcher::Callback {
     }
   }
 
-  void expectIsPartialReliabilityEnabled(StreamInspectF impl = nullptr) {
-    auto& exp = EXPECT_CALL(*this, isPartialReliabilityEnabled(::testing::_));
-    if (impl) {
-      exp.WillOnce(::testing::Invoke(impl));
-    }
-  }
-
   void expectPartialDataAvailable(StreamPeekF impl = nullptr) {
     auto& exp =
         EXPECT_CALL(*this, onPartialDataAvailable(::testing::_, ::testing::_));
@@ -113,95 +107,91 @@ class MockDispatcher : public HQUnidirStreamDispatcher::Callback {
     }
   }
 
-  void expectProcessExpiredData(PRSeekF impl = nullptr) {
-    auto& exp =
-        EXPECT_CALL(*this, processExpiredData(::testing::_, ::testing::_));
-    if (impl) {
-      exp.WillOnce(::testing::Invoke(impl));
-    }
-  }
-
-  void expectProcessRejectedData(PRSeekF impl = nullptr) {
-    auto& exp =
-        EXPECT_CALL(*this, processRejectedData(::testing::_, ::testing::_));
-    if (impl) {
-      exp.WillOnce(::testing::Invoke(impl));
-    }
-  }
-
-  MOCK_METHOD3(onNewPushStream, void(quic::StreamId, hq::PushId, size_t));
-  MOCK_METHOD4(assignReadCallback,
-               void(quic::StreamId,
-                    hq::UnidirectionalStreamType,
-                    size_t,
-                    quic::QuicSocket::ReadCallback* const));
-  MOCK_METHOD4(assignPeekCallback,
-               void(quic::StreamId,
-                    hq::UnidirectionalStreamType,
-                    size_t,
-                    quic::QuicSocket::PeekCallback* const));
-  MOCK_METHOD1(isPartialReliabilityEnabled, bool(quic::StreamId));
-  MOCK_METHOD1(parseStreamPreface,
-               folly::Optional<hq::UnidirectionalStreamType>(uint64_t));
-  MOCK_METHOD1(rejectStream, void(quic::StreamId));
-  MOCK_METHOD1(controlStreamReadAvailable, void(quic::StreamId));
-  MOCK_METHOD2(controlStreamReadError, void(quic::StreamId, const ReadError&));
-  MOCK_METHOD2(onPartialDataAvailable, void(quic::StreamId, const PeekData&));
-  MOCK_METHOD2(processExpiredData, void(quic::StreamId, uint64_t));
-  MOCK_METHOD2(processRejectedData, void(quic::StreamId, uint64_t));
+  MOCK_METHOD(void, onNewPushStream, (quic::StreamId, hq::PushId, size_t));
+  MOCK_METHOD(void,
+              assignReadCallback,
+              (quic::StreamId,
+               hq::UnidirectionalStreamType,
+               size_t,
+               quic::QuicSocket::ReadCallback* const));
+  MOCK_METHOD(void,
+              assignPeekCallback,
+              (quic::StreamId,
+               hq::UnidirectionalStreamType,
+               size_t,
+               quic::QuicSocket::PeekCallback* const));
+  MOCK_METHOD(folly::Optional<hq::UnidirectionalStreamType>,
+              parseStreamPreface,
+              (uint64_t));
+  MOCK_METHOD(void, rejectStream, (quic::StreamId));
+  MOCK_METHOD(void, controlStreamReadAvailable, (quic::StreamId));
+  MOCK_METHOD(void, controlStreamReadError, (quic::StreamId, const ReadError&));
+  MOCK_METHOD(void, onPartialDataAvailable, (quic::StreamId, const PeekData&));
+  MOCK_METHOD(void, processExpiredData, (quic::StreamId, uint64_t));
+  MOCK_METHOD(void, processRejectedData, (quic::StreamId, uint64_t));
 };
 
 class MockServerPushLifecycleCallback : public ServerPushLifecycleCallback {
  public:
   virtual ~MockServerPushLifecycleCallback() = default;
 
-  MOCK_METHOD2(onPushPromiseBegin,
-               void(HTTPCodec::StreamID /* parent streamID */,
-                    hq::PushId /* pushID */));
+  MOCK_METHOD(void,
+              onPushPromiseBegin,
+              (HTTPCodec::StreamID /* parent streamID */,
+               hq::PushId /* pushID */));
 
-  MOCK_METHOD3(onPushPromise,
-               void(HTTPCodec::StreamID /* parent streamID */,
-                    hq::PushId /* pushID */,
-                    HTTPMessage* /* msg */));
+  MOCK_METHOD(void,
+              onPushPromise,
+              (HTTPCodec::StreamID /* parent streamID */,
+               hq::PushId /* pushID */,
+               HTTPMessage* /* msg */));
 
-  MOCK_METHOD2(onNascentPushStreamBegin,
-               void(HTTPCodec::StreamID /* push stream ID */, bool /* eom */));
+  MOCK_METHOD(void,
+              onNascentPushStreamBegin,
+              (HTTPCodec::StreamID /* push stream ID */, bool /* eom */));
 
-  MOCK_METHOD3(onNascentPushStream,
-               void(HTTPCodec::StreamID /* push stream ID */,
-                    hq::PushId /* server push id */,
-                    bool /* eom */));
+  MOCK_METHOD(void,
+              onNascentPushStream,
+              (HTTPCodec::StreamID /* push stream ID */,
+               hq::PushId /* server push id */,
+               bool /* eom */));
 
-  MOCK_METHOD2(onNascentEof,
-               void(HTTPCodec::StreamID /* push stream ID */,
-                    folly::Optional<hq::PushId> /* push id */));
+  MOCK_METHOD(void,
+              onNascentEof,
+              (HTTPCodec::StreamID /* push stream ID */,
+               folly::Optional<hq::PushId> /* push id */));
 
-  MOCK_METHOD2(onOrphanedNascentStream,
-               void(HTTPCodec::StreamID /* push stream ID */,
-                    folly::Optional<hq::PushId> /* push id */));
+  MOCK_METHOD(void,
+              onOrphanedNascentStream,
+              (HTTPCodec::StreamID /* push stream ID */,
+               folly::Optional<hq::PushId> /* push id */));
 
-  MOCK_METHOD4(onHalfOpenPushedTxn,
-               void(const HTTPTransaction* /* txn */,
-                    hq::PushId /* push id */,
-                    HTTPCodec::StreamID /* assoc stream id */,
-                    bool /* eom */));
+  MOCK_METHOD(void,
+              onHalfOpenPushedTxn,
+              (const HTTPTransaction* /* txn */,
+               hq::PushId /* push id */,
+               HTTPCodec::StreamID /* assoc stream id */,
+               bool /* eom */));
 
-  MOCK_METHOD5(onPushedTxn,
-               void(const HTTPTransaction* /* txn */,
-                    HTTPCodec::StreamID /* push stream id */,
-                    hq::PushId /* push id */,
-                    HTTPCodec::StreamID /* assoc stream id */,
-                    bool /* eom */));
+  MOCK_METHOD(void,
+              onPushedTxn,
+              (const HTTPTransaction* /* txn */,
+               HTTPCodec::StreamID /* push stream id */,
+               hq::PushId /* push id */,
+               HTTPCodec::StreamID /* assoc stream id */,
+               bool /* eom */));
 
-  MOCK_METHOD1(onPushedTxnTimeout, void(const HTTPTransaction* /* txn */));
+  MOCK_METHOD(void, onPushedTxnTimeout, (const HTTPTransaction* /* txn */));
 
-  MOCK_METHOD1(onOrphanedHalfOpenPushedTxn,
-               void(const HTTPTransaction* /* txn */));
+  MOCK_METHOD(void,
+              onOrphanedHalfOpenPushedTxn,
+              (const HTTPTransaction* /* txn */));
 
-  MOCK_METHOD3(onPushIdLimitExceeded,
-               void(hq::PushId /* incoming push id */,
-                    folly::Optional<hq::PushId> /* max allowed push id */,
-                    folly::Optional<HTTPCodec::StreamID> /* stream */));
+  MOCK_METHOD(void,
+              onPushIdLimitExceeded,
+              (hq::PushId /* incoming push id */,
+               folly::Optional<hq::PushId> /* max allowed push id */,
+               folly::Optional<HTTPCodec::StreamID> /* stream */));
 
   using PushPromiseBeginF =
       std::function<void(HTTPCodec::StreamID, hq::PushId)>;
@@ -322,10 +312,10 @@ class MockServerPushLifecycleCallback : public ServerPushLifecycleCallback {
 
 class MockConnectCallback : public HQSession::ConnectCallback {
  public:
-  MOCK_METHOD0(connectSuccess, void());
-  MOCK_METHOD0(onReplaySafe, void());
-  MOCK_METHOD1(connectError, void(std::pair<quic::QuicErrorCode, std::string>));
-  MOCK_METHOD0(onFirstPeerPacketProcessed, void());
+  MOCK_METHOD(void, connectSuccess, ());
+  MOCK_METHOD(void, onReplaySafe, ());
+  MOCK_METHOD(void, connectError, (quic::QuicError));
+  MOCK_METHOD(void, onFirstPeerPacketProcessed, ());
 };
 
 class MockHQSession : public HQSession {
@@ -405,22 +395,24 @@ class MockHQSession : public HQSession {
     onHeadersComplete(streamID, std::move(resp), eom);
   }
 
-  MOCK_CONST_METHOD0(isReplaySafe, bool());
+  MOCK_METHOD(bool, isReplaySafe, (), (const));
 
-  MOCK_METHOD1(getTransactionTimeoutHandler,
-               HTTPTransaction::Handler*(HTTPTransaction*));
+  MOCK_METHOD(HTTPTransaction::Handler*,
+              getTransactionTimeoutHandler,
+              (HTTPTransaction*));
 
-  MOCK_METHOD2(setupOnHeadersComplete, void(HTTPTransaction*, HTTPMessage*));
+  MOCK_METHOD(void, setupOnHeadersComplete, (HTTPTransaction*, HTTPMessage*));
 
-  GMOCK_METHOD1_(,
-                 noexcept,
-                 ,
-                 onConnectionErrorHandler,
-                 void(std::pair<quic::QuicErrorCode, std::string> error));
+  MOCK_METHOD((void),
+              onConnectionSetupErrorHandler,
+              (quic::QuicError),
+              (noexcept));
 
-  MOCK_METHOD1(newTransaction, HTTPTransaction*(HTTPTransaction::Handler*));
+  MOCK_METHOD(HTTPTransaction*, newTransaction, (HTTPTransaction::Handler*));
 
-  MOCK_METHOD0(drain, void());
+  MOCK_METHOD(void, drain, ());
+
+  MOCK_CONST_METHOD0(getQuicSocket, quic::QuicSocket*());
 
   MockHTTPTransaction* makeMockTransaction(HTTPCodec::StreamID id) {
     LOG(INFO) << "Creating mocked transaction on stream " << id;
@@ -500,58 +492,15 @@ class MockHQSession : public HQSession {
   HTTPTransaction::Handler* handler_;
 };
 
-class MockHqPrUpstreamHTTPHandler : public MockHTTPHandler {
- public:
-  MockHqPrUpstreamHTTPHandler() {
-  }
-  MockHqPrUpstreamHTTPHandler(HTTPTransaction& txn,
-                              HTTPMessage* msg,
-                              const folly::SocketAddress& addr)
-      : MockHTTPHandler(txn, msg, addr) {
-  }
-
-  void sendPrHeaders(uint32_t code,
-                     uint32_t content_length,
-                     bool keepalive = true,
-                     HeaderMap headers = HeaderMap()) {
-    HTTPMessage reply;
-    reply.setStatusCode(code);
-    reply.setHTTPVersion(1, 1);
-    reply.setWantsKeepalive(keepalive);
-    reply.getHeaders().add(HTTP_HEADER_CONTENT_LENGTH,
-                           folly::to<std::string>(content_length));
-    for (auto& nv : headers) {
-      reply.getHeaders().add(nv.first, nv.second);
-    }
-    reply.setPartiallyReliable();
-    txn_->sendHeaders(reply);
-  }
-
-  void expectBodySkipped(std::function<void(uint64_t)> callback =
-                             std::function<void(uint64_t)>()) {
-    if (callback) {
-      EXPECT_CALL(*this, onBodySkipped(testing::_))
-          .WillOnce(testing::Invoke(callback));
-    } else {
-      EXPECT_CALL(*this, onBodySkipped(testing::_));
-    }
-  }
-
-  void expectBodyRejected(std::function<void(uint64_t)> callback =
-                              std::function<void(uint64_t)>()) {
-    if (callback) {
-      EXPECT_CALL(*this, onBodyRejected(testing::_))
-          .WillOnce(testing::Invoke(callback));
-    } else {
-      EXPECT_CALL(*this, onBodyRejected(testing::_));
-    }
-  }
-};
-
-using MockHqPrDownstreamHTTPHandler = MockHqPrUpstreamHTTPHandler;
-
 class FakeHQHTTPCodecCallback : public FakeHTTPCodecCallback {
  public:
 };
 
+class MockQuicDSRRequestSender
+    : public MockDSRRequestSender
+    , public quic::test::MockDSRPacketizationRequestSender {
+ public:
+  MockQuicDSRRequestSender() = default;
+};
+
 } // namespace proxygen
