@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -15,10 +15,7 @@
 #include <proxygen/lib/http/session/HTTPSessionController.h>
 #include <proxygen/lib/http/session/HTTPSessionStats.h>
 #include <proxygen/lib/http/session/HTTPTransaction.h>
-
-#define GMOCK_NOEXCEPT_METHOD0(m, F) GMOCK_METHOD0_(, noexcept, , m, F)
-#define GMOCK_NOEXCEPT_METHOD1(m, F) GMOCK_METHOD1_(, noexcept, , m, F)
-#define GMOCK_NOEXCEPT_METHOD2(m, F) GMOCK_METHOD2_(, noexcept, , m, F)
+#include <proxygen/lib/http/session/test/MockHTTPSessionStats.h>
 
 namespace proxygen {
 
@@ -61,6 +58,17 @@ class HTTPHandlerBase {
     txn_->sendHeaders(reply);
   }
 
+  bool sendHeadersWithDelegate(uint32_t code,
+                               uint32_t content_length,
+                               std::unique_ptr<DSRRequestSender> dsrSender) {
+    HTTPMessage response;
+    response.setStatusCode(code);
+    response.setHTTPVersion(1, 1);
+    response.getHeaders().add(HTTP_HEADER_CONTENT_LENGTH,
+                              folly::to<std::string>(content_length));
+    return txn_->sendHeadersWithDelegate(response, std::move(dsrSender));
+  }
+
   void sendReply() {
     sendReplyCode(200);
   }
@@ -146,84 +154,130 @@ class MockHTTPHandler
     , public HTTPTransaction::Handler {
  public:
   MockHTTPHandler() {
+    setupInvariantViolation();
   }
   MockHTTPHandler(HTTPTransaction& txn,
                   HTTPMessage* msg,
                   const folly::SocketAddress&)
       : HTTPHandlerBase(&txn, msg) {
+    setupInvariantViolation();
   }
 
-  GMOCK_NOEXCEPT_METHOD1(setTransaction, void(HTTPTransaction* txn));
+  void setupInvariantViolation() {
+    ON_CALL(*this, _onInvariantViolation(testing::_))
+        .WillByDefault(testing::Invoke(
+            [](const HTTPException& ex) { LOG(FATAL) << ex.what(); }));
+  }
 
-  GMOCK_NOEXCEPT_METHOD0(detachTransaction, void());
+  void setTransaction(HTTPTransaction* txn) noexcept override {
+    _setTransaction(txn);
+  }
+  MOCK_METHOD(void, _setTransaction, (HTTPTransaction*));
+
+  void detachTransaction() noexcept override {
+    _detachTransaction();
+  }
+  MOCK_METHOD(void, _detachTransaction, ());
 
   void onHeadersComplete(std::unique_ptr<HTTPMessage> msg) noexcept override {
-    onHeadersComplete(std::shared_ptr<HTTPMessage>(msg.release()));
+    _onHeadersComplete(std::shared_ptr<HTTPMessage>(msg.release()));
   }
 
-  GMOCK_NOEXCEPT_METHOD1(onHeadersComplete,
-                         void(std::shared_ptr<HTTPMessage> msg));
+  MOCK_METHOD(void, _onHeadersComplete, (std::shared_ptr<HTTPMessage>));
 
   void onBody(std::unique_ptr<folly::IOBuf> chain) noexcept override {
-    onBody(std::shared_ptr<folly::IOBuf>(chain.release()));
+    _onBody(std::shared_ptr<folly::IOBuf>(chain.release()));
   }
-  GMOCK_NOEXCEPT_METHOD1(onBody, void(std::shared_ptr<folly::IOBuf> chain));
+  MOCK_METHOD(void, _onBody, (std::shared_ptr<folly::IOBuf>));
 
   void onBodyWithOffset(uint64_t bodyOffset,
                         std::unique_ptr<folly::IOBuf> chain) noexcept override {
-    onBodyWithOffset(bodyOffset,
-                     std::shared_ptr<folly::IOBuf>(chain.release()));
+    _onBodyWithOffset(bodyOffset,
+                      std::shared_ptr<folly::IOBuf>(chain.release()));
   }
-  GMOCK_NOEXCEPT_METHOD2(onBodyWithOffset,
-                         void(uint64_t bodyOffset,
-                              std::shared_ptr<folly::IOBuf> chain));
+  MOCK_METHOD(void,
+              _onBodyWithOffset,
+              (uint64_t, std::shared_ptr<folly::IOBuf>));
 
-  GMOCK_NOEXCEPT_METHOD1(onChunkHeader, void(size_t length));
+  void onDatagram(std::unique_ptr<folly::IOBuf> chain) noexcept override {
+    _onDatagram(std::shared_ptr<folly::IOBuf>(chain.release()));
+  }
+  MOCK_METHOD(void, _onDatagram, (std::shared_ptr<folly::IOBuf>));
 
-  GMOCK_NOEXCEPT_METHOD0(onChunkComplete, void());
+  void onChunkHeader(size_t length) noexcept override {
+    _onChunkHeader(length);
+  }
+  MOCK_METHOD(void, _onChunkHeader, (size_t));
+
+  void onChunkComplete() noexcept override {
+    _onChunkComplete();
+  }
+  MOCK_METHOD(void, _onChunkComplete, ());
 
   void onTrailers(std::unique_ptr<HTTPHeaders> trailers) noexcept override {
-    onTrailers(std::shared_ptr<HTTPHeaders>(trailers.release()));
+    _onTrailers(std::shared_ptr<HTTPHeaders>(trailers.release()));
   }
 
-  GMOCK_NOEXCEPT_METHOD1(onTrailers,
-                         void(std::shared_ptr<HTTPHeaders> trailers));
+  MOCK_METHOD(void, _onTrailers, (std::shared_ptr<HTTPHeaders>));
 
-  GMOCK_NOEXCEPT_METHOD0(onEOM, void());
+  void onEOM() noexcept override {
+    _onEOM();
+  }
+  MOCK_METHOD(void, _onEOM, ());
 
-  GMOCK_NOEXCEPT_METHOD1(onUpgrade, void(UpgradeProtocol protocol));
+  void onUpgrade(UpgradeProtocol protocol) noexcept override {
+    _onUpgrade(protocol);
+  }
+  MOCK_METHOD(void, _onUpgrade, (UpgradeProtocol));
 
-  GMOCK_NOEXCEPT_METHOD1(onError, void(const HTTPException& error));
+  void onError(const HTTPException& error) noexcept override {
+    _onError(error);
+  }
+  MOCK_METHOD(void, _onError, (const HTTPException&));
 
-  GMOCK_NOEXCEPT_METHOD1(onGoaway, void(ErrorCode));
+  void onInvariantViolation(const HTTPException& error) noexcept override {
+    _onInvariantViolation(error);
+  }
+  MOCK_METHOD(void, _onInvariantViolation, (const HTTPException&));
 
-  GMOCK_NOEXCEPT_METHOD0(onEgressPaused, void());
+  void onGoaway(ErrorCode errCode) noexcept override {
+    _onGoaway(errCode);
+  }
+  MOCK_METHOD(void, _onGoaway, (ErrorCode));
 
-  GMOCK_NOEXCEPT_METHOD0(onEgressResumed, void());
+  void onEgressPaused() noexcept override {
+    _onEgressPaused();
+  }
+  MOCK_METHOD(void, _onEgressPaused, ());
 
-  GMOCK_NOEXCEPT_METHOD1(onPushedTransaction, void(HTTPTransaction*));
+  void onEgressResumed() noexcept override {
+    _onEgressResumed();
+  }
+  MOCK_METHOD(void, _onEgressResumed, ());
 
-  GMOCK_NOEXCEPT_METHOD1(onExTransaction, void(HTTPTransaction*));
+  void onPushedTransaction(HTTPTransaction* txn) noexcept override {
+    _onPushedTransaction(txn);
+  }
+  MOCK_METHOD(void, _onPushedTransaction, (HTTPTransaction*));
 
-  GMOCK_NOEXCEPT_METHOD1(onBodySkipped, void(uint64_t));
-
-  GMOCK_NOEXCEPT_METHOD1(onBodyRejected, void(uint64_t));
-
-  GMOCK_NOEXCEPT_METHOD1(traceEventAvailable, void(TraceEvent));
+  void onExTransaction(HTTPTransaction* txn) noexcept override {
+    _onExTransaction(txn);
+  }
+  MOCK_METHOD(void, _onExTransaction, (HTTPTransaction*));
 
   void expectTransaction(std::function<void(HTTPTransaction* txn)> callback) {
-    EXPECT_CALL(*this, setTransaction(testing::_))
+    EXPECT_CALL(*this, _setTransaction(testing::_))
         .WillOnce(testing::Invoke(callback))
         .RetiresOnSaturation();
   }
 
   void expectTransaction(HTTPTransaction** pTxn = nullptr) {
-    EXPECT_CALL(*this, setTransaction(testing::_))
+    EXPECT_CALL(*this, _setTransaction(testing::_))
         .WillOnce(testing::SaveArg<0>(pTxn ? pTxn : &txn_));
   }
 
   void expectPushedTransaction(HTTPTransactionHandler* handler = nullptr) {
-    EXPECT_CALL(*this, onPushedTransaction(testing::_))
+    EXPECT_CALL(*this, _onPushedTransaction(testing::_))
         .WillOnce(testing::Invoke([handler](HTTPTransaction* txn) {
           if (handler) {
             txn->setHandler(handler);
@@ -233,16 +287,16 @@ class MockHTTPHandler
 
   void expectHeaders(std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onHeadersComplete(testing::_))
+      EXPECT_CALL(*this, _onHeadersComplete(testing::_))
           .WillOnce(testing::InvokeWithoutArgs(callback))
           .RetiresOnSaturation();
     } else {
-      EXPECT_CALL(*this, onHeadersComplete(testing::_));
+      EXPECT_CALL(*this, _onHeadersComplete(testing::_));
     }
   }
 
   void expectHeaders(std::function<void(std::shared_ptr<HTTPMessage>)> cb) {
-    EXPECT_CALL(*this, onHeadersComplete(testing::_))
+    EXPECT_CALL(*this, _onHeadersComplete(testing::_))
         .WillOnce(testing::Invoke(cb))
         .RetiresOnSaturation();
   }
@@ -250,17 +304,17 @@ class MockHTTPHandler
   void expectTrailers(
       std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onTrailers(testing::_))
+      EXPECT_CALL(*this, _onTrailers(testing::_))
           .WillOnce(testing::InvokeWithoutArgs(callback))
           .RetiresOnSaturation();
     } else {
-      EXPECT_CALL(*this, onTrailers(testing::_));
+      EXPECT_CALL(*this, _onTrailers(testing::_));
     }
   }
 
   void expectTrailers(
       std::function<void(std::shared_ptr<HTTPHeaders> trailers)> cb) {
-    EXPECT_CALL(*this, onTrailers(testing::_))
+    EXPECT_CALL(*this, _onTrailers(testing::_))
         .WillOnce(testing::Invoke(cb))
         .RetiresOnSaturation();
   }
@@ -268,91 +322,110 @@ class MockHTTPHandler
   void expectChunkHeader(
       std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onChunkHeader(testing::_))
+      EXPECT_CALL(*this, _onChunkHeader(testing::_))
           .WillOnce(testing::InvokeWithoutArgs(callback));
     } else {
-      EXPECT_CALL(*this, onChunkHeader(testing::_));
+      EXPECT_CALL(*this, _onChunkHeader(testing::_));
     }
   }
 
   void expectBody(std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onBodyWithOffset(testing::_, testing::_))
+      EXPECT_CALL(*this, _onBodyWithOffset(testing::_, testing::_))
           .WillOnce(testing::InvokeWithoutArgs(callback));
     } else {
-      EXPECT_CALL(*this, onBodyWithOffset(testing::_, testing::_));
+      EXPECT_CALL(*this, _onBodyWithOffset(testing::_, testing::_));
     }
   }
 
   void expectBody(
       std::function<void(uint64_t, std::shared_ptr<folly::IOBuf>)> callback) {
-    EXPECT_CALL(*this, onBodyWithOffset(testing::_, testing::_))
+    EXPECT_CALL(*this, _onBodyWithOffset(testing::_, testing::_))
+        .WillOnce(testing::Invoke(callback));
+  }
+
+  void expectDatagram(
+      std::function<void()> callback = std::function<void()>()) {
+    if (callback) {
+      EXPECT_CALL(*this, _onDatagram(testing::_))
+          .WillOnce(testing::InvokeWithoutArgs(callback));
+    } else {
+      EXPECT_CALL(*this, _onDatagram(testing::_));
+    }
+  }
+
+  void expectDatagram(
+      std::function<void(std::shared_ptr<folly::IOBuf>)> callback) {
+    EXPECT_CALL(*this, _onDatagram(testing::_))
         .WillOnce(testing::Invoke(callback));
   }
 
   void expectChunkComplete(
       std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onChunkComplete())
+      EXPECT_CALL(*this, _onChunkComplete())
           .WillOnce(testing::InvokeWithoutArgs(callback));
     } else {
-      EXPECT_CALL(*this, onChunkComplete());
+      EXPECT_CALL(*this, _onChunkComplete());
     }
   }
 
   void expectEOM(std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onEOM()).WillOnce(testing::Invoke(callback));
+      EXPECT_CALL(*this, _onEOM()).WillOnce(testing::Invoke(callback));
     } else {
-      EXPECT_CALL(*this, onEOM());
+      EXPECT_CALL(*this, _onEOM());
     }
   }
 
   void expectEgressPaused(
       std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onEgressPaused()).WillOnce(testing::Invoke(callback));
+      EXPECT_CALL(*this, _onEgressPaused()).WillOnce(testing::Invoke(callback));
     } else {
-      EXPECT_CALL(*this, onEgressPaused());
+      EXPECT_CALL(*this, _onEgressPaused());
     }
   }
 
   void expectEgressResumed(
       std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, onEgressResumed()).WillOnce(testing::Invoke(callback));
+      EXPECT_CALL(*this, _onEgressResumed())
+          .WillOnce(testing::Invoke(callback));
     } else {
-      EXPECT_CALL(*this, onEgressResumed());
+      EXPECT_CALL(*this, _onEgressResumed());
     }
   }
 
   void expectError(std::function<void(const HTTPException& ex)> callback =
                        std::function<void(const HTTPException& ex)>()) {
     if (callback) {
-      EXPECT_CALL(*this, onError(testing::_))
-          .WillOnce(testing::Invoke(callback));
+      EXPECT_CALL(*this, _onError(testing::_))
+          .WillOnce(testing::Invoke(callback))
+          .RetiresOnSaturation();
     } else {
-      EXPECT_CALL(*this, onError(testing::_));
+      EXPECT_CALL(*this, _onError(testing::_)).RetiresOnSaturation();
     }
   }
 
   void expectGoaway(std::function<void(ErrorCode)> callback =
                         std::function<void(ErrorCode)>()) {
     if (callback) {
-      EXPECT_CALL(*this, onGoaway(testing::_))
+      EXPECT_CALL(*this, _onGoaway(testing::_))
           .WillOnce(testing::Invoke(callback));
     } else {
-      EXPECT_CALL(*this, onGoaway(testing::_));
+      EXPECT_CALL(*this, _onGoaway(testing::_));
     }
   }
 
   void expectDetachTransaction(
       std::function<void()> callback = std::function<void()>()) {
     if (callback) {
-      EXPECT_CALL(*this, detachTransaction())
-          .WillOnce(testing::Invoke(callback));
+      EXPECT_CALL(*this, _detachTransaction())
+          .WillOnce(testing::Invoke(callback))
+          .RetiresOnSaturation();
     } else {
-      EXPECT_CALL(*this, detachTransaction());
+      EXPECT_CALL(*this, _detachTransaction()).RetiresOnSaturation();
     }
   }
 };
@@ -369,21 +442,35 @@ class MockHTTPPushHandler
       : HTTPHandlerBase(&txn, msg) {
   }
 
-  GMOCK_NOEXCEPT_METHOD1(setTransaction, void(HTTPTransaction* txn));
+  void setTransaction(HTTPTransaction* txn) noexcept override {
+    _setTransaction(txn);
+  }
+  MOCK_METHOD(void, _setTransaction, (HTTPTransaction*));
 
-  GMOCK_NOEXCEPT_METHOD0(detachTransaction, void());
+  void detachTransaction() noexcept override {
+    _detachTransaction();
+  }
+  MOCK_METHOD(void, _detachTransaction, ());
 
-  GMOCK_NOEXCEPT_METHOD1(onError, void(const HTTPException& error));
+  void onError(const HTTPException& error) noexcept override {
+    _onError(error);
+  }
+  MOCK_METHOD(void, _onError, (const HTTPException&));
 
-  GMOCK_NOEXCEPT_METHOD1(onGoaway, void(ErrorCode));
+  void onGoaway(ErrorCode errCode) noexcept override {
+    _onGoaway(errCode);
+  }
+  MOCK_METHOD(void, _onGoaway, (ErrorCode));
 
-  GMOCK_NOEXCEPT_METHOD0(onEgressPaused, void());
+  void onEgressPaused() noexcept override {
+    _onEgressPaused();
+  }
+  MOCK_METHOD(void, _onEgressPaused, ());
 
-  GMOCK_NOEXCEPT_METHOD0(onEgressResumed, void());
-
-  GMOCK_NOEXCEPT_METHOD1(onBodySkipped, void(uint64_t));
-
-  GMOCK_NOEXCEPT_METHOD1(onBodyRejected, void(uint64_t));
+  void onEgressResumed() noexcept override {
+    _onEgressResumed();
+  }
+  MOCK_METHOD(void, _onEgressResumed, ());
 
   void sendPushHeaders(const std::string& path,
                        const std::string& host,
@@ -402,37 +489,46 @@ class MockHTTPPushHandler
 
 class MockController : public HTTPSessionController {
  public:
-  MOCK_METHOD2(getRequestHandler,
-               HTTPTransactionHandler*(HTTPTransaction&, HTTPMessage* msg));
+  MOCK_METHOD(HTTPTransactionHandler*,
+              getRequestHandler,
+              (HTTPTransaction&, HTTPMessage* msg));
 
-  MOCK_METHOD3(getParseErrorHandler,
-               HTTPTransactionHandler*(HTTPTransaction*,
-                                       const HTTPException&,
-                                       const folly::SocketAddress&));
+  MOCK_METHOD(HTTPTransactionHandler*,
+              getParseErrorHandler,
+              (HTTPTransaction*,
+               const HTTPException&,
+               const folly::SocketAddress&));
 
-  MOCK_METHOD2(getTransactionTimeoutHandler,
-               HTTPTransactionHandler*(HTTPTransaction* txn,
-                                       const folly::SocketAddress&));
+  MOCK_METHOD(HTTPTransactionHandler*,
+              getTransactionTimeoutHandler,
+              (HTTPTransaction * txn, const folly::SocketAddress&));
 
-  MOCK_METHOD1(attachSession, void(HTTPSessionBase*));
-  MOCK_METHOD1(detachSession, void(const HTTPSessionBase*));
-  MOCK_METHOD1(onSessionCodecChange, void(HTTPSessionBase*));
-  MOCK_METHOD1(onTransportReady, void(HTTPSessionBase*));
+  MOCK_METHOD(void, attachSession, (HTTPSessionBase*));
+  MOCK_METHOD(void, detachSession, (const HTTPSessionBase*));
+  MOCK_METHOD(void, onSessionCodecChange, (HTTPSessionBase*));
+  MOCK_METHOD(void, onTransportReady, (HTTPSessionBase*));
 
-  MOCK_CONST_METHOD0(getGracefulShutdownTimeout, std::chrono::milliseconds());
+  MOCK_METHOD(std::chrono::milliseconds,
+              getGracefulShutdownTimeout,
+              (),
+              (const));
 
-  MOCK_CONST_METHOD0(getHeaderIndexingStrategy,
-                     const HeaderIndexingStrategy*());
+  MOCK_METHOD(const HeaderIndexingStrategy*,
+              getHeaderIndexingStrategy,
+              (),
+              (const));
 };
 
 class MockUpstreamController : public HTTPUpstreamSessionController {
  public:
-  MOCK_METHOD1(attachSession, void(HTTPSessionBase*));
-  MOCK_METHOD1(detachSession, void(const HTTPSessionBase*));
-  MOCK_METHOD1(onSessionCodecChange, void(HTTPSessionBase*));
+  MOCK_METHOD(void, attachSession, (HTTPSessionBase*));
+  MOCK_METHOD(void, detachSession, (const HTTPSessionBase*));
+  MOCK_METHOD(void, onSessionCodecChange, (HTTPSessionBase*));
 
-  MOCK_CONST_METHOD0(getHeaderIndexingStrategy,
-                     const HeaderIndexingStrategy*());
+  MOCK_METHOD(const HeaderIndexingStrategy*,
+              getHeaderIndexingStrategy,
+              (),
+              (const));
 };
 
 ACTION_P(ExpectString, expected) {
@@ -446,73 +542,44 @@ ACTION_P(ExpectBodyLen, expectedLen) {
 
 class MockHTTPSessionInfoCallback : public HTTPSession::InfoCallback {
  public:
-  MOCK_METHOD1(onCreate, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onTransportReady, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onConnectionError, void(const HTTPSessionBase&));
-  MOCK_METHOD2(onIngressError, void(const HTTPSessionBase&, ProxygenError));
-  MOCK_METHOD0(onIngressEOF, void());
-  MOCK_METHOD2(onRead, void(const HTTPSessionBase&, size_t));
-  MOCK_METHOD3(onRead,
-               void(const HTTPSessionBase&,
-                    size_t,
-                    folly::Optional<HTTPCodec::StreamID>));
-  MOCK_METHOD2(onWrite, void(const HTTPSessionBase&, size_t));
-  MOCK_METHOD1(onRequestBegin, void(const HTTPSessionBase&));
-  MOCK_METHOD2(onRequestEnd, void(const HTTPSessionBase&, uint32_t));
-  MOCK_METHOD1(onActivateConnection, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onDeactivateConnection, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onDestroy, void(const HTTPSessionBase&));
-  MOCK_METHOD2(onIngressMessage,
-               void(const HTTPSessionBase&, const HTTPMessage&));
-  MOCK_METHOD1(onIngressLimitExceeded, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onIngressPaused, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onTransactionDetached, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onPingReplySent, void(int64_t));
-  MOCK_METHOD0(onPingReplyReceived, void());
-  MOCK_METHOD1(onSettingsOutgoingStreamsFull, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onSettingsOutgoingStreamsNotFull, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onFlowControlWindowClosed, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onEgressBuffered, void(const HTTPSessionBase&));
-  MOCK_METHOD1(onEgressBufferCleared, void(const HTTPSessionBase&));
-  MOCK_METHOD2(onSettings, void(const HTTPSessionBase&, const SettingsList&));
-  MOCK_METHOD1(onSettingsAck, void(const HTTPSessionBase&));
+  MOCK_METHOD(void, onCreate, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onTransportReady, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onConnectionError, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onIngressError, (const HTTPSessionBase&, ProxygenError));
+  MOCK_METHOD(void, onIngressEOF, ());
+  MOCK_METHOD(void, onRead, (const HTTPSessionBase&, size_t));
+  MOCK_METHOD(void,
+              onRead,
+              (const HTTPSessionBase&,
+               size_t,
+               folly::Optional<HTTPCodec::StreamID>));
+  MOCK_METHOD(void, onWrite, (const HTTPSessionBase&, size_t));
+  MOCK_METHOD(void, onRequestBegin, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onRequestEnd, (const HTTPSessionBase&, uint32_t));
+  MOCK_METHOD(void, onActivateConnection, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onDeactivateConnection, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onDestroy, (const HTTPSessionBase&));
+  MOCK_METHOD(void,
+              onIngressMessage,
+              (const HTTPSessionBase&, const HTTPMessage&));
+  MOCK_METHOD(void, onIngressLimitExceeded, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onIngressPaused, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onTransactionDetached, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onPingReplySent, (int64_t));
+  MOCK_METHOD(void, onPingReplyReceived, ());
+  MOCK_METHOD(void, onSettingsOutgoingStreamsFull, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onSettingsOutgoingStreamsNotFull, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onFlowControlWindowClosed, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onEgressBuffered, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onEgressBufferCleared, (const HTTPSessionBase&));
+  MOCK_METHOD(void, onSettings, (const HTTPSessionBase&, const SettingsList&));
+  MOCK_METHOD(void, onSettingsAck, (const HTTPSessionBase&));
 };
 
-class DummyHTTPSessionStats : public HTTPSessionStats {
+class MockDSRRequestSender : public DSRRequestSender {
  public:
-  void recordTransactionOpened() noexcept override{};
-  void recordTransactionClosed() noexcept override{};
-  void recordTransactionsServed(uint64_t) noexcept override{};
-  void recordSessionReused() noexcept override{};
-  // virtual void recordSessionIdleTime(std::chrono::seconds) noexcept {};
-  void recordTransactionStalled() noexcept override{};
-  void recordSessionStalled() noexcept override{};
-
-  void recordPresendIOSplit() noexcept override{};
-  void recordPresendExceedLimit() noexcept override{};
-  void recordTTLBAExceedLimit() noexcept override{};
-  void recordTTLBANotFound() noexcept override{};
-  void recordTTLBAReceived() noexcept override{};
-  void recordTTLBATimeout() noexcept override{};
-  void recordTTLBATracked() noexcept override{};
-  void recordTTBTXExceedLimit() noexcept override{};
-  void recordTTBTXReceived() noexcept override{};
-  void recordTTBTXTimeout() noexcept override{};
-  void recordTTBTXNotFound() noexcept override{};
-  void recordTTBTXTracked() noexcept override{};
-};
-
-class MockHTTPSessionStats : public DummyHTTPSessionStats {
- public:
-  MockHTTPSessionStats() {
-  }
-  GMOCK_NOEXCEPT_METHOD0(recordTransactionOpened, void());
-  GMOCK_NOEXCEPT_METHOD0(recordTransactionClosed, void());
-  GMOCK_NOEXCEPT_METHOD1(recordTransactionsServed, void(uint64_t));
-  GMOCK_NOEXCEPT_METHOD0(recordSessionReused, void());
-  GMOCK_NOEXCEPT_METHOD1(recordSessionIdleTime, void(std::chrono::seconds));
-  GMOCK_NOEXCEPT_METHOD0(recordTransactionStalled, void());
-  GMOCK_NOEXCEPT_METHOD0(recordSessionStalled, void());
+  MockDSRRequestSender() = default;
+  MOCK_METHOD1(onHeaderBytesGenerated, void(size_t));
 };
 
 } // namespace proxygen
