@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -7,8 +7,10 @@
  */
 
 #include <proxygen/lib/http/HTTPMessage.h>
+
 #include <fcntl.h>
 #include <folly/portability/GTest.h>
+#include <proxygen/lib/http/HTTPPriorityFunctions.h>
 #include <proxygen/lib/utils/TestUtils.h>
 #include <signal.h>
 #include <string.h>
@@ -138,6 +140,25 @@ TEST(HTTPMessage, SetInvalidURL) {
   EXPECT_EQ(msg.getPathAsStringPiece(), "");
 }
 
+TEST(HTTPMessage, SetURLEmpty) {
+  HTTPMessage msg;
+
+  auto res = msg.setURL("");
+  EXPECT_FALSE(res.valid());
+  EXPECT_EQ(msg.getPathAsStringPiece(), "");
+}
+
+TEST(HTTPMessage, SetAbsoluteURLNoPath) {
+  HTTPMessage msg;
+
+  auto res = msg.setURL("http://www.foo.com");
+  EXPECT_TRUE(res.valid());
+  EXPECT_EQ(msg.getPathAsStringPiece(), "/");
+  EXPECT_EQ(msg.getPath(), msg.getPathAsStringPiece());
+  // getPathAsStringPiece points to constant string and getPath points to copy
+  EXPECT_NE(msg.getPath().data(), msg.getPathAsStringPiece().begin());
+}
+
 TEST(HTTPMessage, TestHeaderPreservation) {
   HTTPMessage msg;
   HTTPHeaders& hdrs = msg.getHeaders();
@@ -430,7 +451,7 @@ void testPathAndQuery(const string& url,
 TEST(GetPathAndQuery, ParseURL) {
   testPathAndQuery("http://localhost:80/foo?bar#qqq", "/foo", "bar");
   testPathAndQuery("localhost:80/foo?bar#qqq", "/foo", "bar");
-  testPathAndQuery("localhost", "", "");
+  testPathAndQuery("localhost", "/", "");
   testPathAndQuery("/f/o/o?bar#qqq", "/f/o/o", "bar");
   testPathAndQuery("#?hello", "", "");
 }
@@ -703,6 +724,43 @@ TEST(HTTPMessage, PrettyPrint) {
   checkPrettyPrint(response.getHeaders());
 }
 
+TEST(HTTPMessage, NoDefaultHTTPPriority) {
+  HTTPMessage message;
+  EXPECT_FALSE(httpPriorityFromHTTPMessage(message).hasValue());
+}
+
+TEST(HTTPMessage, HTTPPrioritySetGet) {
+  HTTPMessage message;
+  message.setHTTPPriority(HTTPPriority(0, true));
+  EXPECT_EQ(0, httpPriorityFromHTTPMessage(message)->urgency);
+  EXPECT_TRUE(httpPriorityFromHTTPMessage(message)->incremental);
+  auto& priHeader = message.getHeaders().getSingleOrEmpty(HTTP_HEADER_PRIORITY);
+  EXPECT_EQ("u=0,i", priHeader);
+  auto priHeaderViaGetter = message.getHTTPPriority();
+  EXPECT_EQ(0, priHeaderViaGetter->urgency);
+  EXPECT_TRUE(priHeaderViaGetter->incremental);
+
+  message.setHTTPPriority(1, false);
+  EXPECT_EQ(1, httpPriorityFromHTTPMessage(message)->urgency);
+  EXPECT_FALSE(httpPriorityFromHTTPMessage(message)->incremental);
+  EXPECT_EQ("u=1", message.getHeaders().getSingleOrEmpty(HTTP_HEADER_PRIORITY));
+  priHeaderViaGetter = message.getHTTPPriority();
+  EXPECT_EQ(1, priHeaderViaGetter->urgency);
+  EXPECT_FALSE(priHeaderViaGetter->incremental);
+}
+
+TEST(HTTPMessage, HTTPPrioritySetOutRangeUrgency) {
+  HTTPMessage message;
+  message.setHTTPPriority(HTTPPriority(kMaxPriority + 10, true));
+  EXPECT_EQ(kMaxPriority, httpPriorityFromHTTPMessage(message)->urgency);
+  EXPECT_TRUE(httpPriorityFromHTTPMessage(message)->incremental);
+  auto& priHeader = message.getHeaders().getSingleOrEmpty(HTTP_HEADER_PRIORITY);
+  EXPECT_EQ("u=7,i", priHeader);
+  auto priHeaderViaGetter = message.getHTTPPriority();
+  EXPECT_EQ(7, priHeaderViaGetter->urgency);
+  EXPECT_TRUE(priHeaderViaGetter->incremental);
+}
+
 TEST(HTTPHeaders, GetSetOnResize) {
   HTTPHeaders headers;
   for (size_t i = 0; i < kInitialVectorReserve - 1; i++) {
@@ -732,3 +790,81 @@ TEST(HTTPHeaders, MoveFromTest) {
   EXPECT_EQ(h1.size(), 0);
   EXPECT_EQ(h3.size(), 1);
 }
+
+TEST(HTTPMessage, DefaultSchemeHttp) {
+  HTTPMessage message;
+  EXPECT_EQ(message.getScheme(), "http");
+  EXPECT_FALSE(message.isSecure());
+}
+
+TEST(HTTPMessage, SchemeHttps) {
+  HTTPMessage message;
+  message.setSecure(true);
+  EXPECT_EQ(message.getScheme(), "https");
+  EXPECT_TRUE(message.isSecure());
+  message.setSecure(false);
+  EXPECT_EQ(message.getScheme(), "http");
+  EXPECT_FALSE(message.isSecure());
+}
+
+TEST(HTTPMessage, SchemeMasque) {
+  HTTPMessage message;
+  message.setMasque();
+  EXPECT_EQ(message.getScheme(), "masque");
+  EXPECT_TRUE(message.isSecure());
+  // Masque is already secure, setting secure again has no effect
+  message.setSecure(true);
+  EXPECT_EQ(message.getScheme(), "masque");
+  EXPECT_TRUE(message.isSecure());
+  // Masque must be secure, so unsetting secure falls back to http://
+  message.setSecure(false);
+  EXPECT_EQ(message.getScheme(), "http");
+  EXPECT_FALSE(message.isSecure());
+}
+
+TEST(HTTPMessage, StrictMode) {
+  HTTPMessage message;
+  EXPECT_FALSE(message.setURL("/foo\xff", /*strict=*/true));
+  EXPECT_EQ(message.getURL(), "/foo\xff");
+  EXPECT_EQ(message.getPath(), "");
+
+  message.setURL("/");
+  // Not strict mode, high ascii OK
+  EXPECT_TRUE(message.setQueryString("a=b\xff"));
+  EXPECT_EQ(message.getURL(), "/?a=b\xff");
+  EXPECT_EQ(message.getPath(), "/");
+  EXPECT_EQ(message.getQueryString(), "a=b\xff");
+
+  EXPECT_TRUE(message.setQueryString("a=b"));
+  EXPECT_EQ(message.getURL(), "/?a=b");
+  EXPECT_FALSE(message.setQueryString("a=b\xff", /*strict=*/true));
+  EXPECT_EQ(message.getURL(), "/?a=b\xff");
+  EXPECT_EQ(message.getQueryString(), "");
+
+  EXPECT_TRUE(message.setQueryString("a=b"));
+  EXPECT_FALSE(message.setQueryParam("c", "d\xff", /*strict=*/true));
+  EXPECT_EQ(message.getURL(), "/?a=b&c=d\xff");
+  EXPECT_EQ(message.getQueryString(), "");
+  EXPECT_TRUE(message.setQueryParam("c", "d\xff", /*strict=*/false));
+  EXPECT_EQ(message.getURL(), "/?a=b&c=d\xff");
+
+  // Can remove a param without failing the strict parsing
+  EXPECT_TRUE(message.setURL("/?a=b&c=d\xff"));
+  EXPECT_TRUE(message.removeQueryParam("a"));
+  EXPECT_TRUE(message.removeQueryParam("c"));
+}
+
+#ifdef NDEBUG
+// This fails DCHECKs in debug mode, throws in opt
+TEST(HTTPMessage, BadAPIUsage) {
+  HTTPMessage req;
+  req.setURL("/");
+  EXPECT_THROW(req.getStatusCode(), std::runtime_error);
+  EXPECT_EQ(req.getURL(), "/");
+
+  HTTPMessage resp;
+  resp.setStatusCode(200);
+  EXPECT_THROW(resp.getQueryString(), std::runtime_error);
+  EXPECT_EQ(resp.getStatusCode(), 200);
+}
+#endif
