@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,7 +8,7 @@
 
 #pragma once
 
-#include "fb303/ThreadCachedServiceData.h"
+#include <fb303/ThreadCachedServiceData.h>
 
 namespace proxygen {
 
@@ -28,12 +28,24 @@ class BaseStats {
   // thread safe via the use of atomics, we may only want single local
   // instance instead of wrapped (per thread) instances.
   using TLCounter = facebook::fb303::CounterWrapper;
-  using TLTimeseries = facebook::fb303::TimeseriesPolymorphicWrapper;
+  // Please avoid adding DynamicTimeseriesWrapper if we can.
+  // At a minimum they require formatters and map lookups for
+  // operations and make it easier to violate the constraint that all counters
+  // are created at startup.
   using TLTimeseriesQuarterMinuteOnly =
       facebook::fb303::QuarterMinuteOnlyTimeseriesWrapper;
   using TLTimeseriesMinute = facebook::fb303::MinuteOnlyTimeseriesWrapper;
+  // TLTimeseries was exporting as TimeseriesPolymorphicWrapper
+  // were are trying to get rid of .600/.3600 counters
+  // therefore aliasing it to TLTTimeSeriesMinute which only
+  // exports .60 counters
+  using TLTimeseries = TLTimeseriesMinute;
   using TLTimeseriesMinuteAndAllTime = facebook::fb303::MinuteTimeseriesWrapper;
   using TLHistogram = facebook::fb303::HistogramWrapper;
+  // Please avoid adding DynamicHistogramWrapper if we can.
+  // At a minimum they require formatters and map lookups for
+  // operations and make it easier to violate the constraint that all counters
+  // are created at startup.
 };
 
 } // namespace proxygen
