@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -29,6 +29,15 @@ void PersistentQuicPskCache::setMaxPskUses(size_t maxUses) {
   maxPskUses_ = maxUses;
 }
 
+folly::Optional<size_t> PersistentQuicPskCache::getPskUses(
+    const std::string& identity) {
+  auto cachedPsk = cache_.get(identity);
+  if (cachedPsk) {
+    return cachedPsk->uses;
+  }
+  return folly::none;
+}
+
 folly::Optional<quic::QuicCachedPsk> PersistentQuicPskCache::getPsk(
     const std::string& identity) {
   auto cachedPsk = cache_.get(identity);
