@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -32,3 +32,12 @@ TEST(UtilTest, findLastOf) {
   EXPECT_EQ(findLastOf(p4, '.'), std::string::npos);
   EXPECT_EQ(findLastOf(p5, '.'), 3);
 }
+
+folly::ByteRange input(const char *str) {
+  return folly::ByteRange(reinterpret_cast<const uint8_t *>(str), strlen(str));
+}
+
+TEST(UtilTest, validateURL) {
+  EXPECT_TRUE(validateURL(input("/foo\xff"), URLValidateMode::STRICT_COMPAT));
+  EXPECT_FALSE(validateURL(input("/foo\xff"), URLValidateMode::STRICT));
+}
